part of 'resource.dart';

class FhirField {
  const FhirField(this.isList, this.type);
  final bool isList;
  final String type;
}

/// Because I don't want to root around in StructureDefinitions just to see
/// what kind of an object something is or if it's a list, I'm using this
final Map<String, Map<String, FhirField>> fhirFieldMap =
    <String, Map<String, FhirField>>{
  'Population': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'ageRange': const FhirField(false, 'Range'),
    'ageCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'gender': const FhirField(false, 'CodeableConcept'),
    'race': const FhirField(false, 'CodeableConcept'),
    'physiologicalCondition': const FhirField(false, 'CodeableConcept'),
  },
  'ProductShelfLife': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Quantity'),
    'specialPrecautionsForStorage': const FhirField(true, 'CodeableConcept'),
  },
  'ProdCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'height': const FhirField(false, 'Quantity'),
    'width': const FhirField(false, 'Quantity'),
    'depth': const FhirField(false, 'Quantity'),
    'weight': const FhirField(false, 'Quantity'),
    'nominalVolume': const FhirField(false, 'Quantity'),
    'externalDiameter': const FhirField(false, 'Quantity'),
    'shape': const FhirField(false, 'String'),
    '_shape': const FhirField(false, 'Element'),
    'color': const FhirField(true, 'String'),
    '_color': const FhirField(true, 'Element'),
    'imprint': const FhirField(true, 'String'),
    '_imprint': const FhirField(true, 'Element'),
    'image': const FhirField(true, 'Attachment'),
    'scoring': const FhirField(false, 'CodeableConcept'),
  },
  'MarketingStatus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'country': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'dateRange': const FhirField(false, 'Period'),
    'restoreDate': const FhirField(false, 'FhirDateTime'),
    '_restoreDate': const FhirField(false, 'Element'),
  },
  'Narrative': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'String'),
    '_status': const FhirField(false, 'Element'),
    'div': const FhirField(false, 'String'),
  },
  'CodeableReference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'concept': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'Reference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'String'),
    '_reference': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'FhirUri'),
    '_type': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
  },
  'Meta': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'versionId': const FhirField(false, 'Id'),
    '_versionId': const FhirField(false, 'Element'),
    'lastUpdated': const FhirField(false, 'Instant'),
    '_lastUpdated': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'FhirUri'),
    '_source': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'security': const FhirField(true, 'Coding'),
    'tag': const FhirField(true, 'Coding'),
  },
  'Dosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'Integer'),
    '_sequence': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'additionalInstruction': const FhirField(true, 'CodeableConcept'),
    'patientInstruction': const FhirField(false, 'String'),
    '_patientInstruction': const FhirField(false, 'Element'),
    'timing': const FhirField(false, 'Timing'),
    'asNeededBoolean': const FhirField(false, 'Boolean'),
    '_asNeededBoolean': const FhirField(false, 'Element'),
    'asNeededCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'doseAndRate': const FhirField(true, 'DosageDoseAndRate'),
    'maxDosePerPeriod': const FhirField(false, 'Ratio'),
    'maxDosePerAdministration': const FhirField(false, 'Quantity'),
    'maxDosePerLifetime': const FhirField(false, 'Quantity'),
  },
  'DosageDoseAndRate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'doseRange': const FhirField(false, 'Range'),
    'doseQuantity': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
    'rateRange': const FhirField(false, 'Range'),
    'rateQuantity': const FhirField(false, 'Quantity'),
  },
  'ElementDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'representation': const FhirField(true, 'String'),
    '_representation': const FhirField(true, 'Element'),
    'sliceName': const FhirField(false, 'String'),
    '_sliceName': const FhirField(false, 'Element'),
    'sliceIsConstraining': const FhirField(false, 'Boolean'),
    '_sliceIsConstraining': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    '_label': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'Coding'),
    'slicing': const FhirField(false, 'ElementDefinitionSlicing'),
    'short': const FhirField(false, 'String'),
    '_short': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Markdown'),
    '_definition': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    '_comment': const FhirField(false, 'Element'),
    'requirements': const FhirField(false, 'Markdown'),
    '_requirements': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    '_alias': const FhirField(true, 'Element'),
    'min': const FhirField(false, 'UnsignedInt'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'ElementDefinitionBase'),
    'contentReference': const FhirField(false, 'FhirUri'),
    '_contentReference': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'ElementDefinitionType'),
    'defaultValueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_defaultValueBase64Binary': const FhirField(false, 'Element'),
    'defaultValueBoolean': const FhirField(false, 'Boolean'),
    '_defaultValueBoolean': const FhirField(false, 'Element'),
    'defaultValueCanonical': const FhirField(false, 'Canonical'),
    '_defaultValueCanonical': const FhirField(false, 'Element'),
    'defaultValueCode': const FhirField(false, 'Code'),
    '_defaultValueCode': const FhirField(false, 'Element'),
    'defaultValueDate': const FhirField(false, 'Date'),
    '_defaultValueDate': const FhirField(false, 'Element'),
    'defaultValueDateTime': const FhirField(false, 'FhirDateTime'),
    '_defaultValueDateTime': const FhirField(false, 'Element'),
    'defaultValueDecimal': const FhirField(false, 'Decimal'),
    '_defaultValueDecimal': const FhirField(false, 'Element'),
    'defaultValueId': const FhirField(false, 'Id'),
    '_defaultValueId': const FhirField(false, 'Element'),
    'defaultValueInstant': const FhirField(false, 'Instant'),
    '_defaultValueInstant': const FhirField(false, 'Element'),
    'defaultValueInteger': const FhirField(false, 'Integer'),
    '_defaultValueInteger': const FhirField(false, 'Element'),
    'defaultValueMarkdown': const FhirField(false, 'Markdown'),
    '_defaultValueMarkdown': const FhirField(false, 'Element'),
    'defaultValueOid': const FhirField(false, 'Oid'),
    '_defaultValueOid': const FhirField(false, 'Element'),
    'defaultValuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_defaultValuePositiveInt': const FhirField(false, 'Element'),
    'defaultValueString': const FhirField(false, 'String'),
    '_defaultValueString': const FhirField(false, 'Element'),
    'defaultValueTime': const FhirField(false, 'Time'),
    '_defaultValueTime': const FhirField(false, 'Element'),
    'defaultValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_defaultValueUnsignedInt': const FhirField(false, 'Element'),
    'defaultValueUri': const FhirField(false, 'FhirUri'),
    '_defaultValueUri': const FhirField(false, 'Element'),
    'defaultValueUrl': const FhirField(false, 'FhirUrl'),
    '_defaultValueUrl': const FhirField(false, 'Element'),
    'defaultValueUuid': const FhirField(false, 'Uuid'),
    '_defaultValueUuid': const FhirField(false, 'Element'),
    'defaultValueAddress': const FhirField(false, 'Address'),
    'defaultValueAge': const FhirField(false, 'Age'),
    'defaultValueAnnotation': const FhirField(false, 'Annotation'),
    'defaultValueAttachment': const FhirField(false, 'Attachment'),
    'defaultValueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'defaultValueCodeableReference':
        const FhirField(false, 'CodeableReference'),
    'defaultValueCoding': const FhirField(false, 'Coding'),
    'defaultValueContactPoint': const FhirField(false, 'ContactPoint'),
    'defaultValueCount': const FhirField(false, 'Count'),
    'defaultValueDistance': const FhirField(false, 'Distance'),
    'defaultValueDuration': const FhirField(false, 'FhirDuration'),
    'defaultValueHumanName': const FhirField(false, 'HumanName'),
    'defaultValueIdentifier': const FhirField(false, 'Identifier'),
    'defaultValueMoney': const FhirField(false, 'Money'),
    'defaultValuePeriod': const FhirField(false, 'Period'),
    'defaultValueQuantity': const FhirField(false, 'Quantity'),
    'defaultValueRange': const FhirField(false, 'Range'),
    'defaultValueRatio': const FhirField(false, 'Ratio'),
    'defaultValueRatioRange': const FhirField(false, 'RatioRange'),
    'defaultValueReference': const FhirField(false, 'Reference'),
    'defaultValueSampledData': const FhirField(false, 'SampledData'),
    'defaultValueSignature': const FhirField(false, 'Signature'),
    'defaultValueTiming': const FhirField(false, 'Timing'),
    'defaultValueContactDetail': const FhirField(false, 'ContactDetail'),
    'defaultValueContributor': const FhirField(false, 'Contributor'),
    'defaultValueDataRequirement': const FhirField(false, 'DataRequirement'),
    'defaultValueExpression': const FhirField(false, 'Expression'),
    'defaultValueParameterDefinition':
        const FhirField(false, 'ParameterDefinition'),
    'defaultValueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'defaultValueTriggerDefinition':
        const FhirField(false, 'TriggerDefinition'),
    'defaultValueUsageContext': const FhirField(false, 'UsageContext'),
    'defaultValueDosage': const FhirField(false, 'Dosage'),
    'meaningWhenMissing': const FhirField(false, 'Markdown'),
    '_meaningWhenMissing': const FhirField(false, 'Element'),
    'orderMeaning': const FhirField(false, 'String'),
    '_orderMeaning': const FhirField(false, 'Element'),
    'fixedBase64Binary': const FhirField(false, 'Base64Binary'),
    '_fixedBase64Binary': const FhirField(false, 'Element'),
    'fixedBoolean': const FhirField(false, 'Boolean'),
    '_fixedBoolean': const FhirField(false, 'Element'),
    'fixedCanonical': const FhirField(false, 'Canonical'),
    '_fixedCanonical': const FhirField(false, 'Element'),
    'fixedCode': const FhirField(false, 'Code'),
    '_fixedCode': const FhirField(false, 'Element'),
    'fixedDate': const FhirField(false, 'Date'),
    '_fixedDate': const FhirField(false, 'Element'),
    'fixedDateTime': const FhirField(false, 'FhirDateTime'),
    '_fixedDateTime': const FhirField(false, 'Element'),
    'fixedDecimal': const FhirField(false, 'Decimal'),
    '_fixedDecimal': const FhirField(false, 'Element'),
    'fixedId': const FhirField(false, 'Id'),
    '_fixedId': const FhirField(false, 'Element'),
    'fixedInstant': const FhirField(false, 'Instant'),
    '_fixedInstant': const FhirField(false, 'Element'),
    'fixedInteger': const FhirField(false, 'Integer'),
    '_fixedInteger': const FhirField(false, 'Element'),
    'fixedMarkdown': const FhirField(false, 'Markdown'),
    '_fixedMarkdown': const FhirField(false, 'Element'),
    'fixedOid': const FhirField(false, 'Oid'),
    '_fixedOid': const FhirField(false, 'Element'),
    'fixedPositiveInt': const FhirField(false, 'PositiveInt'),
    '_fixedPositiveInt': const FhirField(false, 'Element'),
    'fixedString': const FhirField(false, 'String'),
    '_fixedString': const FhirField(false, 'Element'),
    'fixedTime': const FhirField(false, 'Time'),
    '_fixedTime': const FhirField(false, 'Element'),
    'fixedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_fixedUnsignedInt': const FhirField(false, 'Element'),
    'fixedUri': const FhirField(false, 'FhirUri'),
    '_fixedUri': const FhirField(false, 'Element'),
    'fixedUrl': const FhirField(false, 'FhirUrl'),
    '_fixedUrl': const FhirField(false, 'Element'),
    'fixedUuid': const FhirField(false, 'Uuid'),
    '_fixedUuid': const FhirField(false, 'Element'),
    'fixedAddress': const FhirField(false, 'Address'),
    'fixedAge': const FhirField(false, 'Age'),
    'fixedAnnotation': const FhirField(false, 'Annotation'),
    'fixedAttachment': const FhirField(false, 'Attachment'),
    'fixedCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'fixedCodeableReference': const FhirField(false, 'CodeableReference'),
    'fixedCoding': const FhirField(false, 'Coding'),
    'fixedContactPoint': const FhirField(false, 'ContactPoint'),
    'fixedCount': const FhirField(false, 'Count'),
    'fixedDistance': const FhirField(false, 'Distance'),
    'fixedDuration': const FhirField(false, 'FhirDuration'),
    'fixedHumanName': const FhirField(false, 'HumanName'),
    'fixedIdentifier': const FhirField(false, 'Identifier'),
    'fixedMoney': const FhirField(false, 'Money'),
    'fixedPeriod': const FhirField(false, 'Period'),
    'fixedQuantity': const FhirField(false, 'Quantity'),
    'fixedRange': const FhirField(false, 'Range'),
    'fixedRatio': const FhirField(false, 'Ratio'),
    'fixedRatioRange': const FhirField(false, 'RatioRange'),
    'fixedReference': const FhirField(false, 'Reference'),
    'fixedSampledData': const FhirField(false, 'SampledData'),
    'fixedSignature': const FhirField(false, 'Signature'),
    'fixedTiming': const FhirField(false, 'Timing'),
    'fixedContactDetail': const FhirField(false, 'ContactDetail'),
    'fixedContributor': const FhirField(false, 'Contributor'),
    'fixedDataRequirement': const FhirField(false, 'DataRequirement'),
    'fixedExpression': const FhirField(false, 'Expression'),
    'fixedParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'fixedRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'fixedTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'fixedUsageContext': const FhirField(false, 'UsageContext'),
    'fixedDosage': const FhirField(false, 'Dosage'),
    'patternBase64Binary': const FhirField(false, 'Base64Binary'),
    '_patternBase64Binary': const FhirField(false, 'Element'),
    'patternBoolean': const FhirField(false, 'Boolean'),
    '_patternBoolean': const FhirField(false, 'Element'),
    'patternCanonical': const FhirField(false, 'Canonical'),
    '_patternCanonical': const FhirField(false, 'Element'),
    'patternCode': const FhirField(false, 'Code'),
    '_patternCode': const FhirField(false, 'Element'),
    'patternDate': const FhirField(false, 'Date'),
    '_patternDate': const FhirField(false, 'Element'),
    'patternDateTime': const FhirField(false, 'FhirDateTime'),
    '_patternDateTime': const FhirField(false, 'Element'),
    'patternDecimal': const FhirField(false, 'Decimal'),
    '_patternDecimal': const FhirField(false, 'Element'),
    'patternId': const FhirField(false, 'Id'),
    '_patternId': const FhirField(false, 'Element'),
    'patternInstant': const FhirField(false, 'Instant'),
    '_patternInstant': const FhirField(false, 'Element'),
    'patternInteger': const FhirField(false, 'Integer'),
    '_patternInteger': const FhirField(false, 'Element'),
    'patternMarkdown': const FhirField(false, 'Markdown'),
    '_patternMarkdown': const FhirField(false, 'Element'),
    'patternOid': const FhirField(false, 'Oid'),
    '_patternOid': const FhirField(false, 'Element'),
    'patternPositiveInt': const FhirField(false, 'PositiveInt'),
    '_patternPositiveInt': const FhirField(false, 'Element'),
    'patternString': const FhirField(false, 'String'),
    '_patternString': const FhirField(false, 'Element'),
    'patternTime': const FhirField(false, 'Time'),
    '_patternTime': const FhirField(false, 'Element'),
    'patternUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_patternUnsignedInt': const FhirField(false, 'Element'),
    'patternUri': const FhirField(false, 'FhirUri'),
    '_patternUri': const FhirField(false, 'Element'),
    'patternUrl': const FhirField(false, 'FhirUrl'),
    '_patternUrl': const FhirField(false, 'Element'),
    'patternUuid': const FhirField(false, 'Uuid'),
    '_patternUuid': const FhirField(false, 'Element'),
    'patternAddress': const FhirField(false, 'Address'),
    'patternAge': const FhirField(false, 'Age'),
    'patternAnnotation': const FhirField(false, 'Annotation'),
    'patternAttachment': const FhirField(false, 'Attachment'),
    'patternCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'patternCodeableReference': const FhirField(false, 'CodeableReference'),
    'patternCoding': const FhirField(false, 'Coding'),
    'patternContactPoint': const FhirField(false, 'ContactPoint'),
    'patternCount': const FhirField(false, 'Count'),
    'patternDistance': const FhirField(false, 'Distance'),
    'patternDuration': const FhirField(false, 'FhirDuration'),
    'patternHumanName': const FhirField(false, 'HumanName'),
    'patternIdentifier': const FhirField(false, 'Identifier'),
    'patternMoney': const FhirField(false, 'Money'),
    'patternPeriod': const FhirField(false, 'Period'),
    'patternQuantity': const FhirField(false, 'Quantity'),
    'patternRange': const FhirField(false, 'Range'),
    'patternRatio': const FhirField(false, 'Ratio'),
    'patternRatioRange': const FhirField(false, 'RatioRange'),
    'patternReference': const FhirField(false, 'Reference'),
    'patternSampledData': const FhirField(false, 'SampledData'),
    'patternSignature': const FhirField(false, 'Signature'),
    'patternTiming': const FhirField(false, 'Timing'),
    'patternContactDetail': const FhirField(false, 'ContactDetail'),
    'patternContributor': const FhirField(false, 'Contributor'),
    'patternDataRequirement': const FhirField(false, 'DataRequirement'),
    'patternExpression': const FhirField(false, 'Expression'),
    'patternParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'patternRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'patternTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'patternUsageContext': const FhirField(false, 'UsageContext'),
    'patternDosage': const FhirField(false, 'Dosage'),
    'example': const FhirField(true, 'ElementDefinitionExample'),
    'minValueDate': const FhirField(false, 'Date'),
    '_minValueDate': const FhirField(false, 'Element'),
    'minValueDateTime': const FhirField(false, 'FhirDateTime'),
    '_minValueDateTime': const FhirField(false, 'Element'),
    'minValueInstant': const FhirField(false, 'Instant'),
    '_minValueInstant': const FhirField(false, 'Element'),
    'minValueTime': const FhirField(false, 'Time'),
    '_minValueTime': const FhirField(false, 'Element'),
    'minValueDecimal': const FhirField(false, 'Decimal'),
    '_minValueDecimal': const FhirField(false, 'Element'),
    'minValueInteger': const FhirField(false, 'Integer'),
    '_minValueInteger': const FhirField(false, 'Element'),
    'minValuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_minValuePositiveInt': const FhirField(false, 'Element'),
    'minValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_minValueUnsignedInt': const FhirField(false, 'Element'),
    'minValueQuantity': const FhirField(false, 'Quantity'),
    'maxValueDate': const FhirField(false, 'Date'),
    '_maxValueDate': const FhirField(false, 'Element'),
    'maxValueDateTime': const FhirField(false, 'FhirDateTime'),
    '_maxValueDateTime': const FhirField(false, 'Element'),
    'maxValueInstant': const FhirField(false, 'Instant'),
    '_maxValueInstant': const FhirField(false, 'Element'),
    'maxValueTime': const FhirField(false, 'Time'),
    '_maxValueTime': const FhirField(false, 'Element'),
    'maxValueDecimal': const FhirField(false, 'Decimal'),
    '_maxValueDecimal': const FhirField(false, 'Element'),
    'maxValueInteger': const FhirField(false, 'Integer'),
    '_maxValueInteger': const FhirField(false, 'Element'),
    'maxValuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_maxValuePositiveInt': const FhirField(false, 'Element'),
    'maxValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_maxValueUnsignedInt': const FhirField(false, 'Element'),
    'maxValueQuantity': const FhirField(false, 'Quantity'),
    'maxLength': const FhirField(false, 'Integer'),
    '_maxLength': const FhirField(false, 'Element'),
    'condition': const FhirField(true, 'Id'),
    '_condition': const FhirField(true, 'Element'),
    'constraint': const FhirField(true, 'ElementDefinitionConstraint'),
    'mustSupport': const FhirField(false, 'Boolean'),
    '_mustSupport': const FhirField(false, 'Element'),
    'isModifier': const FhirField(false, 'Boolean'),
    '_isModifier': const FhirField(false, 'Element'),
    'isModifierReason': const FhirField(false, 'String'),
    '_isModifierReason': const FhirField(false, 'Element'),
    'isSummary': const FhirField(false, 'Boolean'),
    '_isSummary': const FhirField(false, 'Element'),
    'binding': const FhirField(false, 'ElementDefinitionBinding'),
    'mapping': const FhirField(true, 'ElementDefinitionMapping'),
  },
  'ElementDefinitionSlicing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'discriminator': const FhirField(true, 'ElementDefinitionDiscriminator'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'ordered': const FhirField(false, 'Boolean'),
    '_ordered': const FhirField(false, 'Element'),
    'rules': const FhirField(false, 'String'),
    '_rules': const FhirField(false, 'Element'),
  },
  'ElementDefinitionDiscriminator': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
  },
  'ElementDefinitionBase': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'UnsignedInt'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
  },
  'ElementDefinitionType': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'FhirUri'),
    '_code': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    '_profile': const FhirField(true, 'Element'),
    'targetProfile': const FhirField(true, 'Canonical'),
    'aggregation': const FhirField(true, 'String'),
    '_aggregation': const FhirField(true, 'Element'),
    'versioning': const FhirField(false, 'String'),
    '_versioning': const FhirField(false, 'Element'),
  },
  'ElementDefinitionExample': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'label': const FhirField(false, 'String'),
    '_label': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    '_valueCanonical': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    '_valueInstant': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    '_valueMarkdown': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Oid'),
    '_valueOid': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_valuePositiveInt': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_valueUnsignedInt': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    '_valueUrl': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Uuid'),
    '_valueUuid': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueContributor': const FhirField(false, 'Contributor'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'ElementDefinitionConstraint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'Id'),
    '_key': const FhirField(false, 'Element'),
    'requirements': const FhirField(false, 'String'),
    '_requirements': const FhirField(false, 'Element'),
    'severity': const FhirField(false, 'String'),
    '_severity': const FhirField(false, 'Element'),
    'human': const FhirField(false, 'String'),
    '_human': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'xpath': const FhirField(false, 'String'),
    '_xpath': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Canonical'),
  },
  'ElementDefinitionBinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'strength': const FhirField(false, 'String'),
    '_strength': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
  },
  'ElementDefinitionMapping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identity': const FhirField(false, 'Id'),
    '_identity': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'map': const FhirField(false, 'String'),
    '_map': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'BodyStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'morphology': const FhirField(false, 'CodeableConcept'),
    'location': const FhirField(false, 'CodeableConcept'),
    'locationQualifier': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'image': const FhirField(true, 'Attachment'),
    'patient': const FhirField(false, 'Reference'),
  },
  'DiagnosticReport': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveDateTime': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'issued': const FhirField(false, 'Instant'),
    '_issued': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'Reference'),
    'resultsInterpreter': const FhirField(true, 'Reference'),
    'specimen': const FhirField(true, 'Reference'),
    'result': const FhirField(true, 'Reference'),
    'imagingStudy': const FhirField(true, 'Reference'),
    'media': const FhirField(true, 'DiagnosticReportMedia'),
    'conclusion': const FhirField(false, 'String'),
    '_conclusion': const FhirField(false, 'Element'),
    'conclusionCode': const FhirField(true, 'CodeableConcept'),
    'presentedForm': const FhirField(true, 'Attachment'),
  },
  'DiagnosticReportMedia': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
    'link': const FhirField(false, 'Reference'),
  },
  'ImagingStudy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'modality': const FhirField(true, 'Coding'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'started': const FhirField(false, 'FhirDateTime'),
    '_started': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'referrer': const FhirField(false, 'Reference'),
    'interpreter': const FhirField(true, 'Reference'),
    'endpoint': const FhirField(true, 'Reference'),
    'numberOfSeries': const FhirField(false, 'UnsignedInt'),
    '_numberOfSeries': const FhirField(false, 'Element'),
    'numberOfInstances': const FhirField(false, 'UnsignedInt'),
    '_numberOfInstances': const FhirField(false, 'Element'),
    'procedureReference': const FhirField(false, 'Reference'),
    'procedureCode': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'series': const FhirField(true, 'ImagingStudySeries'),
  },
  'ImagingStudySeries': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uid': const FhirField(false, 'Id'),
    '_uid': const FhirField(false, 'Element'),
    'number': const FhirField(false, 'UnsignedInt'),
    '_number': const FhirField(false, 'Element'),
    'modality': const FhirField(false, 'Coding'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'numberOfInstances': const FhirField(false, 'UnsignedInt'),
    '_numberOfInstances': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(false, 'Coding'),
    'laterality': const FhirField(false, 'Coding'),
    'specimen': const FhirField(true, 'Reference'),
    'started': const FhirField(false, 'FhirDateTime'),
    '_started': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'ImagingStudyPerformer'),
    'instance': const FhirField(true, 'ImagingStudyInstance'),
  },
  'ImagingStudyPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ImagingStudyInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uid': const FhirField(false, 'Id'),
    '_uid': const FhirField(false, 'Element'),
    'sopClass': const FhirField(false, 'Coding'),
    'number': const FhirField(false, 'UnsignedInt'),
    '_number': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
  },
  'Media': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'modality': const FhirField(false, 'CodeableConcept'),
    'view': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'createdDateTime': const FhirField(false, 'FhirDateTime'),
    '_createdDateTime': const FhirField(false, 'Element'),
    'createdPeriod': const FhirField(false, 'Period'),
    'issued': const FhirField(false, 'Instant'),
    '_issued': const FhirField(false, 'Element'),
    'operator': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'deviceName': const FhirField(false, 'String'),
    '_deviceName': const FhirField(false, 'Element'),
    'device': const FhirField(false, 'Reference'),
    'height': const FhirField(false, 'PositiveInt'),
    '_height': const FhirField(false, 'Element'),
    'width': const FhirField(false, 'PositiveInt'),
    '_width': const FhirField(false, 'Element'),
    'frames': const FhirField(false, 'PositiveInt'),
    '_frames': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Decimal'),
    '_duration': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Attachment'),
    'note': const FhirField(true, 'Annotation'),
  },
  'MolecularSequence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'coordinateSystem': const FhirField(false, 'Integer'),
    '_coordinateSystem': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'specimen': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'referenceSeq': const FhirField(false, 'MolecularSequenceReferenceSeq'),
    'variant': const FhirField(true, 'MolecularSequenceVariant'),
    'observedSeq': const FhirField(false, 'String'),
    '_observedSeq': const FhirField(false, 'Element'),
    'quality': const FhirField(true, 'MolecularSequenceQuality'),
    'readCoverage': const FhirField(false, 'Integer'),
    '_readCoverage': const FhirField(false, 'Element'),
    'repository': const FhirField(true, 'MolecularSequenceRepository'),
    'pointer': const FhirField(true, 'Reference'),
    'structureVariant':
        const FhirField(true, 'MolecularSequenceStructureVariant'),
  },
  'MolecularSequenceReferenceSeq': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'chromosome': const FhirField(false, 'CodeableConcept'),
    'genomeBuild': const FhirField(false, 'String'),
    '_genomeBuild': const FhirField(false, 'Element'),
    'orientation': const FhirField(false, 'Code'),
    '_orientation': const FhirField(false, 'Element'),
    'referenceSeqId': const FhirField(false, 'CodeableConcept'),
    'referenceSeqPointer': const FhirField(false, 'Reference'),
    'referenceSeqString': const FhirField(false, 'String'),
    '_referenceSeqString': const FhirField(false, 'Element'),
    'strand': const FhirField(false, 'Code'),
    '_strand': const FhirField(false, 'Element'),
    'windowStart': const FhirField(false, 'Integer'),
    '_windowStart': const FhirField(false, 'Element'),
    'windowEnd': const FhirField(false, 'Integer'),
    '_windowEnd': const FhirField(false, 'Element'),
  },
  'MolecularSequenceVariant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'Integer'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Integer'),
    '_end': const FhirField(false, 'Element'),
    'observedAllele': const FhirField(false, 'String'),
    '_observedAllele': const FhirField(false, 'Element'),
    'referenceAllele': const FhirField(false, 'String'),
    '_referenceAllele': const FhirField(false, 'Element'),
    'cigar': const FhirField(false, 'String'),
    '_cigar': const FhirField(false, 'Element'),
    'variantPointer': const FhirField(false, 'Reference'),
  },
  'MolecularSequenceQuality': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'standardSequence': const FhirField(false, 'CodeableConcept'),
    'start': const FhirField(false, 'Integer'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Integer'),
    '_end': const FhirField(false, 'Element'),
    'score': const FhirField(false, 'Quantity'),
    'method': const FhirField(false, 'CodeableConcept'),
    'truthTP': const FhirField(false, 'Decimal'),
    '_truthTP': const FhirField(false, 'Element'),
    'queryTP': const FhirField(false, 'Decimal'),
    '_queryTP': const FhirField(false, 'Element'),
    'truthFN': const FhirField(false, 'Decimal'),
    '_truthFN': const FhirField(false, 'Element'),
    'queryFP': const FhirField(false, 'Decimal'),
    '_queryFP': const FhirField(false, 'Element'),
    'gtFP': const FhirField(false, 'Decimal'),
    '_gtFP': const FhirField(false, 'Element'),
    'precision': const FhirField(false, 'Decimal'),
    '_precision': const FhirField(false, 'Element'),
    'recall': const FhirField(false, 'Decimal'),
    '_recall': const FhirField(false, 'Element'),
    'fScore': const FhirField(false, 'Decimal'),
    '_fScore': const FhirField(false, 'Element'),
    'roc': const FhirField(false, 'MolecularSequenceRoc'),
  },
  'MolecularSequenceRoc': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'score': const FhirField(true, 'Integer'),
    '_score': const FhirField(true, 'Element'),
    'numTP': const FhirField(true, 'Integer'),
    '_numTP': const FhirField(true, 'Element'),
    'numFP': const FhirField(true, 'Integer'),
    '_numFP': const FhirField(true, 'Element'),
    'numFN': const FhirField(true, 'Integer'),
    '_numFN': const FhirField(true, 'Element'),
    'precision': const FhirField(true, 'Decimal'),
    '_precision': const FhirField(true, 'Element'),
    'sensitivity': const FhirField(true, 'Decimal'),
    '_sensitivity': const FhirField(true, 'Element'),
    'fMeasure': const FhirField(true, 'Decimal'),
    '_fMeasure': const FhirField(true, 'Element'),
  },
  'MolecularSequenceRepository': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'datasetId': const FhirField(false, 'String'),
    '_datasetId': const FhirField(false, 'Element'),
    'variantsetId': const FhirField(false, 'String'),
    '_variantsetId': const FhirField(false, 'Element'),
    'readsetId': const FhirField(false, 'String'),
    '_readsetId': const FhirField(false, 'Element'),
  },
  'MolecularSequenceStructureVariant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'variantType': const FhirField(false, 'CodeableConcept'),
    'exact': const FhirField(false, 'Boolean'),
    '_exact': const FhirField(false, 'Element'),
    'length': const FhirField(false, 'Integer'),
    '_length': const FhirField(false, 'Element'),
    'outer': const FhirField(false, 'MolecularSequenceOuter'),
    'inner': const FhirField(false, 'MolecularSequenceInner'),
  },
  'MolecularSequenceOuter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'Integer'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Integer'),
    '_end': const FhirField(false, 'Element'),
  },
  'MolecularSequenceInner': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'Integer'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Integer'),
    '_end': const FhirField(false, 'Element'),
  },
  'Observation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'focus': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveDateTime': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'effectiveTiming': const FhirField(false, 'Timing'),
    'effectiveInstant': const FhirField(false, 'Instant'),
    '_effectiveInstant': const FhirField(false, 'Element'),
    'issued': const FhirField(false, 'Instant'),
    '_issued': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'Reference'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'interpretation': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'specimen': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'Reference'),
    'referenceRange': const FhirField(true, 'ObservationReferenceRange'),
    'hasMember': const FhirField(true, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'component': const FhirField(true, 'ObservationComponent'),
  },
  'ObservationReferenceRange': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'low': const FhirField(false, 'Quantity'),
    'high': const FhirField(false, 'Quantity'),
    'type': const FhirField(false, 'CodeableConcept'),
    'appliesTo': const FhirField(true, 'CodeableConcept'),
    'age': const FhirField(false, 'Range'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
  },
  'ObservationComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'interpretation': const FhirField(true, 'CodeableConcept'),
    'referenceRange': const FhirField(true, 'ObservationReferenceRange'),
  },
  'QuestionnaireResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'questionnaire': const FhirField(false, 'Canonical'),
    '_questionnaire': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'authored': const FhirField(false, 'FhirDateTime'),
    '_authored': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'QuestionnaireResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    '_linkId': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'FhirUri'),
    '_definition': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'answer': const FhirField(true, 'QuestionnaireResponseAnswer'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'QuestionnaireResponseAnswer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'Specimen': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'accessionIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'receivedTime': const FhirField(false, 'FhirDateTime'),
    '_receivedTime': const FhirField(false, 'Element'),
    'parent': const FhirField(true, 'Reference'),
    'request': const FhirField(true, 'Reference'),
    'collection': const FhirField(false, 'SpecimenCollection'),
    'processing': const FhirField(true, 'SpecimenProcessing'),
    'container': const FhirField(true, 'SpecimenContainer'),
    'condition': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
  },
  'SpecimenCollection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'collector': const FhirField(false, 'Reference'),
    'collectedDateTime': const FhirField(false, 'FhirDateTime'),
    '_collectedDateTime': const FhirField(false, 'Element'),
    'collectedPeriod': const FhirField(false, 'Period'),
    'duration': const FhirField(false, 'FhirDuration'),
    'quantity': const FhirField(false, 'Quantity'),
    'method': const FhirField(false, 'CodeableConcept'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'fastingStatusCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'fastingStatusDuration': const FhirField(false, 'FhirDuration'),
  },
  'SpecimenProcessing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'procedure': const FhirField(false, 'CodeableConcept'),
    'additive': const FhirField(true, 'Reference'),
    'timeDateTime': const FhirField(false, 'FhirDateTime'),
    '_timeDateTime': const FhirField(false, 'Element'),
    'timePeriod': const FhirField(false, 'Period'),
  },
  'SpecimenContainer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'capacity': const FhirField(false, 'Quantity'),
    'specimenQuantity': const FhirField(false, 'Quantity'),
    'additiveCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'additiveReference': const FhirField(false, 'Reference'),
  },
  'Immunization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'vaccineCode': const FhirField(false, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrenceString': const FhirField(false, 'String'),
    '_occurrenceString': const FhirField(false, 'Element'),
    'recorded': const FhirField(false, 'FhirDateTime'),
    '_recorded': const FhirField(false, 'Element'),
    'primarySource': const FhirField(false, 'Boolean'),
    '_primarySource': const FhirField(false, 'Element'),
    'reportOrigin': const FhirField(false, 'CodeableConcept'),
    'location': const FhirField(false, 'Reference'),
    'manufacturer': const FhirField(false, 'Reference'),
    'lotNumber': const FhirField(false, 'String'),
    '_lotNumber': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'Date'),
    '_expirationDate': const FhirField(false, 'Element'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'doseQuantity': const FhirField(false, 'Quantity'),
    'performer': const FhirField(true, 'ImmunizationPerformer'),
    'note': const FhirField(true, 'Annotation'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'isSubpotent': const FhirField(false, 'Boolean'),
    '_isSubpotent': const FhirField(false, 'Element'),
    'subpotentReason': const FhirField(true, 'CodeableConcept'),
    'education': const FhirField(true, 'ImmunizationEducation'),
    'programEligibility': const FhirField(true, 'CodeableConcept'),
    'fundingSource': const FhirField(false, 'CodeableConcept'),
    'reaction': const FhirField(true, 'ImmunizationReaction'),
    'protocolApplied': const FhirField(true, 'ImmunizationProtocolApplied'),
  },
  'ImmunizationPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ImmunizationEducation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'documentType': const FhirField(false, 'String'),
    '_documentType': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'FhirUri'),
    '_reference': const FhirField(false, 'Element'),
    'publicationDate': const FhirField(false, 'FhirDateTime'),
    '_publicationDate': const FhirField(false, 'Element'),
    'presentationDate': const FhirField(false, 'FhirDateTime'),
    '_presentationDate': const FhirField(false, 'Element'),
  },
  'ImmunizationReaction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'detail': const FhirField(false, 'Reference'),
    'reported': const FhirField(false, 'Boolean'),
    '_reported': const FhirField(false, 'Element'),
  },
  'ImmunizationProtocolApplied': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'series': const FhirField(false, 'String'),
    '_series': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'targetDisease': const FhirField(true, 'CodeableConcept'),
    'doseNumberPositiveInt': const FhirField(false, 'PositiveInt'),
    '_doseNumberPositiveInt': const FhirField(false, 'Element'),
    'doseNumberString': const FhirField(false, 'String'),
    '_doseNumberString': const FhirField(false, 'Element'),
    'seriesDosesPositiveInt': const FhirField(false, 'PositiveInt'),
    '_seriesDosesPositiveInt': const FhirField(false, 'Element'),
    'seriesDosesString': const FhirField(false, 'String'),
    '_seriesDosesString': const FhirField(false, 'Element'),
  },
  'ImmunizationEvaluation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'targetDisease': const FhirField(false, 'CodeableConcept'),
    'immunizationEvent': const FhirField(false, 'Reference'),
    'doseStatus': const FhirField(false, 'CodeableConcept'),
    'doseStatusReason': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'series': const FhirField(false, 'String'),
    '_series': const FhirField(false, 'Element'),
    'doseNumberPositiveInt': const FhirField(false, 'PositiveInt'),
    '_doseNumberPositiveInt': const FhirField(false, 'Element'),
    'doseNumberString': const FhirField(false, 'String'),
    '_doseNumberString': const FhirField(false, 'Element'),
    'seriesDosesPositiveInt': const FhirField(false, 'PositiveInt'),
    '_seriesDosesPositiveInt': const FhirField(false, 'Element'),
    'seriesDosesString': const FhirField(false, 'String'),
    '_seriesDosesString': const FhirField(false, 'Element'),
  },
  'ImmunizationRecommendation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'recommendation':
        const FhirField(true, 'ImmunizationRecommendationRecommendation'),
  },
  'ImmunizationRecommendationRecommendation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'vaccineCode': const FhirField(true, 'CodeableConcept'),
    'targetDisease': const FhirField(false, 'CodeableConcept'),
    'contraindicatedVaccineCode': const FhirField(true, 'CodeableConcept'),
    'forecastStatus': const FhirField(false, 'CodeableConcept'),
    'forecastReason': const FhirField(true, 'CodeableConcept'),
    'dateCriterion':
        const FhirField(true, 'ImmunizationRecommendationDateCriterion'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'series': const FhirField(false, 'String'),
    '_series': const FhirField(false, 'Element'),
    'doseNumberPositiveInt': const FhirField(false, 'PositiveInt'),
    '_doseNumberPositiveInt': const FhirField(false, 'Element'),
    'doseNumberString': const FhirField(false, 'String'),
    '_doseNumberString': const FhirField(false, 'Element'),
    'seriesDosesPositiveInt': const FhirField(false, 'PositiveInt'),
    '_seriesDosesPositiveInt': const FhirField(false, 'Element'),
    'seriesDosesString': const FhirField(false, 'String'),
    '_seriesDosesString': const FhirField(false, 'Element'),
    'supportingImmunization': const FhirField(true, 'Reference'),
    'supportingPatientInformation': const FhirField(true, 'Reference'),
  },
  'ImmunizationRecommendationDateCriterion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'FhirDateTime'),
    '_value': const FhirField(false, 'Element'),
  },
  'Medication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'Reference'),
    'form': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Ratio'),
    'ingredient': const FhirField(true, 'MedicationIngredient'),
    'batch': const FhirField(false, 'MedicationBatch'),
  },
  'MedicationIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
    'isActive': const FhirField(false, 'Boolean'),
    '_isActive': const FhirField(false, 'Element'),
    'strength': const FhirField(false, 'Ratio'),
  },
  'MedicationBatch': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'lotNumber': const FhirField(false, 'String'),
    '_lotNumber': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'FhirDateTime'),
    '_expirationDate': const FhirField(false, 'Element'),
  },
  'MedicationAdministration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiates': const FhirField(true, 'FhirUri'),
    '_instantiates': const FhirField(true, 'Element'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'medicationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'medicationReference': const FhirField(false, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'context': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveDateTime': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'performer': const FhirField(true, 'MedicationAdministrationPerformer'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'device': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'dosage': const FhirField(false, 'MedicationAdministrationDosage'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationAdministrationPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'MedicationAdministrationDosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'dose': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
    'rateQuantity': const FhirField(false, 'Quantity'),
  },
  'MedicationDispense': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReasonCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'statusReasonReference': const FhirField(false, 'Reference'),
    'category': const FhirField(false, 'CodeableConcept'),
    'medicationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'medicationReference': const FhirField(false, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'context': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'performer': const FhirField(true, 'MedicationDispensePerformer'),
    'location': const FhirField(false, 'Reference'),
    'authorizingPrescription': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'daysSupply': const FhirField(false, 'Quantity'),
    'whenPrepared': const FhirField(false, 'FhirDateTime'),
    '_whenPrepared': const FhirField(false, 'Element'),
    'whenHandedOver': const FhirField(false, 'FhirDateTime'),
    '_whenHandedOver': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Reference'),
    'receiver': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'dosageInstruction': const FhirField(true, 'Dosage'),
    'substitution': const FhirField(false, 'MedicationDispenseSubstitution'),
    'detectedIssue': const FhirField(true, 'Reference'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationDispensePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'MedicationDispenseSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'wasSubstituted': const FhirField(false, 'Boolean'),
    '_wasSubstituted': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'responsibleParty': const FhirField(true, 'Reference'),
  },
  'MedicationKnowledge': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'Reference'),
    'doseForm': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Quantity'),
    'synonym': const FhirField(true, 'String'),
    '_synonym': const FhirField(true, 'Element'),
    'relatedMedicationKnowledge':
        const FhirField(true, 'MedicationKnowledgeRelatedMedicationKnowledge'),
    'associatedMedication': const FhirField(true, 'Reference'),
    'productType': const FhirField(true, 'CodeableConcept'),
    'monograph': const FhirField(true, 'MedicationKnowledgeMonograph'),
    'ingredient': const FhirField(true, 'MedicationKnowledgeIngredient'),
    'preparationInstruction': const FhirField(false, 'Markdown'),
    '_preparationInstruction': const FhirField(false, 'Element'),
    'intendedRoute': const FhirField(true, 'CodeableConcept'),
    'cost': const FhirField(true, 'MedicationKnowledgeCost'),
    'monitoringProgram':
        const FhirField(true, 'MedicationKnowledgeMonitoringProgram'),
    'administrationGuidelines':
        const FhirField(true, 'MedicationKnowledgeAdministrationGuidelines'),
    'medicineClassification':
        const FhirField(true, 'MedicationKnowledgeMedicineClassification'),
    'packaging': const FhirField(false, 'MedicationKnowledgePackaging'),
    'drugCharacteristic':
        const FhirField(true, 'MedicationKnowledgeDrugCharacteristic'),
    'contraindication': const FhirField(true, 'Reference'),
    'regulatory': const FhirField(true, 'MedicationKnowledgeRegulatory'),
    'kinetics': const FhirField(true, 'MedicationKnowledgeKinetics'),
  },
  'MedicationKnowledgeRelatedMedicationKnowledge': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(true, 'Reference'),
  },
  'MedicationKnowledgeMonograph': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'Reference'),
  },
  'MedicationKnowledgeIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
    'isActive': const FhirField(false, 'Boolean'),
    '_isActive': const FhirField(false, 'Element'),
    'strength': const FhirField(false, 'Ratio'),
  },
  'MedicationKnowledgeCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'String'),
    '_source': const FhirField(false, 'Element'),
    'cost': const FhirField(false, 'Money'),
  },
  'MedicationKnowledgeMonitoringProgram': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
  },
  'MedicationKnowledgeAdministrationGuidelines': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'dosage': const FhirField(true, 'MedicationKnowledgeDosage'),
    'indicationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'indicationReference': const FhirField(false, 'Reference'),
    'patientCharacteristics':
        const FhirField(true, 'MedicationKnowledgePatientCharacteristics'),
  },
  'MedicationKnowledgeDosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'dosage': const FhirField(true, 'Dosage'),
  },
  'MedicationKnowledgePatientCharacteristics': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'characteristicCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'characteristicQuantity': const FhirField(false, 'Quantity'),
    'value': const FhirField(true, 'String'),
    '_value': const FhirField(true, 'Element'),
  },
  'MedicationKnowledgeMedicineClassification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classification': const FhirField(true, 'CodeableConcept'),
  },
  'MedicationKnowledgePackaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
  },
  'MedicationKnowledgeDrugCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
  },
  'MedicationKnowledgeRegulatory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'regulatoryAuthority': const FhirField(false, 'Reference'),
    'substitution': const FhirField(true, 'MedicationKnowledgeSubstitution'),
    'schedule': const FhirField(true, 'MedicationKnowledgeSchedule'),
    'maxDispense': const FhirField(false, 'MedicationKnowledgeMaxDispense'),
  },
  'MedicationKnowledgeSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowed': const FhirField(false, 'Boolean'),
    '_allowed': const FhirField(false, 'Element'),
  },
  'MedicationKnowledgeSchedule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'schedule': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationKnowledgeMaxDispense': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'period': const FhirField(false, 'FhirDuration'),
  },
  'MedicationKnowledgeKinetics': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'areaUnderCurve': const FhirField(true, 'Quantity'),
    'lethalDose50': const FhirField(true, 'Quantity'),
    'halfLifePeriod': const FhirField(false, 'FhirDuration'),
  },
  'MedicationRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'reportedBoolean': const FhirField(false, 'Boolean'),
    '_reportedBoolean': const FhirField(false, 'Element'),
    'reportedReference': const FhirField(false, 'Reference'),
    'medicationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'medicationReference': const FhirField(false, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'Reference'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'recorder': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    '_instantiatesCanonical': const FhirField(true, 'Element'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'courseOfTherapyType': const FhirField(false, 'CodeableConcept'),
    'insurance': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'dosageInstruction': const FhirField(true, 'Dosage'),
    'dispenseRequest':
        const FhirField(false, 'MedicationRequestDispenseRequest'),
    'substitution': const FhirField(false, 'MedicationRequestSubstitution'),
    'priorPrescription': const FhirField(false, 'Reference'),
    'detectedIssue': const FhirField(true, 'Reference'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationRequestDispenseRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'initialFill': const FhirField(false, 'MedicationRequestInitialFill'),
    'dispenseInterval': const FhirField(false, 'FhirDuration'),
    'validityPeriod': const FhirField(false, 'Period'),
    'numberOfRepeatsAllowed': const FhirField(false, 'UnsignedInt'),
    '_numberOfRepeatsAllowed': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'expectedSupplyDuration': const FhirField(false, 'FhirDuration'),
    'performer': const FhirField(false, 'Reference'),
  },
  'MedicationRequestInitialFill': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'duration': const FhirField(false, 'FhirDuration'),
  },
  'MedicationRequestSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'allowedBoolean': const FhirField(false, 'Boolean'),
    '_allowedBoolean': const FhirField(false, 'Element'),
    'allowedCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationStatement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'medicationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'medicationReference': const FhirField(false, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'context': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveDateTime': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'dateAsserted': const FhirField(false, 'FhirDateTime'),
    '_dateAsserted': const FhirField(false, 'Element'),
    'informationSource': const FhirField(false, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'dosage': const FhirField(true, 'Dosage'),
  },
  'CarePlan': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'contributor': const FhirField(true, 'Reference'),
    'careTeam': const FhirField(true, 'Reference'),
    'addresses': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'goal': const FhirField(true, 'Reference'),
    'activity': const FhirField(true, 'CarePlanActivity'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CarePlanActivity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'outcomeCodeableConcept': const FhirField(true, 'CodeableConcept'),
    'outcomeReference': const FhirField(true, 'Reference'),
    'progress': const FhirField(true, 'Annotation'),
    'reference': const FhirField(false, 'Reference'),
    'detail': const FhirField(false, 'CarePlanDetail'),
  },
  'CarePlanDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'goal': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'scheduledTiming': const FhirField(false, 'Timing'),
    'scheduledPeriod': const FhirField(false, 'Period'),
    'scheduledString': const FhirField(false, 'String'),
    '_scheduledString': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'Reference'),
    'productCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'productReference': const FhirField(false, 'Reference'),
    'dailyAmount': const FhirField(false, 'Quantity'),
    'quantity': const FhirField(false, 'Quantity'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
  },
  'CareTeam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'participant': const FhirField(true, 'CareTeamParticipant'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'managingOrganization': const FhirField(true, 'Reference'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CareTeamParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(true, 'CodeableConcept'),
    'member': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'Goal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'lifecycleStatus': const FhirField(false, 'Code'),
    '_lifecycleStatus': const FhirField(false, 'Element'),
    'achievementStatus': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'startDate': const FhirField(false, 'Date'),
    '_startDate': const FhirField(false, 'Element'),
    'startCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'target': const FhirField(true, 'GoalTarget'),
    'statusDate': const FhirField(false, 'Date'),
    '_statusDate': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'String'),
    '_statusReason': const FhirField(false, 'Element'),
    'expressedBy': const FhirField(false, 'Reference'),
    'addresses': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'outcomeCode': const FhirField(true, 'CodeableConcept'),
    'outcomeReference': const FhirField(true, 'Reference'),
  },
  'GoalTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'measure': const FhirField(false, 'CodeableConcept'),
    'detailQuantity': const FhirField(false, 'Quantity'),
    'detailRange': const FhirField(false, 'Range'),
    'detailCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'detailString': const FhirField(false, 'String'),
    '_detailString': const FhirField(false, 'Element'),
    'detailBoolean': const FhirField(false, 'Boolean'),
    '_detailBoolean': const FhirField(false, 'Element'),
    'detailInteger': const FhirField(false, 'Integer'),
    '_detailInteger': const FhirField(false, 'Element'),
    'detailRatio': const FhirField(false, 'Ratio'),
    'dueDate': const FhirField(false, 'Date'),
    '_dueDate': const FhirField(false, 'Element'),
    'dueDuration': const FhirField(false, 'FhirDuration'),
  },
  'NutritionOrder': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'instantiates': const FhirField(true, 'FhirUri'),
    '_instantiates': const FhirField(true, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'dateTime': const FhirField(false, 'FhirDateTime'),
    '_dateTime': const FhirField(false, 'Element'),
    'orderer': const FhirField(false, 'Reference'),
    'allergyIntolerance': const FhirField(true, 'Reference'),
    'foodPreferenceModifier': const FhirField(true, 'CodeableConcept'),
    'excludeFoodModifier': const FhirField(true, 'CodeableConcept'),
    'oralDiet': const FhirField(false, 'NutritionOrderOralDiet'),
    'supplement': const FhirField(true, 'NutritionOrderSupplement'),
    'enteralFormula': const FhirField(false, 'NutritionOrderEnteralFormula'),
    'note': const FhirField(true, 'Annotation'),
  },
  'NutritionOrderOralDiet': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'schedule': const FhirField(true, 'Timing'),
    'nutrient': const FhirField(true, 'NutritionOrderNutrient'),
    'texture': const FhirField(true, 'NutritionOrderTexture'),
    'fluidConsistencyType': const FhirField(true, 'CodeableConcept'),
    'instruction': const FhirField(false, 'String'),
    '_instruction': const FhirField(false, 'Element'),
  },
  'NutritionOrderNutrient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'modifier': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'NutritionOrderTexture': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'modifier': const FhirField(false, 'CodeableConcept'),
    'foodType': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionOrderSupplement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'productName': const FhirField(false, 'String'),
    '_productName': const FhirField(false, 'Element'),
    'schedule': const FhirField(true, 'Timing'),
    'quantity': const FhirField(false, 'Quantity'),
    'instruction': const FhirField(false, 'String'),
    '_instruction': const FhirField(false, 'Element'),
  },
  'NutritionOrderEnteralFormula': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'baseFormulaType': const FhirField(false, 'CodeableConcept'),
    'baseFormulaProductName': const FhirField(false, 'String'),
    '_baseFormulaProductName': const FhirField(false, 'Element'),
    'additiveType': const FhirField(false, 'CodeableConcept'),
    'additiveProductName': const FhirField(false, 'String'),
    '_additiveProductName': const FhirField(false, 'Element'),
    'caloricDensity': const FhirField(false, 'Quantity'),
    'routeofAdministration': const FhirField(false, 'CodeableConcept'),
    'administration': const FhirField(true, 'NutritionOrderAdministration'),
    'maxVolumeToDeliver': const FhirField(false, 'Quantity'),
    'administrationInstruction': const FhirField(false, 'String'),
    '_administrationInstruction': const FhirField(false, 'Element'),
  },
  'NutritionOrderAdministration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'schedule': const FhirField(false, 'Timing'),
    'quantity': const FhirField(false, 'Quantity'),
    'rateQuantity': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
  },
  'RequestGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    '_instantiatesCanonical': const FhirField(true, 'Element'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'action': const FhirField(true, 'RequestGroupAction'),
  },
  'RequestGroupAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'prefix': const FhirField(false, 'String'),
    '_prefix': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'textEquivalent': const FhirField(false, 'String'),
    '_textEquivalent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'condition': const FhirField(true, 'RequestGroupCondition'),
    'relatedAction': const FhirField(true, 'RequestGroupRelatedAction'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    '_timingDateTime': const FhirField(false, 'Element'),
    'timingAge': const FhirField(false, 'Age'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'timingRange': const FhirField(false, 'Range'),
    'timingTiming': const FhirField(false, 'Timing'),
    'participant': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupingBehavior': const FhirField(false, 'Code'),
    '_groupingBehavior': const FhirField(false, 'Element'),
    'selectionBehavior': const FhirField(false, 'Code'),
    '_selectionBehavior': const FhirField(false, 'Element'),
    'requiredBehavior': const FhirField(false, 'Code'),
    '_requiredBehavior': const FhirField(false, 'Element'),
    'precheckBehavior': const FhirField(false, 'Code'),
    '_precheckBehavior': const FhirField(false, 'Element'),
    'cardinalityBehavior': const FhirField(false, 'Code'),
    '_cardinalityBehavior': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
    'action': const FhirField(true, 'RequestGroupAction'),
  },
  'RequestGroupCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'RequestGroupRelatedAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actionId': const FhirField(false, 'Id'),
    '_actionId': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'Code'),
    '_relationship': const FhirField(false, 'Element'),
    'offsetDuration': const FhirField(false, 'FhirDuration'),
    'offsetRange': const FhirField(false, 'Range'),
  },
  'RiskAssessment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(false, 'Reference'),
    'parent': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'condition': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'basis': const FhirField(true, 'Reference'),
    'prediction': const FhirField(true, 'RiskAssessmentPrediction'),
    'mitigation': const FhirField(false, 'String'),
    '_mitigation': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'RiskAssessmentPrediction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'probabilityDecimal': const FhirField(false, 'Decimal'),
    '_probabilityDecimal': const FhirField(false, 'Element'),
    'probabilityRange': const FhirField(false, 'Range'),
    'qualitativeRisk': const FhirField(false, 'CodeableConcept'),
    'relativeRisk': const FhirField(false, 'Decimal'),
    '_relativeRisk': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
    'whenRange': const FhirField(false, 'Range'),
    'rationale': const FhirField(false, 'String'),
    '_rationale': const FhirField(false, 'Element'),
  },
  'ServiceRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'requisition': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'orderDetail': const FhirField(true, 'CodeableConcept'),
    'quantityQuantity': const FhirField(false, 'Quantity'),
    'quantityRatio': const FhirField(false, 'Ratio'),
    'quantityRange': const FhirField(false, 'Range'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'asNeededBoolean': const FhirField(false, 'Boolean'),
    '_asNeededBoolean': const FhirField(false, 'Element'),
    'asNeededCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(true, 'Reference'),
    'locationCode': const FhirField(true, 'CodeableConcept'),
    'locationReference': const FhirField(true, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'insurance': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'specimen': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'patientInstruction': const FhirField(false, 'String'),
    '_patientInstruction': const FhirField(false, 'Element'),
    'relevantHistory': const FhirField(true, 'Reference'),
  },
  'VisionPrescription': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'dateWritten': const FhirField(false, 'FhirDateTime'),
    '_dateWritten': const FhirField(false, 'Element'),
    'prescriber': const FhirField(false, 'Reference'),
    'lensSpecification':
        const FhirField(true, 'VisionPrescriptionLensSpecification'),
  },
  'VisionPrescriptionLensSpecification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'product': const FhirField(false, 'CodeableConcept'),
    'eye': const FhirField(false, 'Code'),
    '_eye': const FhirField(false, 'Element'),
    'sphere': const FhirField(false, 'Decimal'),
    '_sphere': const FhirField(false, 'Element'),
    'cylinder': const FhirField(false, 'Decimal'),
    '_cylinder': const FhirField(false, 'Element'),
    'axis': const FhirField(false, 'Integer'),
    '_axis': const FhirField(false, 'Element'),
    'prism': const FhirField(true, 'VisionPrescriptionPrism'),
    'add': const FhirField(false, 'Decimal'),
    '_add': const FhirField(false, 'Element'),
    'power': const FhirField(false, 'Decimal'),
    '_power': const FhirField(false, 'Element'),
    'backCurve': const FhirField(false, 'Decimal'),
    '_backCurve': const FhirField(false, 'Element'),
    'diameter': const FhirField(false, 'Decimal'),
    '_diameter': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Quantity'),
    'color': const FhirField(false, 'String'),
    '_color': const FhirField(false, 'Element'),
    'brand': const FhirField(false, 'String'),
    '_brand': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'VisionPrescriptionPrism': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'amount': const FhirField(false, 'Decimal'),
    '_amount': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'Code'),
    '_base': const FhirField(false, 'Element'),
  },
  'Communication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'inResponseTo': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'medium': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'topic': const FhirField(false, 'CodeableConcept'),
    'about': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'sent': const FhirField(false, 'FhirDateTime'),
    '_sent': const FhirField(false, 'Element'),
    'received': const FhirField(false, 'FhirDateTime'),
    '_received': const FhirField(false, 'Element'),
    'recipient': const FhirField(true, 'Reference'),
    'sender': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'payload': const FhirField(true, 'CommunicationPayload'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CommunicationPayload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentString': const FhirField(false, 'String'),
    '_contentString': const FhirField(false, 'Element'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'CommunicationRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'medium': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'about': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'payload': const FhirField(true, 'CommunicationRequestPayload'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'recipient': const FhirField(true, 'Reference'),
    'sender': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CommunicationRequestPayload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentString': const FhirField(false, 'String'),
    '_contentString': const FhirField(false, 'Element'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'DeviceRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'priorRequest': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'codeReference': const FhirField(false, 'Reference'),
    'codeCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'parameter': const FhirField(true, 'DeviceRequestParameter'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'insurance': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'relevantHistory': const FhirField(true, 'Reference'),
  },
  'DeviceRequestParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
  },
  'DeviceUseStatement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    '_timingDateTime': const FhirField(false, 'Element'),
    'recordedOn': const FhirField(false, 'FhirDateTime'),
    '_recordedOn': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
  },
  'GuidanceResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'requestIdentifier': const FhirField(false, 'Identifier'),
    'identifier': const FhirField(true, 'Identifier'),
    'moduleUri': const FhirField(false, 'FhirUri'),
    '_moduleUri': const FhirField(false, 'Element'),
    'moduleCanonical': const FhirField(false, 'Canonical'),
    '_moduleCanonical': const FhirField(false, 'Element'),
    'moduleCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'performer': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'evaluationMessage': const FhirField(true, 'Reference'),
    'outputParameters': const FhirField(false, 'Reference'),
    'result': const FhirField(false, 'Reference'),
    'dataRequirement': const FhirField(true, 'DataRequirement'),
  },
  'SupplyDelivery': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'suppliedItem': const FhirField(false, 'SupplyDeliverySuppliedItem'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'supplier': const FhirField(false, 'Reference'),
    'destination': const FhirField(false, 'Reference'),
    'receiver': const FhirField(true, 'Reference'),
  },
  'SupplyDeliverySuppliedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
  },
  'SupplyRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'parameter': const FhirField(true, 'SupplyRequestParameter'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'supplier': const FhirField(true, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'deliverFrom': const FhirField(false, 'Reference'),
    'deliverTo': const FhirField(false, 'Reference'),
  },
  'SupplyRequestParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
  },
  'AdverseEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'actuality': const FhirField(false, 'Code'),
    '_actuality': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'event': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'detected': const FhirField(false, 'FhirDateTime'),
    '_detected': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    '_recordedDate': const FhirField(false, 'Element'),
    'resultingCondition': const FhirField(true, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'seriousness': const FhirField(false, 'CodeableConcept'),
    'severity': const FhirField(false, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'recorder': const FhirField(false, 'Reference'),
    'contributor': const FhirField(true, 'Reference'),
    'suspectEntity': const FhirField(true, 'AdverseEventSuspectEntity'),
    'subjectMedicalHistory': const FhirField(true, 'Reference'),
    'referenceDocument': const FhirField(true, 'Reference'),
    'study': const FhirField(true, 'Reference'),
  },
  'AdverseEventSuspectEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'instance': const FhirField(false, 'Reference'),
    'causality': const FhirField(true, 'AdverseEventCausality'),
  },
  'AdverseEventCausality': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'assessment': const FhirField(false, 'CodeableConcept'),
    'productRelatedness': const FhirField(false, 'String'),
    '_productRelatedness': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'method': const FhirField(false, 'CodeableConcept'),
  },
  'AllergyIntolerance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'clinicalStatus': const FhirField(false, 'CodeableConcept'),
    'verificationStatus': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'Code'),
    '_category': const FhirField(true, 'Element'),
    'criticality': const FhirField(false, 'Code'),
    '_criticality': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'onsetDateTime': const FhirField(false, 'FhirDateTime'),
    '_onsetDateTime': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetString': const FhirField(false, 'String'),
    '_onsetString': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    '_recordedDate': const FhirField(false, 'Element'),
    'recorder': const FhirField(false, 'Reference'),
    'asserter': const FhirField(false, 'Reference'),
    'lastOccurrence': const FhirField(false, 'FhirDateTime'),
    '_lastOccurrence': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'reaction': const FhirField(true, 'AllergyIntoleranceReaction'),
  },
  'AllergyIntoleranceReaction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableConcept'),
    'manifestation': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'onset': const FhirField(false, 'FhirDateTime'),
    '_onset': const FhirField(false, 'Element'),
    'severity': const FhirField(false, 'Code'),
    '_severity': const FhirField(false, 'Element'),
    'exposureRoute': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
  },
  'ClinicalImpression': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveDateTime': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'assessor': const FhirField(false, 'Reference'),
    'previous': const FhirField(false, 'Reference'),
    'problem': const FhirField(true, 'Reference'),
    'investigation': const FhirField(true, 'ClinicalImpressionInvestigation'),
    'protocol': const FhirField(true, 'FhirUri'),
    '_protocol': const FhirField(true, 'Element'),
    'summary': const FhirField(false, 'String'),
    '_summary': const FhirField(false, 'Element'),
    'finding': const FhirField(true, 'ClinicalImpressionFinding'),
    'prognosisCodeableConcept': const FhirField(true, 'CodeableConcept'),
    'prognosisReference': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'ClinicalImpressionInvestigation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'item': const FhirField(true, 'Reference'),
  },
  'ClinicalImpressionFinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
    'basis': const FhirField(false, 'String'),
    '_basis': const FhirField(false, 'Element'),
  },
  'Condition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'clinicalStatus': const FhirField(false, 'CodeableConcept'),
    'verificationStatus': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'severity': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'onsetDateTime': const FhirField(false, 'FhirDateTime'),
    '_onsetDateTime': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetString': const FhirField(false, 'String'),
    '_onsetString': const FhirField(false, 'Element'),
    'abatementDateTime': const FhirField(false, 'FhirDateTime'),
    '_abatementDateTime': const FhirField(false, 'Element'),
    'abatementAge': const FhirField(false, 'Age'),
    'abatementPeriod': const FhirField(false, 'Period'),
    'abatementRange': const FhirField(false, 'Range'),
    'abatementString': const FhirField(false, 'String'),
    '_abatementString': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    '_recordedDate': const FhirField(false, 'Element'),
    'recorder': const FhirField(false, 'Reference'),
    'asserter': const FhirField(false, 'Reference'),
    'stage': const FhirField(true, 'ConditionStage'),
    'evidence': const FhirField(true, 'ConditionEvidence'),
    'note': const FhirField(true, 'Annotation'),
  },
  'ConditionStage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'summary': const FhirField(false, 'CodeableConcept'),
    'assessment': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'ConditionEvidence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(true, 'CodeableConcept'),
    'detail': const FhirField(true, 'Reference'),
  },
  'DetectedIssue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'severity': const FhirField(false, 'Code'),
    '_severity': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'identifiedDateTime': const FhirField(false, 'FhirDateTime'),
    '_identifiedDateTime': const FhirField(false, 'Element'),
    'identifiedPeriod': const FhirField(false, 'Period'),
    'author': const FhirField(false, 'Reference'),
    'implicated': const FhirField(true, 'Reference'),
    'evidence': const FhirField(true, 'DetectedIssueEvidence'),
    'detail': const FhirField(false, 'String'),
    '_detail': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'FhirUri'),
    '_reference': const FhirField(false, 'Element'),
    'mitigation': const FhirField(true, 'DetectedIssueMitigation'),
  },
  'DetectedIssueEvidence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(true, 'CodeableConcept'),
    'detail': const FhirField(true, 'Reference'),
  },
  'DetectedIssueMitigation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
  },
  'FamilyMemberHistory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'sex': const FhirField(false, 'CodeableConcept'),
    'bornPeriod': const FhirField(false, 'Period'),
    'bornDate': const FhirField(false, 'Date'),
    '_bornDate': const FhirField(false, 'Element'),
    'bornString': const FhirField(false, 'String'),
    '_bornString': const FhirField(false, 'Element'),
    'ageAge': const FhirField(false, 'Age'),
    'ageRange': const FhirField(false, 'Range'),
    'ageString': const FhirField(false, 'String'),
    '_ageString': const FhirField(false, 'Element'),
    'estimatedAge': const FhirField(false, 'Boolean'),
    '_estimatedAge': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    '_deceasedBoolean': const FhirField(false, 'Element'),
    'deceasedAge': const FhirField(false, 'Age'),
    'deceasedRange': const FhirField(false, 'Range'),
    'deceasedDate': const FhirField(false, 'Date'),
    '_deceasedDate': const FhirField(false, 'Element'),
    'deceasedString': const FhirField(false, 'String'),
    '_deceasedString': const FhirField(false, 'Element'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'condition': const FhirField(true, 'FamilyMemberHistoryCondition'),
  },
  'FamilyMemberHistoryCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'contributedToDeath': const FhirField(false, 'Boolean'),
    '_contributedToDeath': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetString': const FhirField(false, 'String'),
    '_onsetString': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'Procedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    '_instantiatesUri': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'performedDateTime': const FhirField(false, 'FhirDateTime'),
    '_performedDateTime': const FhirField(false, 'Element'),
    'performedPeriod': const FhirField(false, 'Period'),
    'performedString': const FhirField(false, 'String'),
    '_performedString': const FhirField(false, 'Element'),
    'performedAge': const FhirField(false, 'Age'),
    'performedRange': const FhirField(false, 'Range'),
    'recorder': const FhirField(false, 'Reference'),
    'asserter': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'ProcedurePerformer'),
    'location': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'report': const FhirField(true, 'Reference'),
    'complication': const FhirField(true, 'CodeableConcept'),
    'complicationDetail': const FhirField(true, 'Reference'),
    'followUp': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'focalDevice': const FhirField(true, 'ProcedureFocalDevice'),
    'usedReference': const FhirField(true, 'Reference'),
    'usedCode': const FhirField(true, 'CodeableConcept'),
  },
  'ProcedurePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
  },
  'ProcedureFocalDevice': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(false, 'CodeableConcept'),
    'manipulated': const FhirField(false, 'Reference'),
  },
  'Basic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'Date'),
    '_created': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
  },
  'Binary': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    '_contentType': const FhirField(false, 'Element'),
    'securityContext': const FhirField(false, 'Reference'),
    'data': const FhirField(false, 'Base64Binary'),
    '_data': const FhirField(false, 'Element'),
  },
  'Bundle': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'Instant'),
    '_timestamp': const FhirField(false, 'Element'),
    'total': const FhirField(false, 'UnsignedInt'),
    '_total': const FhirField(false, 'Element'),
    'link': const FhirField(true, 'BundleLink'),
    'entry': const FhirField(true, 'BundleEntry'),
    'signature': const FhirField(false, 'Signature'),
  },
  'BundleLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relation': const FhirField(false, 'String'),
    '_relation': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
  },
  'BundleEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'link': const FhirField(true, 'BundleLink'),
    'fullUrl': const FhirField(false, 'FhirUri'),
    '_fullUrl': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Resource'),
    'search': const FhirField(false, 'BundleSearch'),
    'request': const FhirField(false, 'BundleRequest'),
    'response': const FhirField(false, 'BundleResponse'),
  },
  'BundleSearch': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'score': const FhirField(false, 'Decimal'),
    '_score': const FhirField(false, 'Element'),
  },
  'BundleRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'method': const FhirField(false, 'Code'),
    '_method': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'ifNoneMatch': const FhirField(false, 'String'),
    '_ifNoneMatch': const FhirField(false, 'Element'),
    'ifModifiedSince': const FhirField(false, 'Instant'),
    '_ifModifiedSince': const FhirField(false, 'Element'),
    'ifMatch': const FhirField(false, 'String'),
    '_ifMatch': const FhirField(false, 'Element'),
    'ifNoneExist': const FhirField(false, 'String'),
    '_ifNoneExist': const FhirField(false, 'Element'),
  },
  'BundleResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'String'),
    '_status': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'FhirUri'),
    '_location': const FhirField(false, 'Element'),
    'etag': const FhirField(false, 'String'),
    '_etag': const FhirField(false, 'Element'),
    'lastModified': const FhirField(false, 'Instant'),
    '_lastModified': const FhirField(false, 'Element'),
    'outcome': const FhirField(false, 'Resource'),
  },
  'Linkage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'LinkageItem'),
  },
  'LinkageItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
  },
  'MessageHeader': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'eventCoding': const FhirField(false, 'Coding'),
    'eventUri': const FhirField(false, 'FhirUri'),
    '_eventUri': const FhirField(false, 'Element'),
    'destination': const FhirField(true, 'MessageHeaderDestination'),
    'sender': const FhirField(false, 'Reference'),
    'enterer': const FhirField(false, 'Reference'),
    'author': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'MessageHeaderSource'),
    'responsible': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'response': const FhirField(false, 'MessageHeaderResponse'),
    'focus': const FhirField(true, 'Reference'),
    'definition': const FhirField(false, 'Canonical'),
  },
  'MessageHeaderDestination': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'target': const FhirField(false, 'Reference'),
    'endpoint': const FhirField(false, 'FhirUrl'),
    '_endpoint': const FhirField(false, 'Element'),
    'receiver': const FhirField(false, 'Reference'),
  },
  'MessageHeaderSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'software': const FhirField(false, 'String'),
    '_software': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'contact': const FhirField(false, 'ContactPoint'),
    'endpoint': const FhirField(false, 'FhirUrl'),
    '_endpoint': const FhirField(false, 'Element'),
  },
  'MessageHeaderResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Id'),
    '_identifier': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'details': const FhirField(false, 'Reference'),
  },
  'OperationOutcome': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'issue': const FhirField(true, 'OperationOutcomeIssue'),
  },
  'OperationOutcomeIssue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'severity': const FhirField(false, 'Code'),
    '_severity': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'details': const FhirField(false, 'CodeableConcept'),
    'diagnostics': const FhirField(false, 'String'),
    '_diagnostics': const FhirField(false, 'Element'),
    'location': const FhirField(true, 'String'),
    '_location': const FhirField(true, 'Element'),
    'expression': const FhirField(true, 'String'),
    '_expression': const FhirField(true, 'Element'),
  },
  'Parameters': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'ParametersParameter'),
  },
  'ParametersParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    '_valueCanonical': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    '_valueInstant': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    '_valueMarkdown': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Oid'),
    '_valueOid': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_valuePositiveInt': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_valueUnsignedInt': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    '_valueUrl': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Uuid'),
    '_valueUuid': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueContributor': const FhirField(false, 'Contributor'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
    'resource': const FhirField(false, 'Resource'),
    'part': const FhirField(true, 'ParametersParameter'),
  },
  'Subscription': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactPoint'),
    'end': const FhirField(false, 'Instant'),
    '_end': const FhirField(false, 'Element'),
    'reason': const FhirField(false, 'String'),
    '_reason': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'String'),
    '_criteria': const FhirField(false, 'Element'),
    'error': const FhirField(false, 'String'),
    '_error': const FhirField(false, 'Element'),
    'channel': const FhirField(false, 'SubscriptionChannel'),
  },
  'SubscriptionChannel': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'endpoint': const FhirField(false, 'FhirUrl'),
    '_endpoint': const FhirField(false, 'Element'),
    'payload': const FhirField(false, 'Code'),
    '_payload': const FhirField(false, 'Element'),
    'header': const FhirField(true, 'String'),
    '_header': const FhirField(true, 'Element'),
  },
  'SubscriptionStatus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'eventsSinceSubscriptionStart': const FhirField(false, 'Integer64'),
    '_eventsSinceSubscriptionStart': const FhirField(false, 'Element'),
    'notificationEvent':
        const FhirField(true, 'SubscriptionStatusNotificationEvent'),
    'subscription': const FhirField(false, 'Reference'),
    'topic': const FhirField(false, 'Canonical'),
    'error': const FhirField(true, 'CodeableConcept'),
  },
  'SubscriptionStatusNotificationEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'eventNumber': const FhirField(false, 'Integer64'),
    '_eventNumber': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'Instant'),
    '_timestamp': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'Reference'),
    'additionalContext': const FhirField(true, 'Reference'),
  },
  'SubscriptionTopic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'resourceTrigger':
        const FhirField(true, 'SubscriptionTopicResourceTrigger'),
    'eventTrigger': const FhirField(true, 'SubscriptionTopicEventTrigger'),
    'canFilterBy': const FhirField(true, 'SubscriptionTopicCanFilterBy'),
    'notificationShape':
        const FhirField(true, 'SubscriptionTopicNotificationShape'),
  },
  'SubscriptionTopicResourceTrigger': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'FhirUri'),
    '_resource': const FhirField(false, 'Element'),
    'supportedInteraction': const FhirField(true, 'Code'),
    '_supportedInteraction': const FhirField(true, 'Element'),
    'queryCriteria': const FhirField(false, 'SubscriptionTopicQueryCriteria'),
    'fhirPathCriteria': const FhirField(false, 'String'),
    '_fhirPathCriteria': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicQueryCriteria': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'previous': const FhirField(false, 'String'),
    '_previous': const FhirField(false, 'Element'),
    'resultForCreate': const FhirField(false, 'Code'),
    '_resultForCreate': const FhirField(false, 'Element'),
    'current': const FhirField(false, 'String'),
    '_current': const FhirField(false, 'Element'),
    'resultForDelete': const FhirField(false, 'Code'),
    '_resultForDelete': const FhirField(false, 'Element'),
    'requireBoth': const FhirField(false, 'Boolean'),
    '_requireBoth': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicEventTrigger': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'event': const FhirField(false, 'CodeableConcept'),
    'resource': const FhirField(false, 'FhirUri'),
    '_resource': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicCanFilterBy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'FhirUri'),
    '_resource': const FhirField(false, 'Element'),
    'filterParameter': const FhirField(false, 'String'),
    '_filterParameter': const FhirField(false, 'Element'),
    'filterDefinition': const FhirField(false, 'FhirUri'),
    '_filterDefinition': const FhirField(false, 'Element'),
    'modifier': const FhirField(true, 'Code'),
    '_modifier': const FhirField(true, 'Element'),
  },
  'SubscriptionTopicNotificationShape': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resource': const FhirField(false, 'FhirUri'),
    '_resource': const FhirField(false, 'Element'),
    'include': const FhirField(true, 'String'),
    '_include': const FhirField(true, 'Element'),
    'revInclude': const FhirField(true, 'String'),
    '_revInclude': const FhirField(true, 'Element'),
  },
  'CodeSystem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'caseSensitive': const FhirField(false, 'Boolean'),
    '_caseSensitive': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'hierarchyMeaning': const FhirField(false, 'Code'),
    '_hierarchyMeaning': const FhirField(false, 'Element'),
    'compositional': const FhirField(false, 'Boolean'),
    '_compositional': const FhirField(false, 'Element'),
    'versionNeeded': const FhirField(false, 'Boolean'),
    '_versionNeeded': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Code'),
    '_content': const FhirField(false, 'Element'),
    'supplements': const FhirField(false, 'Canonical'),
    'count': const FhirField(false, 'UnsignedInt'),
    '_count': const FhirField(false, 'Element'),
    'filter': const FhirField(true, 'CodeSystemFilter'),
    'property': const FhirField(true, 'CodeSystemProperty'),
    'concept': const FhirField(true, 'CodeSystemConcept'),
  },
  'CodeSystemFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'operator': const FhirField(true, 'Code'),
    '_operator': const FhirField(true, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'CodeSystemProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    '_uri': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
  },
  'CodeSystemConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'String'),
    '_definition': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'CodeSystemDesignation'),
    'property': const FhirField(true, 'CodeSystemProperty1'),
    'concept': const FhirField(true, 'CodeSystemConcept'),
  },
  'CodeSystemDesignation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Coding'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'CodeSystemProperty1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
  },
  'ConceptMap': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'sourceUri': const FhirField(false, 'FhirUri'),
    '_sourceUri': const FhirField(false, 'Element'),
    'sourceCanonical': const FhirField(false, 'Canonical'),
    '_sourceCanonical': const FhirField(false, 'Element'),
    'targetUri': const FhirField(false, 'FhirUri'),
    '_targetUri': const FhirField(false, 'Element'),
    'targetCanonical': const FhirField(false, 'Canonical'),
    '_targetCanonical': const FhirField(false, 'Element'),
    'group': const FhirField(true, 'ConceptMapGroup'),
  },
  'ConceptMapGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'source': const FhirField(false, 'FhirUri'),
    '_source': const FhirField(false, 'Element'),
    'sourceVersion': const FhirField(false, 'String'),
    '_sourceVersion': const FhirField(false, 'Element'),
    'target': const FhirField(false, 'FhirUri'),
    '_target': const FhirField(false, 'Element'),
    'targetVersion': const FhirField(false, 'String'),
    '_targetVersion': const FhirField(false, 'Element'),
    'element': const FhirField(true, 'ConceptMapElement'),
    'unmapped': const FhirField(false, 'ConceptMapUnmapped'),
  },
  'ConceptMapElement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'target': const FhirField(true, 'ConceptMapTarget'),
  },
  'ConceptMapTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'equivalence': const FhirField(false, 'Code'),
    '_equivalence': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
    'dependsOn': const FhirField(true, 'ConceptMapDependsOn'),
    'product': const FhirField(true, 'ConceptMapDependsOn'),
  },
  'ConceptMapDependsOn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'property': const FhirField(false, 'FhirUri'),
    '_property': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'Canonical'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
  },
  'ConceptMapUnmapped': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'Canonical'),
  },
  'NamingSystem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'responsible': const FhirField(false, 'String'),
    '_responsible': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'uniqueId': const FhirField(true, 'NamingSystemUniqueId'),
  },
  'NamingSystemUniqueId': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'preferred': const FhirField(false, 'Boolean'),
    '_preferred': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'TerminologyCapabilities': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'software': const FhirField(false, 'TerminologyCapabilitiesSoftware'),
    'implementation':
        const FhirField(false, 'TerminologyCapabilitiesImplementation'),
    'lockedDate': const FhirField(false, 'Boolean'),
    '_lockedDate': const FhirField(false, 'Element'),
    'codeSystem': const FhirField(true, 'TerminologyCapabilitiesCodeSystem'),
    'expansion': const FhirField(false, 'TerminologyCapabilitiesExpansion'),
    'codeSearch': const FhirField(false, 'Code'),
    '_codeSearch': const FhirField(false, 'Element'),
    'validateCode':
        const FhirField(false, 'TerminologyCapabilitiesValidateCode'),
    'translation': const FhirField(false, 'TerminologyCapabilitiesTranslation'),
    'closure': const FhirField(false, 'TerminologyCapabilitiesClosure'),
  },
  'TerminologyCapabilitiesSoftware': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesImplementation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    '_url': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesCodeSystem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uri': const FhirField(false, 'Canonical'),
    'version': const FhirField(true, 'TerminologyCapabilitiesVersion'),
    'subsumption': const FhirField(false, 'Boolean'),
    '_subsumption': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'String'),
    '_code': const FhirField(false, 'Element'),
    'isDefault': const FhirField(false, 'Boolean'),
    '_isDefault': const FhirField(false, 'Element'),
    'compositional': const FhirField(false, 'Boolean'),
    '_compositional': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'Code'),
    '_language': const FhirField(true, 'Element'),
    'filter': const FhirField(true, 'TerminologyCapabilitiesFilter'),
    'property': const FhirField(true, 'Code'),
    '_property': const FhirField(true, 'Element'),
  },
  'TerminologyCapabilitiesFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'op': const FhirField(true, 'Code'),
    '_op': const FhirField(true, 'Element'),
  },
  'TerminologyCapabilitiesExpansion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'hierarchical': const FhirField(false, 'Boolean'),
    '_hierarchical': const FhirField(false, 'Element'),
    'paging': const FhirField(false, 'Boolean'),
    '_paging': const FhirField(false, 'Element'),
    'incomplete': const FhirField(false, 'Boolean'),
    '_incomplete': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'TerminologyCapabilitiesParameter'),
    'textFilter': const FhirField(false, 'Markdown'),
    '_textFilter': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    '_name': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesValidateCode': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'translations': const FhirField(false, 'Boolean'),
    '_translations': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesTranslation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'needsMap': const FhirField(false, 'Boolean'),
    '_needsMap': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesClosure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'translation': const FhirField(false, 'Boolean'),
    '_translation': const FhirField(false, 'Element'),
  },
  'ValueSet': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'immutable': const FhirField(false, 'Boolean'),
    '_immutable': const FhirField(false, 'Element'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'compose': const FhirField(false, 'ValueSetCompose'),
    'expansion': const FhirField(false, 'ValueSetExpansion'),
  },
  'ValueSetCompose': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'lockedDate': const FhirField(false, 'Date'),
    '_lockedDate': const FhirField(false, 'Element'),
    'inactive': const FhirField(false, 'Boolean'),
    '_inactive': const FhirField(false, 'Element'),
    'include': const FhirField(true, 'ValueSetInclude'),
    'exclude': const FhirField(true, 'ValueSetInclude'),
  },
  'ValueSetInclude': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'concept': const FhirField(true, 'ValueSetConcept'),
    'filter': const FhirField(true, 'ValueSetFilter'),
    'valueSet': const FhirField(true, 'Canonical'),
  },
  'ValueSetConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'ValueSetDesignation'),
  },
  'ValueSetDesignation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Coding'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'ValueSetFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'property': const FhirField(false, 'Code'),
    '_property': const FhirField(false, 'Element'),
    'op': const FhirField(false, 'Code'),
    '_op': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'ValueSetExpansion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'FhirUri'),
    '_identifier': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'FhirDateTime'),
    '_timestamp': const FhirField(false, 'Element'),
    'total': const FhirField(false, 'Integer'),
    '_total': const FhirField(false, 'Element'),
    'offset': const FhirField(false, 'Integer'),
    '_offset': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'ValueSetParameter'),
    'contains': const FhirField(true, 'ValueSetContains'),
  },
  'ValueSetParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
  },
  'ValueSetContains': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'abstract': const FhirField(false, 'Boolean'),
    '_abstract': const FhirField(false, 'Element'),
    'inactive': const FhirField(false, 'Boolean'),
    '_inactive': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'ValueSetDesignation'),
    'contains': const FhirField(true, 'ValueSetContains'),
  },
  'CatalogEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'orderable': const FhirField(false, 'Boolean'),
    '_orderable': const FhirField(false, 'Element'),
    'referencedItem': const FhirField(false, 'Reference'),
    'additionalIdentifier': const FhirField(true, 'Identifier'),
    'classification': const FhirField(true, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'validityPeriod': const FhirField(false, 'Period'),
    'validTo': const FhirField(false, 'FhirDateTime'),
    '_validTo': const FhirField(false, 'Element'),
    'lastUpdated': const FhirField(false, 'FhirDateTime'),
    '_lastUpdated': const FhirField(false, 'Element'),
    'additionalCharacteristic': const FhirField(true, 'CodeableConcept'),
    'additionalClassification': const FhirField(true, 'CodeableConcept'),
    'relatedEntry': const FhirField(true, 'CatalogEntryRelatedEntry'),
  },
  'CatalogEntryRelatedEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationtype': const FhirField(false, 'Code'),
    '_relationtype': const FhirField(false, 'Element'),
    'item': const FhirField(false, 'Reference'),
  },
  'Composition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'Reference'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'confidentiality': const FhirField(false, 'Code'),
    '_confidentiality': const FhirField(false, 'Element'),
    'attester': const FhirField(true, 'CompositionAttester'),
    'custodian': const FhirField(false, 'Reference'),
    'relatesTo': const FhirField(true, 'CompositionRelatesTo'),
    'event': const FhirField(true, 'CompositionEvent'),
    'section': const FhirField(true, 'CompositionSection'),
  },
  'CompositionAttester': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'time': const FhirField(false, 'FhirDateTime'),
    '_time': const FhirField(false, 'Element'),
    'party': const FhirField(false, 'Reference'),
  },
  'CompositionRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'targetIdentifier': const FhirField(false, 'Identifier'),
    'targetReference': const FhirField(false, 'Reference'),
  },
  'CompositionEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'detail': const FhirField(true, 'Reference'),
  },
  'CompositionSection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'author': const FhirField(true, 'Reference'),
    'focus': const FhirField(false, 'Reference'),
    'text': const FhirField(false, 'Narrative'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'entry': const FhirField(true, 'Reference'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
    'section': const FhirField(true, 'CompositionSection'),
  },
  'DocumentManifest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'masterIdentifier': const FhirField(false, 'Identifier'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'Reference'),
    'recipient': const FhirField(true, 'Reference'),
    'source': const FhirField(false, 'FhirUri'),
    '_source': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'content': const FhirField(true, 'Reference'),
    'related': const FhirField(true, 'DocumentManifestRelated'),
  },
  'DocumentManifestRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'ref': const FhirField(false, 'Reference'),
  },
  'DocumentReference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'masterIdentifier': const FhirField(false, 'Identifier'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'docStatus': const FhirField(false, 'Code'),
    '_docStatus': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'Instant'),
    '_date': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'Reference'),
    'authenticator': const FhirField(false, 'Reference'),
    'custodian': const FhirField(false, 'Reference'),
    'relatesTo': const FhirField(true, 'DocumentReferenceRelatesTo'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'securityLabel': const FhirField(true, 'CodeableConcept'),
    'content': const FhirField(true, 'DocumentReferenceContent'),
    'context': const FhirField(false, 'DocumentReferenceContext'),
  },
  'DocumentReferenceRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'target': const FhirField(false, 'Reference'),
  },
  'DocumentReferenceContent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'attachment': const FhirField(false, 'Attachment'),
    'format': const FhirField(false, 'Coding'),
  },
  'DocumentReferenceContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'encounter': const FhirField(true, 'Reference'),
    'event': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'facilityType': const FhirField(false, 'CodeableConcept'),
    'practiceSetting': const FhirField(false, 'CodeableConcept'),
    'sourcePatientInfo': const FhirField(false, 'Reference'),
    'related': const FhirField(true, 'Reference'),
  },
  'CapabilityStatement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'instantiates': const FhirField(true, 'Canonical'),
    'imports': const FhirField(true, 'Canonical'),
    'software': const FhirField(false, 'CapabilityStatementSoftware'),
    'implementation':
        const FhirField(false, 'CapabilityStatementImplementation'),
    'fhirVersion': const FhirField(false, 'Code'),
    '_fhirVersion': const FhirField(false, 'Element'),
    'format': const FhirField(true, 'Code'),
    '_format': const FhirField(true, 'Element'),
    'patchFormat': const FhirField(true, 'Code'),
    '_patchFormat': const FhirField(true, 'Element'),
    'implementationGuide': const FhirField(true, 'Canonical'),
    'rest': const FhirField(true, 'CapabilityStatementRest'),
    'messaging': const FhirField(true, 'CapabilityStatementMessaging'),
    'document': const FhirField(true, 'CapabilityStatementDocument'),
  },
  'CapabilityStatementSoftware': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'releaseDate': const FhirField(false, 'FhirDateTime'),
    '_releaseDate': const FhirField(false, 'Element'),
  },
  'CapabilityStatementImplementation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    '_url': const FhirField(false, 'Element'),
    'custodian': const FhirField(false, 'Reference'),
  },
  'CapabilityStatementRest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
    'security': const FhirField(false, 'CapabilityStatementSecurity'),
    'resource': const FhirField(true, 'CapabilityStatementResource'),
    'interaction': const FhirField(true, 'CapabilityStatementInteraction1'),
    'searchParam': const FhirField(true, 'CapabilityStatementSearchParam'),
    'operation': const FhirField(true, 'CapabilityStatementOperation'),
    'compartment': const FhirField(true, 'Canonical'),
  },
  'CapabilityStatementSecurity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'cors': const FhirField(false, 'Boolean'),
    '_cors': const FhirField(false, 'Element'),
    'service': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
  },
  'CapabilityStatementResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'supportedProfile': const FhirField(true, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
    'interaction': const FhirField(true, 'CapabilityStatementInteraction'),
    'versioning': const FhirField(false, 'Code'),
    '_versioning': const FhirField(false, 'Element'),
    'readHistory': const FhirField(false, 'Boolean'),
    '_readHistory': const FhirField(false, 'Element'),
    'updateCreate': const FhirField(false, 'Boolean'),
    '_updateCreate': const FhirField(false, 'Element'),
    'conditionalCreate': const FhirField(false, 'Boolean'),
    '_conditionalCreate': const FhirField(false, 'Element'),
    'conditionalRead': const FhirField(false, 'Code'),
    '_conditionalRead': const FhirField(false, 'Element'),
    'conditionalUpdate': const FhirField(false, 'Boolean'),
    '_conditionalUpdate': const FhirField(false, 'Element'),
    'conditionalDelete': const FhirField(false, 'Code'),
    '_conditionalDelete': const FhirField(false, 'Element'),
    'referencePolicy': const FhirField(true, 'Code'),
    '_referencePolicy': const FhirField(true, 'Element'),
    'searchInclude': const FhirField(true, 'String'),
    '_searchInclude': const FhirField(true, 'Element'),
    'searchRevInclude': const FhirField(true, 'String'),
    '_searchRevInclude': const FhirField(true, 'Element'),
    'searchParam': const FhirField(true, 'CapabilityStatementSearchParam'),
    'operation': const FhirField(true, 'CapabilityStatementOperation'),
  },
  'CapabilityStatementInteraction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'CapabilityStatementSearchParam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'CapabilityStatementOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'CapabilityStatementInteraction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'CapabilityStatementMessaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'endpoint': const FhirField(true, 'CapabilityStatementEndpoint'),
    'reliableCache': const FhirField(false, 'UnsignedInt'),
    '_reliableCache': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
    'supportedMessage':
        const FhirField(true, 'CapabilityStatementSupportedMessage'),
  },
  'CapabilityStatementEndpoint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'protocol': const FhirField(false, 'Coding'),
    'address': const FhirField(false, 'FhirUrl'),
    '_address': const FhirField(false, 'Element'),
  },
  'CapabilityStatementSupportedMessage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
  },
  'CapabilityStatementDocument': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    '_documentation': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'CompartmentDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'search': const FhirField(false, 'Boolean'),
    '_search': const FhirField(false, 'Element'),
    'resource': const FhirField(true, 'CompartmentDefinitionResource'),
  },
  'CompartmentDefinitionResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'param': const FhirField(true, 'String'),
    '_param': const FhirField(true, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'ExampleScenario': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'actor': const FhirField(true, 'ExampleScenarioActor'),
    'instance': const FhirField(true, 'ExampleScenarioInstance'),
    'process': const FhirField(true, 'ExampleScenarioProcess'),
    'workflow': const FhirField(true, 'Canonical'),
  },
  'ExampleScenarioActor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actorId': const FhirField(false, 'String'),
    '_actorId': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
  },
  'ExampleScenarioInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resourceId': const FhirField(false, 'String'),
    '_resourceId': const FhirField(false, 'Element'),
    'resourceType': const FhirField(false, 'Code'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'version': const FhirField(true, 'ExampleScenarioVersion'),
    'containedInstance':
        const FhirField(true, 'ExampleScenarioContainedInstance'),
  },
  'ExampleScenarioVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'versionId': const FhirField(false, 'String'),
    '_versionId': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
  },
  'ExampleScenarioContainedInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resourceId': const FhirField(false, 'String'),
    '_resourceId': const FhirField(false, 'Element'),
    'versionId': const FhirField(false, 'String'),
    '_versionId': const FhirField(false, 'Element'),
  },
  'ExampleScenarioProcess': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'preConditions': const FhirField(false, 'Markdown'),
    '_preConditions': const FhirField(false, 'Element'),
    'postConditions': const FhirField(false, 'Markdown'),
    '_postConditions': const FhirField(false, 'Element'),
    'step': const FhirField(true, 'ExampleScenarioStep'),
  },
  'ExampleScenarioStep': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'process': const FhirField(true, 'ExampleScenarioProcess'),
    'pause': const FhirField(false, 'Boolean'),
    '_pause': const FhirField(false, 'Element'),
    'operation': const FhirField(false, 'ExampleScenarioOperation'),
    'alternative': const FhirField(true, 'ExampleScenarioAlternative'),
  },
  'ExampleScenarioOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'String'),
    '_number': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'initiator': const FhirField(false, 'String'),
    '_initiator': const FhirField(false, 'Element'),
    'receiver': const FhirField(false, 'String'),
    '_receiver': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'initiatorActive': const FhirField(false, 'Boolean'),
    '_initiatorActive': const FhirField(false, 'Element'),
    'receiverActive': const FhirField(false, 'Boolean'),
    '_receiverActive': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'ExampleScenarioContainedInstance'),
    'response': const FhirField(false, 'ExampleScenarioContainedInstance'),
  },
  'ExampleScenarioAlternative': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'step': const FhirField(true, 'ExampleScenarioStep'),
  },
  'GraphDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'start': const FhirField(false, 'Code'),
    '_start': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'link': const FhirField(true, 'GraphDefinitionLink'),
  },
  'GraphDefinitionLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'sliceName': const FhirField(false, 'String'),
    '_sliceName': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'target': const FhirField(true, 'GraphDefinitionTarget'),
  },
  'GraphDefinitionTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'params': const FhirField(false, 'String'),
    '_params': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'compartment': const FhirField(true, 'GraphDefinitionCompartment'),
    'link': const FhirField(true, 'GraphDefinitionLink'),
  },
  'GraphDefinitionCompartment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'rule': const FhirField(false, 'Code'),
    '_rule': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
  },
  'ImplementationGuide': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'packageId': const FhirField(false, 'Id'),
    '_packageId': const FhirField(false, 'Element'),
    'license': const FhirField(false, 'Code'),
    '_license': const FhirField(false, 'Element'),
    'fhirVersion': const FhirField(true, 'Code'),
    '_fhirVersion': const FhirField(true, 'Element'),
    'dependsOn': const FhirField(true, 'ImplementationGuideDependsOn'),
    'global': const FhirField(true, 'ImplementationGuideGlobal'),
    'definition': const FhirField(false, 'ImplementationGuideDefinition'),
    'manifest': const FhirField(false, 'ImplementationGuideManifest'),
  },
  'ImplementationGuideDependsOn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uri': const FhirField(false, 'Canonical'),
    'packageId': const FhirField(false, 'Id'),
    '_packageId': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
  },
  'ImplementationGuideGlobal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'ImplementationGuideDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'grouping': const FhirField(true, 'ImplementationGuideGrouping'),
    'resource': const FhirField(true, 'ImplementationGuideResource'),
    'page': const FhirField(false, 'ImplementationGuidePage'),
    'parameter': const FhirField(true, 'ImplementationGuideParameter'),
    'template': const FhirField(true, 'ImplementationGuideTemplate'),
  },
  'ImplementationGuideGrouping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
  },
  'ImplementationGuideResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'fhirVersion': const FhirField(true, 'Code'),
    '_fhirVersion': const FhirField(true, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'exampleBoolean': const FhirField(false, 'Boolean'),
    '_exampleBoolean': const FhirField(false, 'Element'),
    'exampleCanonical': const FhirField(false, 'Canonical'),
    '_exampleCanonical': const FhirField(false, 'Element'),
    'groupingId': const FhirField(false, 'Id'),
    '_groupingId': const FhirField(false, 'Element'),
  },
  'ImplementationGuidePage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'nameUrl': const FhirField(false, 'FhirUrl'),
    '_nameUrl': const FhirField(false, 'Element'),
    'nameReference': const FhirField(false, 'Reference'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'generation': const FhirField(false, 'Code'),
    '_generation': const FhirField(false, 'Element'),
    'page': const FhirField(true, 'ImplementationGuidePage'),
  },
  'ImplementationGuideParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'ImplementationGuideTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'String'),
    '_source': const FhirField(false, 'Element'),
    'scope': const FhirField(false, 'String'),
    '_scope': const FhirField(false, 'Element'),
  },
  'ImplementationGuideManifest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'rendering': const FhirField(false, 'FhirUrl'),
    '_rendering': const FhirField(false, 'Element'),
    'resource': const FhirField(true, 'ImplementationGuideResource1'),
    'page': const FhirField(true, 'ImplementationGuidePage1'),
    'image': const FhirField(true, 'String'),
    '_image': const FhirField(true, 'Element'),
    'other': const FhirField(true, 'String'),
    '_other': const FhirField(true, 'Element'),
  },
  'ImplementationGuideResource1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'exampleBoolean': const FhirField(false, 'Boolean'),
    '_exampleBoolean': const FhirField(false, 'Element'),
    'exampleCanonical': const FhirField(false, 'Canonical'),
    '_exampleCanonical': const FhirField(false, 'Element'),
    'relativePath': const FhirField(false, 'FhirUrl'),
    '_relativePath': const FhirField(false, 'Element'),
  },
  'ImplementationGuidePage1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'anchor': const FhirField(true, 'String'),
    '_anchor': const FhirField(true, 'Element'),
  },
  'MessageDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'replaces': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'Canonical'),
    'parent': const FhirField(true, 'Canonical'),
    'eventCoding': const FhirField(false, 'Coding'),
    'eventUri': const FhirField(false, 'FhirUri'),
    '_eventUri': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'Code'),
    '_category': const FhirField(false, 'Element'),
    'focus': const FhirField(true, 'MessageDefinitionFocus'),
    'responseRequired': const FhirField(false, 'Code'),
    '_responseRequired': const FhirField(false, 'Element'),
    'allowedResponse':
        const FhirField(true, 'MessageDefinitionAllowedResponse'),
    'graph': const FhirField(true, 'Canonical'),
  },
  'MessageDefinitionFocus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'min': const FhirField(false, 'UnsignedInt'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
  },
  'MessageDefinitionAllowedResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'message': const FhirField(false, 'Canonical'),
    'situation': const FhirField(false, 'Markdown'),
    '_situation': const FhirField(false, 'Element'),
  },
  'OperationDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'affectsState': const FhirField(false, 'Boolean'),
    '_affectsState': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    '_comment': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'Canonical'),
    'resource': const FhirField(true, 'Code'),
    '_resource': const FhirField(true, 'Element'),
    'system': const FhirField(false, 'Boolean'),
    '_system': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Boolean'),
    '_type': const FhirField(false, 'Element'),
    'instance': const FhirField(false, 'Boolean'),
    '_instance': const FhirField(false, 'Element'),
    'inputProfile': const FhirField(false, 'Canonical'),
    'outputProfile': const FhirField(false, 'Canonical'),
    'parameter': const FhirField(true, 'OperationDefinitionParameter'),
    'overload': const FhirField(true, 'OperationDefinitionOverload'),
  },
  'OperationDefinitionParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    '_name': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'targetProfile': const FhirField(true, 'Canonical'),
    'searchType': const FhirField(false, 'Code'),
    '_searchType': const FhirField(false, 'Element'),
    'binding': const FhirField(false, 'OperationDefinitionBinding'),
    'referencedFrom':
        const FhirField(true, 'OperationDefinitionReferencedFrom'),
    'part': const FhirField(true, 'OperationDefinitionParameter'),
  },
  'OperationDefinitionBinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'strength': const FhirField(false, 'Code'),
    '_strength': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
  },
  'OperationDefinitionReferencedFrom': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'source': const FhirField(false, 'String'),
    '_source': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'String'),
    '_sourceId': const FhirField(false, 'Element'),
  },
  'OperationDefinitionOverload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'parameterName': const FhirField(true, 'String'),
    '_parameterName': const FhirField(true, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'SearchParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(false, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'base': const FhirField(true, 'Code'),
    '_base': const FhirField(true, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'xpath': const FhirField(false, 'String'),
    '_xpath': const FhirField(false, 'Element'),
    'xpathUsage': const FhirField(false, 'Code'),
    '_xpathUsage': const FhirField(false, 'Element'),
    'target': const FhirField(true, 'Code'),
    '_target': const FhirField(true, 'Element'),
    'multipleOr': const FhirField(false, 'Boolean'),
    '_multipleOr': const FhirField(false, 'Element'),
    'multipleAnd': const FhirField(false, 'Boolean'),
    '_multipleAnd': const FhirField(false, 'Element'),
    'comparator': const FhirField(true, 'Code'),
    '_comparator': const FhirField(true, 'Element'),
    'modifier': const FhirField(true, 'Code'),
    '_modifier': const FhirField(true, 'Element'),
    'chain': const FhirField(true, 'String'),
    '_chain': const FhirField(true, 'Element'),
    'component': const FhirField(true, 'SearchParameterComponent'),
  },
  'SearchParameterComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'definition': const FhirField(false, 'Canonical'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
  },
  'StructureDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'keyword': const FhirField(true, 'Coding'),
    'fhirVersion': const FhirField(false, 'Code'),
    '_fhirVersion': const FhirField(false, 'Element'),
    'mapping': const FhirField(true, 'StructureDefinitionMapping'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'abstract': const FhirField(false, 'Boolean'),
    '_abstract': const FhirField(false, 'Element'),
    'context': const FhirField(true, 'StructureDefinitionContext'),
    'contextInvariant': const FhirField(true, 'String'),
    '_contextInvariant': const FhirField(true, 'Element'),
    'type': const FhirField(false, 'FhirUri'),
    '_type': const FhirField(false, 'Element'),
    'baseDefinition': const FhirField(false, 'Canonical'),
    '_baseDefinition': const FhirField(false, 'Element'),
    'derivation': const FhirField(false, 'Code'),
    '_derivation': const FhirField(false, 'Element'),
    'snapshot': const FhirField(false, 'StructureDefinitionSnapshot'),
    'differential': const FhirField(false, 'StructureDefinitionDifferential'),
  },
  'StructureDefinitionMapping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identity': const FhirField(false, 'Id'),
    '_identity': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    '_uri': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'StructureDefinitionContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
  },
  'StructureDefinitionSnapshot': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'element': const FhirField(true, 'ElementDefinition'),
  },
  'StructureDefinitionDifferential': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'element': const FhirField(true, 'ElementDefinition'),
  },
  'StructureMap': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'structure': const FhirField(true, 'StructureMapStructure'),
    'import': const FhirField(true, 'Canonical'),
    'group': const FhirField(true, 'StructureMapGroup'),
  },
  'StructureMapStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'Canonical'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'alias': const FhirField(false, 'String'),
    '_alias': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'StructureMapGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    '_name': const FhirField(false, 'Element'),
    'extends': const FhirField(false, 'Id'),
    '_extends': const FhirField(false, 'Element'),
    'typeMode': const FhirField(false, 'Code'),
    '_typeMode': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
    'input': const FhirField(true, 'StructureMapInput'),
    'rule': const FhirField(true, 'StructureMapRule'),
  },
  'StructureMapInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'StructureMapRule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    '_name': const FhirField(false, 'Element'),
    'source': const FhirField(true, 'StructureMapSource'),
    'target': const FhirField(true, 'StructureMapTarget'),
    'rule': const FhirField(true, 'StructureMapRule'),
    'dependent': const FhirField(true, 'StructureMapDependent'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
  },
  'StructureMapSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'context': const FhirField(false, 'Id'),
    '_context': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'defaultValueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_defaultValueBase64Binary': const FhirField(false, 'Element'),
    'defaultValueBoolean': const FhirField(false, 'Boolean'),
    '_defaultValueBoolean': const FhirField(false, 'Element'),
    'defaultValueCanonical': const FhirField(false, 'Canonical'),
    '_defaultValueCanonical': const FhirField(false, 'Element'),
    'defaultValueCode': const FhirField(false, 'Code'),
    '_defaultValueCode': const FhirField(false, 'Element'),
    'defaultValueDate': const FhirField(false, 'Date'),
    '_defaultValueDate': const FhirField(false, 'Element'),
    'defaultValueDateTime': const FhirField(false, 'FhirDateTime'),
    '_defaultValueDateTime': const FhirField(false, 'Element'),
    'defaultValueDecimal': const FhirField(false, 'Decimal'),
    '_defaultValueDecimal': const FhirField(false, 'Element'),
    'defaultValueId': const FhirField(false, 'Id'),
    '_defaultValueId': const FhirField(false, 'Element'),
    'defaultValueInstant': const FhirField(false, 'Instant'),
    '_defaultValueInstant': const FhirField(false, 'Element'),
    'defaultValueInteger': const FhirField(false, 'Integer'),
    '_defaultValueInteger': const FhirField(false, 'Element'),
    'defaultValueMarkdown': const FhirField(false, 'Markdown'),
    '_defaultValueMarkdown': const FhirField(false, 'Element'),
    'defaultValueOid': const FhirField(false, 'Oid'),
    '_defaultValueOid': const FhirField(false, 'Element'),
    'defaultValuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_defaultValuePositiveInt': const FhirField(false, 'Element'),
    'defaultValueString': const FhirField(false, 'String'),
    '_defaultValueString': const FhirField(false, 'Element'),
    'defaultValueTime': const FhirField(false, 'Time'),
    '_defaultValueTime': const FhirField(false, 'Element'),
    'defaultValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_defaultValueUnsignedInt': const FhirField(false, 'Element'),
    'defaultValueUri': const FhirField(false, 'FhirUri'),
    '_defaultValueUri': const FhirField(false, 'Element'),
    'defaultValueUrl': const FhirField(false, 'FhirUrl'),
    '_defaultValueUrl': const FhirField(false, 'Element'),
    'defaultValueUuid': const FhirField(false, 'Uuid'),
    '_defaultValueUuid': const FhirField(false, 'Element'),
    'defaultValueAddress': const FhirField(false, 'Address'),
    'defaultValueAge': const FhirField(false, 'Age'),
    'defaultValueAnnotation': const FhirField(false, 'Annotation'),
    'defaultValueAttachment': const FhirField(false, 'Attachment'),
    'defaultValueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'defaultValueCoding': const FhirField(false, 'Coding'),
    'defaultValueContactPoint': const FhirField(false, 'ContactPoint'),
    'defaultValueCount': const FhirField(false, 'Count'),
    'defaultValueDistance': const FhirField(false, 'Distance'),
    'defaultValueDuration': const FhirField(false, 'FhirDuration'),
    'defaultValueHumanName': const FhirField(false, 'HumanName'),
    'defaultValueIdentifier': const FhirField(false, 'Identifier'),
    'defaultValueMoney': const FhirField(false, 'Money'),
    'defaultValuePeriod': const FhirField(false, 'Period'),
    'defaultValueQuantity': const FhirField(false, 'Quantity'),
    'defaultValueRange': const FhirField(false, 'Range'),
    'defaultValueRatio': const FhirField(false, 'Ratio'),
    'defaultValueReference': const FhirField(false, 'Reference'),
    'defaultValueSampledData': const FhirField(false, 'SampledData'),
    'defaultValueSignature': const FhirField(false, 'Signature'),
    'defaultValueTiming': const FhirField(false, 'Timing'),
    'defaultValueContactDetail': const FhirField(false, 'ContactDetail'),
    'defaultValueContributor': const FhirField(false, 'Contributor'),
    'defaultValueDataRequirement': const FhirField(false, 'DataRequirement'),
    'defaultValueExpression': const FhirField(false, 'Expression'),
    'defaultValueParameterDefinition':
        const FhirField(false, 'ParameterDefinition'),
    'defaultValueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'defaultValueTriggerDefinition':
        const FhirField(false, 'TriggerDefinition'),
    'defaultValueUsageContext': const FhirField(false, 'UsageContext'),
    'defaultValueDosage': const FhirField(false, 'Dosage'),
    'defaultValueMeta': const FhirField(false, 'Meta'),
    'element': const FhirField(false, 'String'),
    '_element': const FhirField(false, 'Element'),
    'listMode': const FhirField(false, 'Code'),
    '_listMode': const FhirField(false, 'Element'),
    'variable': const FhirField(false, 'Id'),
    '_variable': const FhirField(false, 'Element'),
    'condition': const FhirField(false, 'String'),
    '_condition': const FhirField(false, 'Element'),
    'check': const FhirField(false, 'String'),
    '_check': const FhirField(false, 'Element'),
    'logMessage': const FhirField(false, 'String'),
    '_logMessage': const FhirField(false, 'Element'),
  },
  'StructureMapTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'context': const FhirField(false, 'Id'),
    '_context': const FhirField(false, 'Element'),
    'contextType': const FhirField(false, 'Code'),
    '_contextType': const FhirField(false, 'Element'),
    'element': const FhirField(false, 'String'),
    '_element': const FhirField(false, 'Element'),
    'variable': const FhirField(false, 'Id'),
    '_variable': const FhirField(false, 'Element'),
    'listMode': const FhirField(true, 'Code'),
    '_listMode': const FhirField(true, 'Element'),
    'listRuleId': const FhirField(false, 'Id'),
    '_listRuleId': const FhirField(false, 'Element'),
    'transform': const FhirField(false, 'Code'),
    '_transform': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'StructureMapParameter'),
  },
  'StructureMapParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
  },
  'StructureMapDependent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    '_name': const FhirField(false, 'Element'),
    'variable': const FhirField(true, 'String'),
    '_variable': const FhirField(true, 'Element'),
  },
  'AuditEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'subtype': const FhirField(true, 'Coding'),
    'action': const FhirField(false, 'Code'),
    '_action': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'recorded': const FhirField(false, 'Instant'),
    '_recorded': const FhirField(false, 'Element'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'outcomeDesc': const FhirField(false, 'String'),
    '_outcomeDesc': const FhirField(false, 'Element'),
    'purposeOfEvent': const FhirField(true, 'CodeableConcept'),
    'agent': const FhirField(true, 'AuditEventAgent'),
    'source': const FhirField(false, 'AuditEventSource'),
    'entity': const FhirField(true, 'AuditEventEntity'),
  },
  'AuditEventAgent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'role': const FhirField(true, 'CodeableConcept'),
    'who': const FhirField(false, 'Reference'),
    'altId': const FhirField(false, 'String'),
    '_altId': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'requestor': const FhirField(false, 'Boolean'),
    '_requestor': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'policy': const FhirField(true, 'FhirUri'),
    '_policy': const FhirField(true, 'Element'),
    'media': const FhirField(false, 'Coding'),
    'network': const FhirField(false, 'AuditEventNetwork'),
    'purposeOfUse': const FhirField(true, 'CodeableConcept'),
  },
  'AuditEventNetwork': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'address': const FhirField(false, 'String'),
    '_address': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
  },
  'AuditEventSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(false, 'String'),
    '_site': const FhirField(false, 'Element'),
    'observer': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'Coding'),
  },
  'AuditEventEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'what': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'Coding'),
    'role': const FhirField(false, 'Coding'),
    'lifecycle': const FhirField(false, 'Coding'),
    'securityLabel': const FhirField(true, 'Coding'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'query': const FhirField(false, 'Base64Binary'),
    '_query': const FhirField(false, 'Element'),
    'detail': const FhirField(true, 'AuditEventDetail'),
  },
  'AuditEventDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
  },
  'Consent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'scope': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'dateTime': const FhirField(false, 'FhirDateTime'),
    '_dateTime': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'Reference'),
    'organization': const FhirField(true, 'Reference'),
    'sourceAttachment': const FhirField(false, 'Attachment'),
    'sourceReference': const FhirField(false, 'Reference'),
    'policy': const FhirField(true, 'ConsentPolicy'),
    'policyRule': const FhirField(false, 'CodeableConcept'),
    'verification': const FhirField(true, 'ConsentVerification'),
    'provision': const FhirField(false, 'ConsentProvision'),
  },
  'ConsentPolicy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'authority': const FhirField(false, 'FhirUri'),
    '_authority': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    '_uri': const FhirField(false, 'Element'),
  },
  'ConsentVerification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'verified': const FhirField(false, 'Boolean'),
    '_verified': const FhirField(false, 'Element'),
    'verifiedWith': const FhirField(false, 'Reference'),
    'verificationDate': const FhirField(false, 'FhirDateTime'),
    '_verificationDate': const FhirField(false, 'Element'),
  },
  'ConsentProvision': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'actor': const FhirField(true, 'ConsentActor'),
    'action': const FhirField(true, 'CodeableConcept'),
    'securityLabel': const FhirField(true, 'Coding'),
    'purpose': const FhirField(true, 'Coding'),
    'class': const FhirField(true, 'Coding'),
    'code': const FhirField(true, 'CodeableConcept'),
    'dataPeriod': const FhirField(false, 'Period'),
    'data': const FhirField(true, 'ConsentData'),
    'provision': const FhirField(true, 'ConsentProvision'),
  },
  'ConsentActor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'ConsentData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'meaning': const FhirField(false, 'Code'),
    '_meaning': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'Reference'),
  },
  'Provenance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(true, 'Reference'),
    'occurredPeriod': const FhirField(false, 'Period'),
    'occurredDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurredDateTime': const FhirField(false, 'Element'),
    'recorded': const FhirField(false, 'Instant'),
    '_recorded': const FhirField(false, 'Element'),
    'policy': const FhirField(true, 'FhirUri'),
    '_policy': const FhirField(true, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'agent': const FhirField(true, 'ProvenanceAgent'),
    'entity': const FhirField(true, 'ProvenanceEntity'),
    'signature': const FhirField(true, 'Signature'),
  },
  'ProvenanceAgent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'role': const FhirField(true, 'CodeableConcept'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
  },
  'ProvenanceEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'Code'),
    '_role': const FhirField(false, 'Element'),
    'what': const FhirField(false, 'Reference'),
    'agent': const FhirField(true, 'ProvenanceAgent'),
  },
  'Account': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'subject': const FhirField(true, 'Reference'),
    'servicePeriod': const FhirField(false, 'Period'),
    'coverage': const FhirField(true, 'AccountCoverage'),
    'owner': const FhirField(false, 'Reference'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'guarantor': const FhirField(true, 'AccountGuarantor'),
    'partOf': const FhirField(false, 'Reference'),
  },
  'AccountCoverage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'coverage': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'PositiveInt'),
    '_priority': const FhirField(false, 'Element'),
  },
  'AccountGuarantor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'party': const FhirField(false, 'Reference'),
    'onHold': const FhirField(false, 'Boolean'),
    '_onHold': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'ChargeItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'definitionUri': const FhirField(true, 'FhirUri'),
    '_definitionUri': const FhirField(true, 'Element'),
    'definitionCanonical': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'partOf': const FhirField(true, 'Reference'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'context': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'performer': const FhirField(true, 'ChargeItemPerformer'),
    'performingOrganization': const FhirField(false, 'Reference'),
    'requestingOrganization': const FhirField(false, 'Reference'),
    'costCenter': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'bodysite': const FhirField(true, 'CodeableConcept'),
    'factorOverride': const FhirField(false, 'Decimal'),
    '_factorOverride': const FhirField(false, 'Element'),
    'priceOverride': const FhirField(false, 'Money'),
    'overrideReason': const FhirField(false, 'String'),
    '_overrideReason': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'enteredDate': const FhirField(false, 'FhirDateTime'),
    '_enteredDate': const FhirField(false, 'Element'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'service': const FhirField(true, 'Reference'),
    'productReference': const FhirField(false, 'Reference'),
    'productCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'account': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'supportingInformation': const FhirField(true, 'Reference'),
  },
  'ChargeItemPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ChargeItemDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'derivedFromUri': const FhirField(true, 'FhirUri'),
    '_derivedFromUri': const FhirField(true, 'Element'),
    'partOf': const FhirField(true, 'Canonical'),
    'replaces': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'code': const FhirField(false, 'CodeableConcept'),
    'instance': const FhirField(true, 'Reference'),
    'applicability': const FhirField(true, 'ChargeItemDefinitionApplicability'),
    'propertyGroup': const FhirField(true, 'ChargeItemDefinitionPropertyGroup'),
  },
  'ChargeItemDefinitionApplicability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'String'),
    '_language': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
  },
  'ChargeItemDefinitionPropertyGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'applicability': const FhirField(true, 'ChargeItemDefinitionApplicability'),
    'priceComponent':
        const FhirField(true, 'ChargeItemDefinitionPriceComponent'),
  },
  'ChargeItemDefinitionPriceComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
  },
  'Contract': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'legalState': const FhirField(false, 'CodeableConcept'),
    'instantiatesCanonical': const FhirField(false, 'Reference'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    '_instantiatesUri': const FhirField(false, 'Element'),
    'contentDerivative': const FhirField(false, 'CodeableConcept'),
    'issued': const FhirField(false, 'FhirDateTime'),
    '_issued': const FhirField(false, 'Element'),
    'applies': const FhirField(false, 'Period'),
    'expirationType': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'authority': const FhirField(true, 'Reference'),
    'domain': const FhirField(true, 'Reference'),
    'site': const FhirField(true, 'Reference'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    '_alias': const FhirField(true, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'scope': const FhirField(false, 'CodeableConcept'),
    'topicCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'topicReference': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(true, 'CodeableConcept'),
    'contentDefinition': const FhirField(false, 'ContractContentDefinition'),
    'term': const FhirField(true, 'ContractTerm'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'relevantHistory': const FhirField(true, 'Reference'),
    'signer': const FhirField(true, 'ContractSigner'),
    'friendly': const FhirField(true, 'ContractFriendly'),
    'legal': const FhirField(true, 'ContractLegal'),
    'rule': const FhirField(true, 'ContractRule'),
    'legallyBindingAttachment': const FhirField(false, 'Attachment'),
    'legallyBindingReference': const FhirField(false, 'Reference'),
  },
  'ContractContentDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'publisher': const FhirField(false, 'Reference'),
    'publicationDate': const FhirField(false, 'FhirDateTime'),
    '_publicationDate': const FhirField(false, 'Element'),
    'publicationStatus': const FhirField(false, 'Code'),
    '_publicationStatus': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
  },
  'ContractTerm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'issued': const FhirField(false, 'FhirDateTime'),
    '_issued': const FhirField(false, 'Element'),
    'applies': const FhirField(false, 'Period'),
    'topicCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'topicReference': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'securityLabel': const FhirField(true, 'ContractSecurityLabel'),
    'offer': const FhirField(false, 'ContractOffer'),
    'asset': const FhirField(true, 'ContractAsset'),
    'action': const FhirField(true, 'ContractAction'),
    'group': const FhirField(true, 'ContractTerm'),
  },
  'ContractSecurityLabel': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(true, 'UnsignedInt'),
    '_number': const FhirField(true, 'Element'),
    'classification': const FhirField(false, 'Coding'),
    'category': const FhirField(true, 'Coding'),
    'control': const FhirField(true, 'Coding'),
  },
  'ContractOffer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'party': const FhirField(true, 'ContractParty'),
    'topic': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'decisionMode': const FhirField(true, 'CodeableConcept'),
    'answer': const FhirField(true, 'ContractAnswer'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'linkId': const FhirField(true, 'String'),
    '_linkId': const FhirField(true, 'Element'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    '_securityLabelNumber': const FhirField(true, 'Element'),
  },
  'ContractParty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'ContractAnswer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'ContractAsset': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'scope': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'typeReference': const FhirField(true, 'Reference'),
    'subtype': const FhirField(true, 'CodeableConcept'),
    'relationship': const FhirField(false, 'Coding'),
    'context': const FhirField(true, 'ContractContext'),
    'condition': const FhirField(false, 'String'),
    '_condition': const FhirField(false, 'Element'),
    'periodType': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(true, 'Period'),
    'usePeriod': const FhirField(true, 'Period'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'linkId': const FhirField(true, 'String'),
    '_linkId': const FhirField(true, 'Element'),
    'answer': const FhirField(true, 'ContractAnswer'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    '_securityLabelNumber': const FhirField(true, 'Element'),
    'valuedItem': const FhirField(true, 'ContractValuedItem'),
  },
  'ContractContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
  },
  'ContractValuedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'entityCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'entityReference': const FhirField(false, 'Reference'),
    'identifier': const FhirField(false, 'Identifier'),
    'effectiveTime': const FhirField(false, 'FhirDateTime'),
    '_effectiveTime': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'points': const FhirField(false, 'Decimal'),
    '_points': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'payment': const FhirField(false, 'String'),
    '_payment': const FhirField(false, 'Element'),
    'paymentDate': const FhirField(false, 'FhirDateTime'),
    '_paymentDate': const FhirField(false, 'Element'),
    'responsible': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'linkId': const FhirField(true, 'String'),
    '_linkId': const FhirField(true, 'Element'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    '_securityLabelNumber': const FhirField(true, 'Element'),
  },
  'ContractAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(true, 'ContractSubject'),
    'intent': const FhirField(false, 'CodeableConcept'),
    'linkId': const FhirField(true, 'String'),
    '_linkId': const FhirField(true, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'context': const FhirField(false, 'Reference'),
    'contextLinkId': const FhirField(true, 'String'),
    '_contextLinkId': const FhirField(true, 'Element'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    '_occurrenceDateTime': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'requester': const FhirField(true, 'Reference'),
    'requesterLinkId': const FhirField(true, 'String'),
    '_requesterLinkId': const FhirField(true, 'Element'),
    'performerType': const FhirField(true, 'CodeableConcept'),
    'performerRole': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(false, 'Reference'),
    'performerLinkId': const FhirField(true, 'String'),
    '_performerLinkId': const FhirField(true, 'Element'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'String'),
    '_reason': const FhirField(true, 'Element'),
    'reasonLinkId': const FhirField(true, 'String'),
    '_reasonLinkId': const FhirField(true, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    '_securityLabelNumber': const FhirField(true, 'Element'),
  },
  'ContractSubject': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'ContractSigner': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'party': const FhirField(false, 'Reference'),
    'signature': const FhirField(true, 'Signature'),
  },
  'ContractFriendly': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'ContractLegal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'ContractRule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'ExplanationOfBenefit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'billablePeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'fundsReserveRequested': const FhirField(false, 'CodeableConcept'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'related': const FhirField(true, 'ExplanationOfBenefitRelated'),
    'prescription': const FhirField(false, 'Reference'),
    'originalPrescription': const FhirField(false, 'Reference'),
    'payee': const FhirField(false, 'ExplanationOfBenefitPayee'),
    'referral': const FhirField(false, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'claim': const FhirField(false, 'Reference'),
    'claimResponse': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    '_disposition': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(true, 'String'),
    '_preAuthRef': const FhirField(true, 'Element'),
    'preAuthRefPeriod': const FhirField(true, 'Period'),
    'careTeam': const FhirField(true, 'ExplanationOfBenefitCareTeam'),
    'supportingInfo':
        const FhirField(true, 'ExplanationOfBenefitSupportingInfo'),
    'diagnosis': const FhirField(true, 'ExplanationOfBenefitDiagnosis'),
    'procedure': const FhirField(true, 'ExplanationOfBenefitProcedure'),
    'precedence': const FhirField(false, 'PositiveInt'),
    '_precedence': const FhirField(false, 'Element'),
    'insurance': const FhirField(true, 'ExplanationOfBenefitInsurance'),
    'accident': const FhirField(false, 'ExplanationOfBenefitAccident'),
    'item': const FhirField(true, 'ExplanationOfBenefitItem'),
    'addItem': const FhirField(true, 'ExplanationOfBenefitAddItem'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'total': const FhirField(true, 'ExplanationOfBenefitTotal'),
    'payment': const FhirField(false, 'ExplanationOfBenefitPayment'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'form': const FhirField(false, 'Attachment'),
    'processNote': const FhirField(true, 'ExplanationOfBenefitProcessNote'),
    'benefitPeriod': const FhirField(false, 'Period'),
    'benefitBalance':
        const FhirField(true, 'ExplanationOfBenefitBenefitBalance'),
  },
  'ExplanationOfBenefitRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'claim': const FhirField(false, 'Reference'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Identifier'),
  },
  'ExplanationOfBenefitPayee': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'party': const FhirField(false, 'Reference'),
  },
  'ExplanationOfBenefitCareTeam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'provider': const FhirField(false, 'Reference'),
    'responsible': const FhirField(false, 'Boolean'),
    '_responsible': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
    'qualification': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'timingDate': const FhirField(false, 'Date'),
    '_timingDate': const FhirField(false, 'Element'),
    'timingPeriod': const FhirField(false, 'Period'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'Coding'),
  },
  'ExplanationOfBenefitDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'onAdmission': const FhirField(false, 'CodeableConcept'),
    'packageCode': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'procedureCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'procedureReference': const FhirField(false, 'Reference'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ExplanationOfBenefitInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'focal': const FhirField(false, 'Boolean'),
    '_focal': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'preAuthRef': const FhirField(true, 'String'),
    '_preAuthRef': const FhirField(true, 'Element'),
  },
  'ExplanationOfBenefitAccident': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
  },
  'ExplanationOfBenefitItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'careTeamSequence': const FhirField(true, 'PositiveInt'),
    '_careTeamSequence': const FhirField(true, 'Element'),
    'diagnosisSequence': const FhirField(true, 'PositiveInt'),
    '_diagnosisSequence': const FhirField(true, 'Element'),
    'procedureSequence': const FhirField(true, 'PositiveInt'),
    '_procedureSequence': const FhirField(true, 'Element'),
    'informationSequence': const FhirField(true, 'PositiveInt'),
    '_informationSequence': const FhirField(true, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'subSite': const FhirField(true, 'CodeableConcept'),
    'encounter': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'detail': const FhirField(true, 'ExplanationOfBenefitDetail'),
  },
  'ExplanationOfBenefitAdjudication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
  },
  'ExplanationOfBenefitDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'subDetail': const FhirField(true, 'ExplanationOfBenefitSubDetail'),
  },
  'ExplanationOfBenefitSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
  },
  'ExplanationOfBenefitAddItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(true, 'PositiveInt'),
    '_itemSequence': const FhirField(true, 'Element'),
    'detailSequence': const FhirField(true, 'PositiveInt'),
    '_detailSequence': const FhirField(true, 'Element'),
    'subDetailSequence': const FhirField(true, 'PositiveInt'),
    '_subDetailSequence': const FhirField(true, 'Element'),
    'provider': const FhirField(true, 'Reference'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'subSite': const FhirField(true, 'CodeableConcept'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'detail': const FhirField(true, 'ExplanationOfBenefitDetail1'),
  },
  'ExplanationOfBenefitDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'subDetail': const FhirField(true, 'ExplanationOfBenefitSubDetail1'),
  },
  'ExplanationOfBenefitSubDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
  },
  'ExplanationOfBenefitTotal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
  },
  'ExplanationOfBenefitPayment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'adjustment': const FhirField(false, 'Money'),
    'adjustmentReason': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
    'identifier': const FhirField(false, 'Identifier'),
  },
  'ExplanationOfBenefitProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'PositiveInt'),
    '_number': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitBenefitBalance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'excluded': const FhirField(false, 'Boolean'),
    '_excluded': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'financial': const FhirField(true, 'ExplanationOfBenefitFinancial'),
  },
  'ExplanationOfBenefitFinancial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_allowedUnsignedInt': const FhirField(false, 'Element'),
    'allowedString': const FhirField(false, 'String'),
    '_allowedString': const FhirField(false, 'Element'),
    'allowedMoney': const FhirField(false, 'Money'),
    'usedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_usedUnsignedInt': const FhirField(false, 'Element'),
    'usedMoney': const FhirField(false, 'Money'),
  },
  'InsurancePlan': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    '_alias': const FhirField(true, 'Element'),
    'period': const FhirField(false, 'Period'),
    'ownedBy': const FhirField(false, 'Reference'),
    'administeredBy': const FhirField(false, 'Reference'),
    'coverageArea': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'InsurancePlanContact'),
    'endpoint': const FhirField(true, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'coverage': const FhirField(true, 'InsurancePlanCoverage'),
    'plan': const FhirField(true, 'InsurancePlanPlan'),
  },
  'InsurancePlanContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'purpose': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
  },
  'InsurancePlanCoverage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'network': const FhirField(true, 'Reference'),
    'benefit': const FhirField(true, 'InsurancePlanBenefit'),
  },
  'InsurancePlanBenefit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'requirement': const FhirField(false, 'String'),
    '_requirement': const FhirField(false, 'Element'),
    'limit': const FhirField(true, 'InsurancePlanLimit'),
  },
  'InsurancePlanLimit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Quantity'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'InsurancePlanPlan': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'coverageArea': const FhirField(true, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'generalCost': const FhirField(true, 'InsurancePlanGeneralCost'),
    'specificCost': const FhirField(true, 'InsurancePlanSpecificCost'),
  },
  'InsurancePlanGeneralCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupSize': const FhirField(false, 'PositiveInt'),
    '_groupSize': const FhirField(false, 'Element'),
    'cost': const FhirField(false, 'Money'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'InsurancePlanSpecificCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'benefit': const FhirField(true, 'InsurancePlanBenefit1'),
  },
  'InsurancePlanBenefit1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'cost': const FhirField(true, 'InsurancePlanCost'),
  },
  'InsurancePlanCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'applicability': const FhirField(false, 'CodeableConcept'),
    'qualifiers': const FhirField(true, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
  },
  'PaymentNotice': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'Reference'),
    'response': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'provider': const FhirField(false, 'Reference'),
    'payment': const FhirField(false, 'Reference'),
    'paymentDate': const FhirField(false, 'Date'),
    '_paymentDate': const FhirField(false, 'Element'),
    'payee': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'amount': const FhirField(false, 'Money'),
    'paymentStatus': const FhirField(false, 'CodeableConcept'),
  },
  'PaymentReconciliation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'paymentIssuer': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'requestor': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    '_disposition': const FhirField(false, 'Element'),
    'paymentDate': const FhirField(false, 'Date'),
    '_paymentDate': const FhirField(false, 'Element'),
    'paymentAmount': const FhirField(false, 'Money'),
    'paymentIdentifier': const FhirField(false, 'Identifier'),
    'detail': const FhirField(true, 'PaymentReconciliationDetail'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'processNote': const FhirField(true, 'PaymentReconciliationProcessNote'),
  },
  'PaymentReconciliationDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'predecessor': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'request': const FhirField(false, 'Reference'),
    'submitter': const FhirField(false, 'Reference'),
    'response': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'responsible': const FhirField(false, 'Reference'),
    'payee': const FhirField(false, 'Reference'),
    'amount': const FhirField(false, 'Money'),
  },
  'PaymentReconciliationProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
  },
  'Claim': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'billablePeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'related': const FhirField(true, 'ClaimRelated'),
    'prescription': const FhirField(false, 'Reference'),
    'originalPrescription': const FhirField(false, 'Reference'),
    'payee': const FhirField(false, 'ClaimPayee'),
    'referral': const FhirField(false, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'careTeam': const FhirField(true, 'ClaimCareTeam'),
    'supportingInfo': const FhirField(true, 'ClaimSupportingInfo'),
    'diagnosis': const FhirField(true, 'ClaimDiagnosis'),
    'procedure': const FhirField(true, 'ClaimProcedure'),
    'insurance': const FhirField(true, 'ClaimInsurance'),
    'accident': const FhirField(false, 'ClaimAccident'),
    'item': const FhirField(true, 'ClaimItem'),
    'total': const FhirField(false, 'Money'),
  },
  'ClaimRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'claim': const FhirField(false, 'Reference'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Identifier'),
  },
  'ClaimPayee': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'party': const FhirField(false, 'Reference'),
  },
  'ClaimCareTeam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'provider': const FhirField(false, 'Reference'),
    'responsible': const FhirField(false, 'Boolean'),
    '_responsible': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
    'qualification': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'timingDate': const FhirField(false, 'Date'),
    '_timingDate': const FhirField(false, 'Element'),
    'timingPeriod': const FhirField(false, 'Period'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'onAdmission': const FhirField(false, 'CodeableConcept'),
    'packageCode': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'procedureCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'procedureReference': const FhirField(false, 'Reference'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ClaimInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'focal': const FhirField(false, 'Boolean'),
    '_focal': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    '_businessArrangement': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(true, 'String'),
    '_preAuthRef': const FhirField(true, 'Element'),
    'claimResponse': const FhirField(false, 'Reference'),
  },
  'ClaimAccident': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
  },
  'ClaimItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'careTeamSequence': const FhirField(true, 'PositiveInt'),
    '_careTeamSequence': const FhirField(true, 'Element'),
    'diagnosisSequence': const FhirField(true, 'PositiveInt'),
    '_diagnosisSequence': const FhirField(true, 'Element'),
    'procedureSequence': const FhirField(true, 'PositiveInt'),
    '_procedureSequence': const FhirField(true, 'Element'),
    'informationSequence': const FhirField(true, 'PositiveInt'),
    '_informationSequence': const FhirField(true, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'subSite': const FhirField(true, 'CodeableConcept'),
    'encounter': const FhirField(true, 'Reference'),
    'detail': const FhirField(true, 'ClaimDetail'),
  },
  'ClaimDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'subDetail': const FhirField(true, 'ClaimSubDetail'),
  },
  'ClaimSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ClaimResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'requestor': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    '_disposition': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(false, 'String'),
    '_preAuthRef': const FhirField(false, 'Element'),
    'preAuthPeriod': const FhirField(false, 'Period'),
    'payeeType': const FhirField(false, 'CodeableConcept'),
    'item': const FhirField(true, 'ClaimResponseItem'),
    'addItem': const FhirField(true, 'ClaimResponseAddItem'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'total': const FhirField(true, 'ClaimResponseTotal'),
    'payment': const FhirField(false, 'ClaimResponsePayment'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'form': const FhirField(false, 'Attachment'),
    'processNote': const FhirField(true, 'ClaimResponseProcessNote'),
    'communicationRequest': const FhirField(true, 'Reference'),
    'insurance': const FhirField(true, 'ClaimResponseInsurance'),
    'error': const FhirField(true, 'ClaimResponseError'),
  },
  'ClaimResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(false, 'PositiveInt'),
    '_itemSequence': const FhirField(false, 'Element'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'detail': const FhirField(true, 'ClaimResponseDetail'),
  },
  'ClaimResponseAdjudication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
  },
  'ClaimResponseDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'detailSequence': const FhirField(false, 'PositiveInt'),
    '_detailSequence': const FhirField(false, 'Element'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'subDetail': const FhirField(true, 'ClaimResponseSubDetail'),
  },
  'ClaimResponseSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'subDetailSequence': const FhirField(false, 'PositiveInt'),
    '_subDetailSequence': const FhirField(false, 'Element'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
  },
  'ClaimResponseAddItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(true, 'PositiveInt'),
    '_itemSequence': const FhirField(true, 'Element'),
    'detailSequence': const FhirField(true, 'PositiveInt'),
    '_detailSequence': const FhirField(true, 'Element'),
    'subdetailSequence': const FhirField(true, 'PositiveInt'),
    '_subdetailSequence': const FhirField(true, 'Element'),
    'provider': const FhirField(true, 'Reference'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'subSite': const FhirField(true, 'CodeableConcept'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'detail': const FhirField(true, 'ClaimResponseDetail1'),
  },
  'ClaimResponseDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'subDetail': const FhirField(true, 'ClaimResponseSubDetail1'),
  },
  'ClaimResponseSubDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    '_noteNumber': const FhirField(true, 'Element'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
  },
  'ClaimResponseTotal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
  },
  'ClaimResponsePayment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'adjustment': const FhirField(false, 'Money'),
    'adjustmentReason': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
    'identifier': const FhirField(false, 'Identifier'),
  },
  'ClaimResponseProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'PositiveInt'),
    '_number': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimResponseInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'focal': const FhirField(false, 'Boolean'),
    '_focal': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    '_businessArrangement': const FhirField(false, 'Element'),
    'claimResponse': const FhirField(false, 'Reference'),
  },
  'ClaimResponseError': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(false, 'PositiveInt'),
    '_itemSequence': const FhirField(false, 'Element'),
    'detailSequence': const FhirField(false, 'PositiveInt'),
    '_detailSequence': const FhirField(false, 'Element'),
    'subDetailSequence': const FhirField(false, 'PositiveInt'),
    '_subDetailSequence': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'Invoice': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'cancelledReason': const FhirField(false, 'String'),
    '_cancelledReason': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'InvoiceParticipant'),
    'issuer': const FhirField(false, 'Reference'),
    'account': const FhirField(false, 'Reference'),
    'lineItem': const FhirField(true, 'InvoiceLineItem'),
    'totalPriceComponent': const FhirField(true, 'InvoicePriceComponent'),
    'totalNet': const FhirField(false, 'Money'),
    'totalGross': const FhirField(false, 'Money'),
    'paymentTerms': const FhirField(false, 'Markdown'),
    '_paymentTerms': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'InvoiceParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'InvoiceLineItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'chargeItemReference': const FhirField(false, 'Reference'),
    'chargeItemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'priceComponent': const FhirField(true, 'InvoicePriceComponent'),
  },
  'InvoicePriceComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
  },
  'Coverage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'policyHolder': const FhirField(false, 'Reference'),
    'subscriber': const FhirField(false, 'Reference'),
    'subscriberId': const FhirField(false, 'String'),
    '_subscriberId': const FhirField(false, 'Element'),
    'beneficiary': const FhirField(false, 'Reference'),
    'dependent': const FhirField(false, 'String'),
    '_dependent': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'payor': const FhirField(true, 'Reference'),
    'class': const FhirField(true, 'CoverageClass'),
    'order': const FhirField(false, 'PositiveInt'),
    '_order': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'String'),
    '_network': const FhirField(false, 'Element'),
    'costToBeneficiary': const FhirField(true, 'CoverageCostToBeneficiary'),
    'subrogation': const FhirField(false, 'Boolean'),
    '_subrogation': const FhirField(false, 'Element'),
    'contract': const FhirField(true, 'Reference'),
  },
  'CoverageClass': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
  },
  'CoverageCostToBeneficiary': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueMoney': const FhirField(false, 'Money'),
    'exception': const FhirField(true, 'CoverageException'),
  },
  'CoverageException': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
  },
  'CoverageEligibilityRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'purpose': const FhirField(true, 'Code'),
    '_purpose': const FhirField(true, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'supportingInfo':
        const FhirField(true, 'CoverageEligibilityRequestSupportingInfo'),
    'insurance': const FhirField(true, 'CoverageEligibilityRequestInsurance'),
    'item': const FhirField(true, 'CoverageEligibilityRequestItem'),
  },
  'CoverageEligibilityRequestSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    '_sequence': const FhirField(false, 'Element'),
    'information': const FhirField(false, 'Reference'),
    'appliesToAll': const FhirField(false, 'Boolean'),
    '_appliesToAll': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityRequestInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'focal': const FhirField(false, 'Boolean'),
    '_focal': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    '_businessArrangement': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityRequestItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'supportingInfoSequence': const FhirField(true, 'PositiveInt'),
    '_supportingInfoSequence': const FhirField(true, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'provider': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'facility': const FhirField(false, 'Reference'),
    'diagnosis': const FhirField(true, 'CoverageEligibilityRequestDiagnosis'),
    'detail': const FhirField(true, 'Reference'),
  },
  'CoverageEligibilityRequestDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
  },
  'CoverageEligibilityResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'purpose': const FhirField(true, 'Code'),
    '_purpose': const FhirField(true, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'servicedDate': const FhirField(false, 'Date'),
    '_servicedDate': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'requestor': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    '_disposition': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'insurance': const FhirField(true, 'CoverageEligibilityResponseInsurance'),
    'preAuthRef': const FhirField(false, 'String'),
    '_preAuthRef': const FhirField(false, 'Element'),
    'form': const FhirField(false, 'CodeableConcept'),
    'error': const FhirField(true, 'CoverageEligibilityResponseError'),
  },
  'CoverageEligibilityResponseInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'coverage': const FhirField(false, 'Reference'),
    'inforce': const FhirField(false, 'Boolean'),
    '_inforce': const FhirField(false, 'Element'),
    'benefitPeriod': const FhirField(false, 'Period'),
    'item': const FhirField(true, 'CoverageEligibilityResponseItem'),
  },
  'CoverageEligibilityResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'provider': const FhirField(false, 'Reference'),
    'excluded': const FhirField(false, 'Boolean'),
    '_excluded': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'benefit': const FhirField(true, 'CoverageEligibilityResponseBenefit'),
    'authorizationRequired': const FhirField(false, 'Boolean'),
    '_authorizationRequired': const FhirField(false, 'Element'),
    'authorizationSupporting': const FhirField(true, 'CodeableConcept'),
    'authorizationUrl': const FhirField(false, 'FhirUri'),
    '_authorizationUrl': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityResponseBenefit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_allowedUnsignedInt': const FhirField(false, 'Element'),
    'allowedString': const FhirField(false, 'String'),
    '_allowedString': const FhirField(false, 'Element'),
    'allowedMoney': const FhirField(false, 'Money'),
    'usedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_usedUnsignedInt': const FhirField(false, 'Element'),
    'usedString': const FhirField(false, 'String'),
    '_usedString': const FhirField(false, 'Element'),
    'usedMoney': const FhirField(false, 'Money'),
  },
  'CoverageEligibilityResponseError': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'EnrollmentRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'candidate': const FhirField(false, 'Reference'),
    'coverage': const FhirField(false, 'Reference'),
  },
  'EnrollmentResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    '_outcome': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    '_disposition': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'organization': const FhirField(false, 'Reference'),
    'requestProvider': const FhirField(false, 'Reference'),
  },
  'Endpoint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'connectionType': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'period': const FhirField(false, 'Period'),
    'payloadType': const FhirField(true, 'CodeableConcept'),
    'payloadMimeType': const FhirField(true, 'Code'),
    '_payloadMimeType': const FhirField(true, 'Element'),
    'address': const FhirField(false, 'FhirUrl'),
    '_address': const FhirField(false, 'Element'),
    'header': const FhirField(true, 'String'),
    '_header': const FhirField(true, 'Element'),
  },
  'HealthcareService': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'providedBy': const FhirField(false, 'Reference'),
    'category': const FhirField(true, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
    'extraDetails': const FhirField(false, 'Markdown'),
    '_extraDetails': const FhirField(false, 'Element'),
    'photo': const FhirField(false, 'Attachment'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'coverageArea': const FhirField(true, 'Reference'),
    'serviceProvisionCode': const FhirField(true, 'CodeableConcept'),
    'eligibility': const FhirField(true, 'HealthcareServiceEligibility'),
    'program': const FhirField(true, 'CodeableConcept'),
    'characteristic': const FhirField(true, 'CodeableConcept'),
    'communication': const FhirField(true, 'CodeableConcept'),
    'referralMethod': const FhirField(true, 'CodeableConcept'),
    'appointmentRequired': const FhirField(false, 'Boolean'),
    '_appointmentRequired': const FhirField(false, 'Element'),
    'availableTime': const FhirField(true, 'HealthcareServiceAvailableTime'),
    'notAvailable': const FhirField(true, 'HealthcareServiceNotAvailable'),
    'availabilityExceptions': const FhirField(false, 'String'),
    '_availabilityExceptions': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'HealthcareServiceEligibility': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'comment': const FhirField(false, 'Markdown'),
    '_comment': const FhirField(false, 'Element'),
  },
  'HealthcareServiceAvailableTime': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'daysOfWeek': const FhirField(true, 'Code'),
    '_daysOfWeek': const FhirField(true, 'Element'),
    'allDay': const FhirField(false, 'Boolean'),
    '_allDay': const FhirField(false, 'Element'),
    'availableStartTime': const FhirField(false, 'Time'),
    '_availableStartTime': const FhirField(false, 'Element'),
    'availableEndTime': const FhirField(false, 'Time'),
    '_availableEndTime': const FhirField(false, 'Element'),
  },
  'HealthcareServiceNotAvailable': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'during': const FhirField(false, 'Period'),
  },
  'Location': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'operationalStatus': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    '_alias': const FhirField(true, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
    'physicalType': const FhirField(false, 'CodeableConcept'),
    'position': const FhirField(false, 'LocationPosition'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'partOf': const FhirField(false, 'Reference'),
    'hoursOfOperation': const FhirField(true, 'LocationHoursOfOperation'),
    'availabilityExceptions': const FhirField(false, 'String'),
    '_availabilityExceptions': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'LocationPosition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'longitude': const FhirField(false, 'Decimal'),
    '_longitude': const FhirField(false, 'Element'),
    'latitude': const FhirField(false, 'Decimal'),
    '_latitude': const FhirField(false, 'Element'),
    'altitude': const FhirField(false, 'Decimal'),
    '_altitude': const FhirField(false, 'Element'),
  },
  'LocationHoursOfOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'daysOfWeek': const FhirField(true, 'Code'),
    '_daysOfWeek': const FhirField(true, 'Element'),
    'allDay': const FhirField(false, 'Boolean'),
    '_allDay': const FhirField(false, 'Element'),
    'openingTime': const FhirField(false, 'Time'),
    '_openingTime': const FhirField(false, 'Element'),
    'closingTime': const FhirField(false, 'Time'),
    '_closingTime': const FhirField(false, 'Element'),
  },
  'Organization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    '_alias': const FhirField(true, 'Element'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(true, 'Address'),
    'partOf': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'OrganizationContact'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'OrganizationContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'purpose': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
  },
  'OrganizationAffiliation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'organization': const FhirField(false, 'Reference'),
    'participatingOrganization': const FhirField(false, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'healthcareService': const FhirField(true, 'Reference'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'BiologicallyDerivedProduct': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'productCategory': const FhirField(false, 'Code'),
    '_productCategory': const FhirField(false, 'Element'),
    'productCode': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'request': const FhirField(true, 'Reference'),
    'quantity': const FhirField(false, 'Integer'),
    '_quantity': const FhirField(false, 'Element'),
    'parent': const FhirField(true, 'Reference'),
    'collection':
        const FhirField(false, 'BiologicallyDerivedProductCollection'),
    'processing': const FhirField(true, 'BiologicallyDerivedProductProcessing'),
    'manipulation':
        const FhirField(false, 'BiologicallyDerivedProductManipulation'),
    'storage': const FhirField(true, 'BiologicallyDerivedProductStorage'),
  },
  'BiologicallyDerivedProductCollection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'collector': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'Reference'),
    'collectedDateTime': const FhirField(false, 'FhirDateTime'),
    '_collectedDateTime': const FhirField(false, 'Element'),
    'collectedPeriod': const FhirField(false, 'Period'),
  },
  'BiologicallyDerivedProductProcessing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'procedure': const FhirField(false, 'CodeableConcept'),
    'additive': const FhirField(false, 'Reference'),
    'timeDateTime': const FhirField(false, 'FhirDateTime'),
    '_timeDateTime': const FhirField(false, 'Element'),
    'timePeriod': const FhirField(false, 'Period'),
  },
  'BiologicallyDerivedProductManipulation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'timeDateTime': const FhirField(false, 'FhirDateTime'),
    '_timeDateTime': const FhirField(false, 'Element'),
    'timePeriod': const FhirField(false, 'Period'),
  },
  'BiologicallyDerivedProductStorage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'temperature': const FhirField(false, 'Decimal'),
    '_temperature': const FhirField(false, 'Element'),
    'scale': const FhirField(false, 'Code'),
    '_scale': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Period'),
  },
  'Device': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'definition': const FhirField(false, 'Reference'),
    'udiCarrier': const FhirField(true, 'DeviceUdiCarrier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'distinctIdentifier': const FhirField(false, 'String'),
    '_distinctIdentifier': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'String'),
    '_manufacturer': const FhirField(false, 'Element'),
    'manufactureDate': const FhirField(false, 'FhirDateTime'),
    '_manufactureDate': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'FhirDateTime'),
    '_expirationDate': const FhirField(false, 'Element'),
    'lotNumber': const FhirField(false, 'String'),
    '_lotNumber': const FhirField(false, 'Element'),
    'serialNumber': const FhirField(false, 'String'),
    '_serialNumber': const FhirField(false, 'Element'),
    'deviceName': const FhirField(true, 'DeviceDeviceName'),
    'modelNumber': const FhirField(false, 'String'),
    '_modelNumber': const FhirField(false, 'Element'),
    'partNumber': const FhirField(false, 'String'),
    '_partNumber': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'specialization': const FhirField(true, 'DeviceSpecialization'),
    'version': const FhirField(true, 'DeviceVersion'),
    'property': const FhirField(true, 'DeviceProperty'),
    'patient': const FhirField(false, 'Reference'),
    'owner': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'location': const FhirField(false, 'Reference'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'safety': const FhirField(true, 'CodeableConcept'),
    'parent': const FhirField(false, 'Reference'),
  },
  'DeviceUdiCarrier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'deviceIdentifier': const FhirField(false, 'String'),
    '_deviceIdentifier': const FhirField(false, 'Element'),
    'issuer': const FhirField(false, 'FhirUri'),
    '_issuer': const FhirField(false, 'Element'),
    'jurisdiction': const FhirField(false, 'FhirUri'),
    '_jurisdiction': const FhirField(false, 'Element'),
    'carrierAIDC': const FhirField(false, 'Base64Binary'),
    '_carrierAIDC': const FhirField(false, 'Element'),
    'carrierHRF': const FhirField(false, 'String'),
    '_carrierHRF': const FhirField(false, 'Element'),
    'entryType': const FhirField(false, 'Code'),
    '_entryType': const FhirField(false, 'Element'),
  },
  'DeviceDeviceName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
  },
  'DeviceSpecialization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'systemType': const FhirField(false, 'CodeableConcept'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
  },
  'DeviceVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'component': const FhirField(false, 'Identifier'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'DeviceProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(true, 'Quantity'),
    'valueCode': const FhirField(true, 'CodeableConcept'),
  },
  'DeviceMetric': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'Reference'),
    'parent': const FhirField(false, 'Reference'),
    'operationalStatus': const FhirField(false, 'Code'),
    '_operationalStatus': const FhirField(false, 'Element'),
    'color': const FhirField(false, 'Code'),
    '_color': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'Code'),
    '_category': const FhirField(false, 'Element'),
    'measurementPeriod': const FhirField(false, 'Timing'),
    'calibration': const FhirField(true, 'DeviceMetricCalibration'),
  },
  'DeviceMetricCalibration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'state': const FhirField(false, 'Code'),
    '_state': const FhirField(false, 'Element'),
    'time': const FhirField(false, 'Instant'),
    '_time': const FhirField(false, 'Element'),
  },
  'NutritionProduct': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'manufacturer': const FhirField(true, 'Reference'),
    'nutrient': const FhirField(true, 'NutritionProductNutrient'),
    'ingredient': const FhirField(true, 'NutritionProductIngredient'),
    'knownAllergen': const FhirField(true, 'CodeableReference'),
    'productCharacteristic':
        const FhirField(true, 'NutritionProductCharacteristic'),
    'instance': const FhirField(true, 'NutritionProductInstance'),
    'note': const FhirField(true, 'Annotation'),
  },
  'NutritionProductNutrient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(true, 'Ratio'),
  },
  'NutritionProductIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(true, 'Ratio'),
  },
  'NutritionProductCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'Markdown'),
    '_valueString': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
  },
  'NutritionProductInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'lotNumber': const FhirField(false, 'String'),
    '_lotNumber': const FhirField(false, 'Element'),
    'expiry': const FhirField(false, 'FhirDateTime'),
    '_expiry': const FhirField(false, 'Element'),
    'useBy': const FhirField(false, 'FhirDateTime'),
    '_useBy': const FhirField(false, 'Element'),
    'biologicalSource': const FhirField(false, 'Identifier'),
  },
  'Substance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'instance': const FhirField(true, 'SubstanceInstance'),
    'ingredient': const FhirField(true, 'SubstanceIngredient'),
  },
  'SubstanceInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'expiry': const FhirField(false, 'FhirDateTime'),
    '_expiry': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
  },
  'SubstanceIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Ratio'),
    'substanceCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'substanceReference': const FhirField(false, 'Reference'),
  },
  'Encounter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusHistory': const FhirField(true, 'EncounterStatusHistory'),
    'class': const FhirField(false, 'Coding'),
    'classHistory': const FhirField(true, 'EncounterClassHistory'),
    'type': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(false, 'CodeableConcept'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'episodeOfCare': const FhirField(true, 'Reference'),
    'basedOn': const FhirField(true, 'Reference'),
    'participant': const FhirField(true, 'EncounterParticipant'),
    'appointment': const FhirField(true, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'length': const FhirField(false, 'FhirDuration'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'diagnosis': const FhirField(true, 'EncounterDiagnosis'),
    'account': const FhirField(true, 'Reference'),
    'hospitalization': const FhirField(false, 'EncounterHospitalization'),
    'location': const FhirField(true, 'EncounterLocation'),
    'serviceProvider': const FhirField(false, 'Reference'),
    'partOf': const FhirField(false, 'Reference'),
  },
  'EncounterStatusHistory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'EncounterClassHistory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'class': const FhirField(false, 'Coding'),
    'period': const FhirField(false, 'Period'),
  },
  'EncounterParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'individual': const FhirField(false, 'Reference'),
  },
  'EncounterDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'condition': const FhirField(false, 'Reference'),
    'use': const FhirField(false, 'CodeableConcept'),
    'rank': const FhirField(false, 'PositiveInt'),
    '_rank': const FhirField(false, 'Element'),
  },
  'EncounterHospitalization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'preAdmissionIdentifier': const FhirField(false, 'Identifier'),
    'origin': const FhirField(false, 'Reference'),
    'admitSource': const FhirField(false, 'CodeableConcept'),
    'reAdmission': const FhirField(false, 'CodeableConcept'),
    'dietPreference': const FhirField(true, 'CodeableConcept'),
    'specialCourtesy': const FhirField(true, 'CodeableConcept'),
    'specialArrangement': const FhirField(true, 'CodeableConcept'),
    'destination': const FhirField(false, 'Reference'),
    'dischargeDisposition': const FhirField(false, 'CodeableConcept'),
  },
  'EncounterLocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'location': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'physicalType': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
  },
  'EpisodeOfCare': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusHistory': const FhirField(true, 'EpisodeOfCareStatusHistory'),
    'type': const FhirField(true, 'CodeableConcept'),
    'diagnosis': const FhirField(true, 'EpisodeOfCareDiagnosis'),
    'patient': const FhirField(false, 'Reference'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'referralRequest': const FhirField(true, 'Reference'),
    'careManager': const FhirField(false, 'Reference'),
    'team': const FhirField(true, 'Reference'),
    'account': const FhirField(true, 'Reference'),
  },
  'EpisodeOfCareStatusHistory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'EpisodeOfCareDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'condition': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'rank': const FhirField(false, 'PositiveInt'),
    '_rank': const FhirField(false, 'Element'),
  },
  'Flag': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'encounter': const FhirField(false, 'Reference'),
    'author': const FhirField(false, 'Reference'),
  },
  'Library': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'parameter': const FhirField(true, 'ParameterDefinition'),
    'dataRequirement': const FhirField(true, 'DataRequirement'),
    'content': const FhirField(true, 'Attachment'),
  },
  'FhirList': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Reference'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'entry': const FhirField(true, 'FhirListEntry'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
  },
  'List': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Reference'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'entry': const FhirField(true, 'FhirListEntry'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
  },
  'FhirListEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'flag': const FhirField(false, 'CodeableConcept'),
    'deleted': const FhirField(false, 'Boolean'),
    '_deleted': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'item': const FhirField(false, 'Reference'),
  },
  'Appointment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'cancelationReason': const FhirField(false, 'CodeableConcept'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'appointmentType': const FhirField(false, 'CodeableConcept'),
    'reasonCode': const FhirField(true, 'CodeableConcept'),
    'reasonReference': const FhirField(true, 'Reference'),
    'priority': const FhirField(false, 'UnsignedInt'),
    '_priority': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'start': const FhirField(false, 'Instant'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    '_end': const FhirField(false, 'Element'),
    'minutesDuration': const FhirField(false, 'PositiveInt'),
    '_minutesDuration': const FhirField(false, 'Element'),
    'slot': const FhirField(true, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    '_created': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
    'patientInstruction': const FhirField(false, 'String'),
    '_patientInstruction': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'participant': const FhirField(true, 'AppointmentParticipant'),
    'requestedPeriod': const FhirField(true, 'Period'),
  },
  'AppointmentParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
    'required': const FhirField(false, 'Code'),
    '_required': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'AppointmentResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'appointment': const FhirField(false, 'Reference'),
    'start': const FhirField(false, 'Instant'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    '_end': const FhirField(false, 'Element'),
    'participantType': const FhirField(true, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
    'participantStatus': const FhirField(false, 'Code'),
    '_participantStatus': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'Schedule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'actor': const FhirField(true, 'Reference'),
    'planningHorizon': const FhirField(false, 'Period'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'Slot': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'appointmentType': const FhirField(false, 'CodeableConcept'),
    'schedule': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'start': const FhirField(false, 'Instant'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    '_end': const FhirField(false, 'Element'),
    'overbooked': const FhirField(false, 'Boolean'),
    '_overbooked': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    '_comment': const FhirField(false, 'Element'),
  },
  'Task': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    '_instantiatesUri': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'businessStatus': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'Reference'),
    'for': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'executionPeriod': const FhirField(false, 'Period'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    '_authoredOn': const FhirField(false, 'Element'),
    'lastModified': const FhirField(false, 'FhirDateTime'),
    '_lastModified': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performerType': const FhirField(true, 'CodeableConcept'),
    'owner': const FhirField(false, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'reasonCode': const FhirField(false, 'CodeableConcept'),
    'reasonReference': const FhirField(false, 'Reference'),
    'insurance': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'relevantHistory': const FhirField(true, 'Reference'),
    'restriction': const FhirField(false, 'TaskRestriction'),
    'input': const FhirField(true, 'TaskInput'),
    'output': const FhirField(true, 'TaskOutput'),
  },
  'TaskRestriction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'repetitions': const FhirField(false, 'PositiveInt'),
    '_repetitions': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'recipient': const FhirField(true, 'Reference'),
  },
  'TaskInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    '_valueCanonical': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    '_valueInstant': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    '_valueMarkdown': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Oid'),
    '_valueOid': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_valuePositiveInt': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_valueUnsignedInt': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    '_valueUrl': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Uuid'),
    '_valueUuid': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueContributor': const FhirField(false, 'Contributor'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'TaskOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    '_valueCanonical': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    '_valueInstant': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    '_valueMarkdown': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Oid'),
    '_valueOid': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_valuePositiveInt': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_valueUnsignedInt': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    '_valueUrl': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Uuid'),
    '_valueUuid': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueContributor': const FhirField(false, 'Contributor'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'VerificationResult': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(true, 'Reference'),
    'targetLocation': const FhirField(true, 'String'),
    '_targetLocation': const FhirField(true, 'Element'),
    'need': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    '_statusDate': const FhirField(false, 'Element'),
    'validationType': const FhirField(false, 'CodeableConcept'),
    'validationProcess': const FhirField(true, 'CodeableConcept'),
    'frequency': const FhirField(false, 'Timing'),
    'lastPerformed': const FhirField(false, 'FhirDateTime'),
    '_lastPerformed': const FhirField(false, 'Element'),
    'nextScheduled': const FhirField(false, 'Date'),
    '_nextScheduled': const FhirField(false, 'Element'),
    'failureAction': const FhirField(false, 'CodeableConcept'),
    'primarySource': const FhirField(true, 'VerificationResultPrimarySource'),
    'attestation': const FhirField(false, 'VerificationResultAttestation'),
    'validator': const FhirField(true, 'VerificationResultValidator'),
  },
  'VerificationResultPrimarySource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'who': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'communicationMethod': const FhirField(true, 'CodeableConcept'),
    'validationStatus': const FhirField(false, 'CodeableConcept'),
    'validationDate': const FhirField(false, 'FhirDateTime'),
    '_validationDate': const FhirField(false, 'Element'),
    'canPushUpdates': const FhirField(false, 'CodeableConcept'),
    'pushTypeAvailable': const FhirField(true, 'CodeableConcept'),
  },
  'VerificationResultAttestation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'communicationMethod': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'sourceIdentityCertificate': const FhirField(false, 'String'),
    '_sourceIdentityCertificate': const FhirField(false, 'Element'),
    'proxyIdentityCertificate': const FhirField(false, 'String'),
    '_proxyIdentityCertificate': const FhirField(false, 'Element'),
    'proxySignature': const FhirField(false, 'Signature'),
    'sourceSignature': const FhirField(false, 'Signature'),
  },
  'VerificationResultValidator': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'organization': const FhirField(false, 'Reference'),
    'identityCertificate': const FhirField(false, 'String'),
    '_identityCertificate': const FhirField(false, 'Element'),
    'attestationSignature': const FhirField(false, 'Signature'),
  },
  'Group': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'actual': const FhirField(false, 'Boolean'),
    '_actual': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'UnsignedInt'),
    '_quantity': const FhirField(false, 'Element'),
    'managingEntity': const FhirField(false, 'Reference'),
    'characteristic': const FhirField(true, 'GroupCharacteristic'),
    'member': const FhirField(true, 'GroupMember'),
  },
  'GroupCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
    'exclude': const FhirField(false, 'Boolean'),
    '_exclude': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'GroupMember': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'entity': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'inactive': const FhirField(false, 'Boolean'),
    '_inactive': const FhirField(false, 'Element'),
  },
  'Patient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    '_birthDate': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    '_deceasedBoolean': const FhirField(false, 'Element'),
    'deceasedDateTime': const FhirField(false, 'FhirDateTime'),
    '_deceasedDateTime': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'maritalStatus': const FhirField(false, 'CodeableConcept'),
    'multipleBirthBoolean': const FhirField(false, 'Boolean'),
    '_multipleBirthBoolean': const FhirField(false, 'Element'),
    'multipleBirthInteger': const FhirField(false, 'Integer'),
    '_multipleBirthInteger': const FhirField(false, 'Element'),
    'photo': const FhirField(true, 'Attachment'),
    'contact': const FhirField(true, 'PatientContact'),
    'communication': const FhirField(true, 'PatientCommunication'),
    'generalPractitioner': const FhirField(true, 'Reference'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'link': const FhirField(true, 'PatientLink'),
  },
  'PatientContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationship': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'organization': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'PatientCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    '_preferred': const FhirField(false, 'Element'),
  },
  'PatientLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'other': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
  },
  'Person': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    '_birthDate': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'photo': const FhirField(false, 'Attachment'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'link': const FhirField(true, 'PersonLink'),
  },
  'PersonLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(false, 'Reference'),
    'assurance': const FhirField(false, 'Code'),
    '_assurance': const FhirField(false, 'Element'),
  },
  'Practitioner': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(true, 'Address'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    '_birthDate': const FhirField(false, 'Element'),
    'photo': const FhirField(true, 'Attachment'),
    'qualification': const FhirField(true, 'PractitionerQualification'),
    'communication': const FhirField(true, 'CodeableConcept'),
  },
  'PractitionerQualification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'issuer': const FhirField(false, 'Reference'),
  },
  'PractitionerRole': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'practitioner': const FhirField(false, 'Reference'),
    'organization': const FhirField(false, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'healthcareService': const FhirField(true, 'Reference'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'availableTime': const FhirField(true, 'PractitionerRoleAvailableTime'),
    'notAvailable': const FhirField(true, 'PractitionerRoleNotAvailable'),
    'availabilityExceptions': const FhirField(false, 'String'),
    '_availabilityExceptions': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'PractitionerRoleAvailableTime': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'daysOfWeek': const FhirField(true, 'Code'),
    '_daysOfWeek': const FhirField(true, 'Element'),
    'allDay': const FhirField(false, 'Boolean'),
    '_allDay': const FhirField(false, 'Element'),
    'availableStartTime': const FhirField(false, 'Time'),
    '_availableStartTime': const FhirField(false, 'Element'),
    'availableEndTime': const FhirField(false, 'Time'),
    '_availableEndTime': const FhirField(false, 'Element'),
  },
  'PractitionerRoleNotAvailable': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'during': const FhirField(false, 'Period'),
  },
  'RelatedPerson': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    '_active': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'relationship': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    '_birthDate': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'photo': const FhirField(true, 'Attachment'),
    'period': const FhirField(false, 'Period'),
    'communication': const FhirField(true, 'RelatedPersonCommunication'),
  },
  'RelatedPersonCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    '_preferred': const FhirField(false, 'Element'),
  },
  'ResearchStudy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'protocol': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'primaryPurposeType': const FhirField(false, 'CodeableConcept'),
    'phase': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'focus': const FhirField(true, 'CodeableConcept'),
    'condition': const FhirField(true, 'CodeableConcept'),
    'contact': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'keyword': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'enrollment': const FhirField(true, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'sponsor': const FhirField(false, 'Reference'),
    'principalInvestigator': const FhirField(false, 'Reference'),
    'site': const FhirField(true, 'Reference'),
    'reasonStopped': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'arm': const FhirField(true, 'ResearchStudyArm'),
    'objective': const FhirField(true, 'ResearchStudyObjective'),
  },
  'ResearchStudyArm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
  },
  'ResearchStudyObjective': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'ResearchSubject': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'study': const FhirField(false, 'Reference'),
    'individual': const FhirField(false, 'Reference'),
    'assignedArm': const FhirField(false, 'String'),
    '_assignedArm': const FhirField(false, 'Element'),
    'actualArm': const FhirField(false, 'String'),
    '_actualArm': const FhirField(false, 'Element'),
    'consent': const FhirField(false, 'Reference'),
  },
  'AdministrableProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'formOf': const FhirField(true, 'Reference'),
    'administrableDoseForm': const FhirField(false, 'CodeableConcept'),
    'unitOfPresentation': const FhirField(false, 'CodeableConcept'),
    'producedFrom': const FhirField(true, 'Reference'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'device': const FhirField(false, 'Reference'),
    'property': const FhirField(true, 'AdministrableProductDefinitionProperty'),
    'routeOfAdministration': const FhirField(
        true, 'AdministrableProductDefinitionRouteOfAdministration'),
  },
  'AdministrableProductDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'status': const FhirField(false, 'CodeableConcept'),
  },
  'AdministrableProductDefinitionRouteOfAdministration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'firstDose': const FhirField(false, 'Quantity'),
    'maxSingleDose': const FhirField(false, 'Quantity'),
    'maxDosePerDay': const FhirField(false, 'Quantity'),
    'maxDosePerTreatmentPeriod': const FhirField(false, 'Ratio'),
    'maxTreatmentPeriod': const FhirField(false, 'FhirDuration'),
    'targetSpecies':
        const FhirField(true, 'AdministrableProductDefinitionTargetSpecies'),
  },
  'AdministrableProductDefinitionTargetSpecies': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'withdrawalPeriod':
        const FhirField(true, 'AdministrableProductDefinitionWithdrawalPeriod'),
  },
  'AdministrableProductDefinitionWithdrawalPeriod': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'tissue': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
    'supportingInformation': const FhirField(false, 'String'),
    '_supportingInformation': const FhirField(false, 'Element'),
  },
  'Ingredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'for': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(true, 'CodeableConcept'),
    'group': const FhirField(false, 'CodeableConcept'),
    'allergenicIndicator': const FhirField(false, 'Boolean'),
    '_allergenicIndicator': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(true, 'IngredientManufacturer'),
    'substance': const FhirField(false, 'IngredientSubstance'),
  },
  'IngredientManufacturer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'Code'),
    '_role': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'Reference'),
  },
  'IngredientSubstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableReference'),
    'strength': const FhirField(true, 'IngredientStrength'),
  },
  'IngredientStrength': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'presentationRatio': const FhirField(false, 'Ratio'),
    'presentationRatioRange': const FhirField(false, 'RatioRange'),
    'textPresentation': const FhirField(false, 'String'),
    '_textPresentation': const FhirField(false, 'Element'),
    'concentrationRatio': const FhirField(false, 'Ratio'),
    'concentrationRatioRange': const FhirField(false, 'RatioRange'),
    'textConcentration': const FhirField(false, 'String'),
    '_textConcentration': const FhirField(false, 'Element'),
    'measurementPoint': const FhirField(false, 'String'),
    '_measurementPoint': const FhirField(false, 'Element'),
    'country': const FhirField(true, 'CodeableConcept'),
    'referenceStrength': const FhirField(true, 'IngredientReferenceStrength'),
  },
  'IngredientReferenceStrength': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableReference'),
    'strengthRatio': const FhirField(false, 'Ratio'),
    'strengthRatioRange': const FhirField(false, 'RatioRange'),
    'strengthQuantity': const FhirField(false, 'Quantity'),
    'measurementPoint': const FhirField(false, 'String'),
    '_measurementPoint': const FhirField(false, 'Element'),
    'country': const FhirField(true, 'CodeableConcept'),
  },
  'ClinicalUseDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'CodeableConcept'),
    'contraindication':
        const FhirField(false, 'ClinicalUseDefinitionContraindication'),
    'indication': const FhirField(false, 'ClinicalUseDefinitionIndication'),
    'interaction': const FhirField(false, 'ClinicalUseDefinitionInteraction'),
    'population': const FhirField(true, 'Reference'),
    'undesirableEffect':
        const FhirField(false, 'ClinicalUseDefinitionUndesirableEffect'),
    'warning': const FhirField(false, 'ClinicalUseDefinitionWarning'),
  },
  'ClinicalUseDefinitionContraindication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diseaseSymptomProcedure': const FhirField(false, 'CodeableReference'),
    'diseaseStatus': const FhirField(false, 'CodeableReference'),
    'comorbidity': const FhirField(true, 'CodeableReference'),
    'indication': const FhirField(true, 'Reference'),
    'otherTherapy': const FhirField(true, 'ClinicalUseDefinitionOtherTherapy'),
  },
  'ClinicalUseDefinitionOtherTherapy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationshipType': const FhirField(false, 'CodeableConcept'),
    'therapy': const FhirField(false, 'CodeableReference'),
  },
  'ClinicalUseDefinitionIndication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diseaseSymptomProcedure': const FhirField(false, 'CodeableReference'),
    'diseaseStatus': const FhirField(false, 'CodeableReference'),
    'comorbidity': const FhirField(true, 'CodeableReference'),
    'intendedEffect': const FhirField(false, 'CodeableReference'),
    'durationRange': const FhirField(false, 'Range'),
    'durationString': const FhirField(false, 'Markdown'),
    '_durationString': const FhirField(false, 'Element'),
    'undesirableEffect': const FhirField(true, 'Reference'),
    'otherTherapy': const FhirField(true, 'ClinicalUseDefinitionOtherTherapy'),
  },
  'ClinicalUseDefinitionInteraction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'interactant': const FhirField(true, 'ClinicalUseDefinitionInteractant'),
    'type': const FhirField(false, 'CodeableConcept'),
    'effect': const FhirField(false, 'CodeableReference'),
    'incidence': const FhirField(false, 'CodeableConcept'),
    'management': const FhirField(true, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionInteractant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionUndesirableEffect': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'symptomConditionEffect': const FhirField(false, 'CodeableReference'),
    'classification': const FhirField(false, 'CodeableConcept'),
    'frequencyOfOccurrence': const FhirField(false, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionWarning': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'ManufacturedItemDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'manufacturedDoseForm': const FhirField(false, 'CodeableConcept'),
    'unitOfPresentation': const FhirField(false, 'CodeableConcept'),
    'manufacturer': const FhirField(true, 'Reference'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'property': const FhirField(true, 'ManufacturedItemDefinitionProperty'),
  },
  'ManufacturedItemDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'MedicinalProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'domain': const FhirField(false, 'CodeableConcept'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    '_statusDate': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'combinedPharmaceuticalDoseForm': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(true, 'CodeableConcept'),
    'indication': const FhirField(false, 'Markdown'),
    '_indication': const FhirField(false, 'Element'),
    'legalStatusOfSupply': const FhirField(false, 'CodeableConcept'),
    'additionalMonitoringIndicator': const FhirField(false, 'CodeableConcept'),
    'specialMeasures': const FhirField(true, 'CodeableConcept'),
    'pediatricUseIndicator': const FhirField(false, 'CodeableConcept'),
    'classification': const FhirField(true, 'CodeableConcept'),
    'marketingStatus': const FhirField(true, 'MarketingStatus'),
    'packagedMedicinalProduct': const FhirField(true, 'CodeableConcept'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'impurity': const FhirField(true, 'CodeableReference'),
    'attachedDocument': const FhirField(true, 'Reference'),
    'masterFile': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'MedicinalProductDefinitionContact'),
    'clinicalTrial': const FhirField(true, 'Reference'),
    'code': const FhirField(true, 'Coding'),
    'name': const FhirField(true, 'MedicinalProductDefinitionName'),
    'crossReference':
        const FhirField(true, 'MedicinalProductDefinitionCrossReference'),
    'operation': const FhirField(true, 'MedicinalProductDefinitionOperation'),
    'characteristic':
        const FhirField(true, 'MedicinalProductDefinitionCharacteristic'),
  },
  'MedicinalProductDefinitionContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'contact': const FhirField(false, 'Reference'),
  },
  'MedicinalProductDefinitionName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productName': const FhirField(false, 'String'),
    '_productName': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'namePart': const FhirField(true, 'MedicinalProductDefinitionNamePart'),
    'countryLanguage':
        const FhirField(true, 'MedicinalProductDefinitionCountryLanguage'),
  },
  'MedicinalProductDefinitionNamePart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'part': const FhirField(false, 'String'),
    '_part': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionCountryLanguage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'country': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionCrossReference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'product': const FhirField(false, 'CodeableReference'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableReference'),
    'effectiveDate': const FhirField(false, 'Period'),
    'organization': const FhirField(true, 'Reference'),
    'confidentialityIndicator': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'PackagedProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'packageFor': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    '_statusDate': const FhirField(false, 'Element'),
    'containedItemQuantity': const FhirField(true, 'Quantity'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'legalStatusOfSupply':
        const FhirField(true, 'PackagedProductDefinitionLegalStatusOfSupply'),
    'marketingStatus': const FhirField(true, 'MarketingStatus'),
    'characteristic': const FhirField(true, 'CodeableConcept'),
    'copackagedIndicator': const FhirField(false, 'Boolean'),
    '_copackagedIndicator': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(true, 'Reference'),
    'package': const FhirField(false, 'PackagedProductDefinitionPackage'),
  },
  'PackagedProductDefinitionLegalStatusOfSupply': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
  },
  'PackagedProductDefinitionPackage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Integer'),
    '_quantity': const FhirField(false, 'Element'),
    'material': const FhirField(true, 'CodeableConcept'),
    'alternateMaterial': const FhirField(true, 'CodeableConcept'),
    'shelfLifeStorage':
        const FhirField(true, 'PackagedProductDefinitionShelfLifeStorage'),
    'manufacturer': const FhirField(true, 'Reference'),
    'property': const FhirField(true, 'PackagedProductDefinitionProperty'),
    'containedItem':
        const FhirField(true, 'PackagedProductDefinitionContainedItem'),
    'package': const FhirField(true, 'PackagedProductDefinitionPackage'),
  },
  'PackagedProductDefinitionShelfLifeStorage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'periodDuration': const FhirField(false, 'FhirDuration'),
    'periodString': const FhirField(false, 'String'),
    '_periodString': const FhirField(false, 'Element'),
    'specialPrecautionsForStorage': const FhirField(true, 'CodeableConcept'),
  },
  'PackagedProductDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'PackagedProductDefinitionContainedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'RegulatedAuthorization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'subject': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'region': const FhirField(true, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    '_statusDate': const FhirField(false, 'Element'),
    'validityPeriod': const FhirField(false, 'Period'),
    'indication': const FhirField(false, 'CodeableReference'),
    'intendedUse': const FhirField(false, 'CodeableConcept'),
    'basis': const FhirField(true, 'CodeableConcept'),
    'holder': const FhirField(false, 'Reference'),
    'regulator': const FhirField(false, 'Reference'),
    'case': const FhirField(false, 'RegulatedAuthorizationCase'),
  },
  'RegulatedAuthorizationCase': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'datePeriod': const FhirField(false, 'Period'),
    'dateDateTime': const FhirField(false, 'FhirDateTime'),
    '_dateDateTime': const FhirField(false, 'Element'),
    'application': const FhirField(true, 'RegulatedAuthorizationCase'),
  },
  'SubstanceDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'classification': const FhirField(true, 'CodeableConcept'),
    'domain': const FhirField(false, 'CodeableConcept'),
    'grade': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'informationSource': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'manufacturer': const FhirField(true, 'Reference'),
    'supplier': const FhirField(true, 'Reference'),
    'moiety': const FhirField(true, 'SubstanceDefinitionMoiety'),
    'property': const FhirField(true, 'SubstanceDefinitionProperty'),
    'molecularWeight':
        const FhirField(true, 'SubstanceDefinitionMolecularWeight'),
    'structure': const FhirField(false, 'SubstanceDefinitionStructure'),
    'code': const FhirField(true, 'SubstanceDefinitionCode'),
    'name': const FhirField(true, 'SubstanceDefinitionName'),
    'relationship': const FhirField(true, 'SubstanceDefinitionRelationship'),
    'sourceMaterial':
        const FhirField(false, 'SubstanceDefinitionSourceMaterial'),
  },
  'SubstanceDefinitionMoiety': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'stereochemistry': const FhirField(false, 'CodeableConcept'),
    'opticalActivity': const FhirField(false, 'CodeableConcept'),
    'molecularFormula': const FhirField(false, 'String'),
    '_molecularFormula': const FhirField(false, 'Element'),
    'amountQuantity': const FhirField(false, 'Quantity'),
    'amountString': const FhirField(false, 'Markdown'),
    '_amountString': const FhirField(false, 'Element'),
    'measurementType': const FhirField(false, 'CodeableConcept'),
  },
  'SubstanceDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'SubstanceDefinitionMolecularWeight': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'method': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'SubstanceDefinitionStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'stereochemistry': const FhirField(false, 'CodeableConcept'),
    'opticalActivity': const FhirField(false, 'CodeableConcept'),
    'molecularFormula': const FhirField(false, 'String'),
    '_molecularFormula': const FhirField(false, 'Element'),
    'molecularFormulaByMoiety': const FhirField(false, 'String'),
    '_molecularFormulaByMoiety': const FhirField(false, 'Element'),
    'molecularWeight':
        const FhirField(false, 'SubstanceDefinitionMolecularWeight'),
    'technique': const FhirField(true, 'CodeableConcept'),
    'sourceDocument': const FhirField(true, 'Reference'),
    'representation':
        const FhirField(true, 'SubstanceDefinitionRepresentation'),
  },
  'SubstanceDefinitionRepresentation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'representation': const FhirField(false, 'String'),
    '_representation': const FhirField(false, 'Element'),
    'format': const FhirField(false, 'CodeableConcept'),
    'document': const FhirField(false, 'Reference'),
  },
  'SubstanceDefinitionCode': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    '_statusDate': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    '_preferred': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'CodeableConcept'),
    'domain': const FhirField(true, 'CodeableConcept'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'synonym': const FhirField(true, 'SubstanceDefinitionName'),
    'translation': const FhirField(true, 'SubstanceDefinitionName'),
    'official': const FhirField(true, 'SubstanceDefinitionOfficial'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionOfficial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'authority': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
  },
  'SubstanceDefinitionRelationship': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substanceDefinitionReference': const FhirField(false, 'Reference'),
    'substanceDefinitionCodeableConcept':
        const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'isDefining': const FhirField(false, 'Boolean'),
    '_isDefining': const FhirField(false, 'Element'),
    'amountQuantity': const FhirField(false, 'Quantity'),
    'amountRatio': const FhirField(false, 'Ratio'),
    'amountString': const FhirField(false, 'Markdown'),
    '_amountString': const FhirField(false, 'Element'),
    'ratioHighLimitAmount': const FhirField(false, 'Ratio'),
    'comparator': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionSourceMaterial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'genus': const FhirField(false, 'CodeableConcept'),
    'species': const FhirField(false, 'CodeableConcept'),
    'part': const FhirField(false, 'CodeableConcept'),
    'countryOfOrigin': const FhirField(true, 'CodeableConcept'),
  },
  'ActivityDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    '_subjectCanonical': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'library': const FhirField(true, 'Canonical'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'code': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    '_intent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    '_doNotPerform': const FhirField(false, 'Element'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    '_timingDateTime': const FhirField(false, 'Element'),
    'timingAge': const FhirField(false, 'Age'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingRange': const FhirField(false, 'Range'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'location': const FhirField(false, 'Reference'),
    'participant': const FhirField(true, 'ActivityDefinitionParticipant'),
    'productReference': const FhirField(false, 'Reference'),
    'productCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'dosage': const FhirField(true, 'Dosage'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'specimenRequirement': const FhirField(true, 'Reference'),
    'observationRequirement': const FhirField(true, 'Reference'),
    'observationResultRequirement': const FhirField(true, 'Reference'),
    'transform': const FhirField(false, 'Canonical'),
    'dynamicValue': const FhirField(true, 'ActivityDefinitionDynamicValue'),
  },
  'ActivityDefinitionParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'ActivityDefinitionDynamicValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'DeviceDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'udiDeviceIdentifier':
        const FhirField(true, 'DeviceDefinitionUdiDeviceIdentifier'),
    'manufacturerString': const FhirField(false, 'String'),
    '_manufacturerString': const FhirField(false, 'Element'),
    'manufacturerReference': const FhirField(false, 'Reference'),
    'deviceName': const FhirField(true, 'DeviceDefinitionDeviceName'),
    'modelNumber': const FhirField(false, 'String'),
    '_modelNumber': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'specialization': const FhirField(true, 'DeviceDefinitionSpecialization'),
    'version': const FhirField(true, 'String'),
    '_version': const FhirField(true, 'Element'),
    'safety': const FhirField(true, 'CodeableConcept'),
    'shelfLifeStorage': const FhirField(true, 'ProductShelfLife'),
    'physicalCharacteristics': const FhirField(false, 'ProdCharacteristic'),
    'languageCode': const FhirField(true, 'CodeableConcept'),
    'capability': const FhirField(true, 'DeviceDefinitionCapability'),
    'property': const FhirField(true, 'DeviceDefinitionProperty'),
    'owner': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'onlineInformation': const FhirField(false, 'FhirUri'),
    '_onlineInformation': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'quantity': const FhirField(false, 'Quantity'),
    'parentDevice': const FhirField(false, 'Reference'),
    'material': const FhirField(true, 'DeviceDefinitionMaterial'),
  },
  'DeviceDefinitionUdiDeviceIdentifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'deviceIdentifier': const FhirField(false, 'String'),
    '_deviceIdentifier': const FhirField(false, 'Element'),
    'issuer': const FhirField(false, 'FhirUri'),
    '_issuer': const FhirField(false, 'Element'),
    'jurisdiction': const FhirField(false, 'FhirUri'),
    '_jurisdiction': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionDeviceName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionSpecialization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'systemType': const FhirField(false, 'String'),
    '_systemType': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionCapability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(true, 'CodeableConcept'),
  },
  'DeviceDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(true, 'Quantity'),
    'valueCode': const FhirField(true, 'CodeableConcept'),
  },
  'DeviceDefinitionMaterial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableConcept'),
    'alternate': const FhirField(false, 'Boolean'),
    '_alternate': const FhirField(false, 'Element'),
    'allergenicIndicator': const FhirField(false, 'Boolean'),
    '_allergenicIndicator': const FhirField(false, 'Element'),
  },
  'EventDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'trigger': const FhirField(true, 'TriggerDefinition'),
  },
  'ObservationDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'identifier': const FhirField(true, 'Identifier'),
    'permittedDataType': const FhirField(true, 'Code'),
    '_permittedDataType': const FhirField(true, 'Element'),
    'multipleResultsAllowed': const FhirField(false, 'Boolean'),
    '_multipleResultsAllowed': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'CodeableConcept'),
    'preferredReportName': const FhirField(false, 'String'),
    '_preferredReportName': const FhirField(false, 'Element'),
    'quantitativeDetails':
        const FhirField(false, 'ObservationDefinitionQuantitativeDetails'),
    'qualifiedInterval':
        const FhirField(true, 'ObservationDefinitionQualifiedInterval'),
    'validCodedValueSet': const FhirField(false, 'Reference'),
    'normalCodedValueSet': const FhirField(false, 'Reference'),
    'abnormalCodedValueSet': const FhirField(false, 'Reference'),
    'criticalCodedValueSet': const FhirField(false, 'Reference'),
  },
  'ObservationDefinitionQuantitativeDetails': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'customaryUnit': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'conversionFactor': const FhirField(false, 'Decimal'),
    '_conversionFactor': const FhirField(false, 'Element'),
    'decimalPrecision': const FhirField(false, 'Integer'),
    '_decimalPrecision': const FhirField(false, 'Element'),
  },
  'ObservationDefinitionQualifiedInterval': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'Code'),
    '_category': const FhirField(false, 'Element'),
    'range': const FhirField(false, 'Range'),
    'context': const FhirField(false, 'CodeableConcept'),
    'appliesTo': const FhirField(true, 'CodeableConcept'),
    'gender': const FhirField(false, 'Code'),
    '_gender': const FhirField(false, 'Element'),
    'age': const FhirField(false, 'Range'),
    'gestationalAge': const FhirField(false, 'Range'),
    'condition': const FhirField(false, 'String'),
    '_condition': const FhirField(false, 'Element'),
  },
  'PlanDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    '_subjectCanonical': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'library': const FhirField(true, 'Canonical'),
    'goal': const FhirField(true, 'PlanDefinitionGoal'),
    'action': const FhirField(true, 'PlanDefinitionAction'),
  },
  'PlanDefinitionGoal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'CodeableConcept'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'start': const FhirField(false, 'CodeableConcept'),
    'addresses': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'target': const FhirField(true, 'PlanDefinitionTarget'),
  },
  'PlanDefinitionTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'measure': const FhirField(false, 'CodeableConcept'),
    'detailQuantity': const FhirField(false, 'Quantity'),
    'detailRange': const FhirField(false, 'Range'),
    'detailCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'due': const FhirField(false, 'FhirDuration'),
  },
  'PlanDefinitionAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'prefix': const FhirField(false, 'String'),
    '_prefix': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'textEquivalent': const FhirField(false, 'String'),
    '_textEquivalent': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    '_priority': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'goalId': const FhirField(true, 'Id'),
    '_goalId': const FhirField(true, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    '_subjectCanonical': const FhirField(true, 'Element'),
    'trigger': const FhirField(true, 'TriggerDefinition'),
    'condition': const FhirField(true, 'PlanDefinitionCondition'),
    'input': const FhirField(true, 'DataRequirement'),
    'output': const FhirField(true, 'DataRequirement'),
    'relatedAction': const FhirField(true, 'PlanDefinitionRelatedAction'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    '_timingDateTime': const FhirField(false, 'Element'),
    'timingAge': const FhirField(false, 'Age'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'timingRange': const FhirField(false, 'Range'),
    'timingTiming': const FhirField(false, 'Timing'),
    'participant': const FhirField(true, 'PlanDefinitionParticipant'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupingBehavior': const FhirField(false, 'Code'),
    '_groupingBehavior': const FhirField(false, 'Element'),
    'selectionBehavior': const FhirField(false, 'Code'),
    '_selectionBehavior': const FhirField(false, 'Element'),
    'requiredBehavior': const FhirField(false, 'Code'),
    '_requiredBehavior': const FhirField(false, 'Element'),
    'precheckBehavior': const FhirField(false, 'Code'),
    '_precheckBehavior': const FhirField(false, 'Element'),
    'cardinalityBehavior': const FhirField(false, 'Code'),
    '_cardinalityBehavior': const FhirField(false, 'Element'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    '_definitionCanonical': const FhirField(false, 'Element'),
    'definitionUri': const FhirField(false, 'FhirUri'),
    '_definitionUri': const FhirField(false, 'Element'),
    'transform': const FhirField(false, 'Canonical'),
    'dynamicValue': const FhirField(true, 'PlanDefinitionDynamicValue'),
    'action': const FhirField(true, 'PlanDefinitionAction'),
  },
  'PlanDefinitionCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    '_kind': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'PlanDefinitionRelatedAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actionId': const FhirField(false, 'Id'),
    '_actionId': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'Code'),
    '_relationship': const FhirField(false, 'Element'),
    'offsetDuration': const FhirField(false, 'FhirDuration'),
    'offsetRange': const FhirField(false, 'Range'),
  },
  'PlanDefinitionParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'PlanDefinitionDynamicValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'Questionnaire': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectType': const FhirField(true, 'Code'),
    '_subjectType': const FhirField(true, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'code': const FhirField(true, 'Coding'),
    'item': const FhirField(true, 'QuestionnaireItem'),
  },
  'QuestionnaireItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    '_linkId': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'FhirUri'),
    '_definition': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'Coding'),
    'prefix': const FhirField(false, 'String'),
    '_prefix': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'enableWhen': const FhirField(true, 'QuestionnaireEnableWhen'),
    'enableBehavior': const FhirField(false, 'Code'),
    '_enableBehavior': const FhirField(false, 'Element'),
    'required': const FhirField(false, 'Boolean'),
    '_required': const FhirField(false, 'Element'),
    'repeats': const FhirField(false, 'Boolean'),
    '_repeats': const FhirField(false, 'Element'),
    'readOnly': const FhirField(false, 'Boolean'),
    '_readOnly': const FhirField(false, 'Element'),
    'maxLength': const FhirField(false, 'Integer'),
    '_maxLength': const FhirField(false, 'Element'),
    'answerValueSet': const FhirField(false, 'Canonical'),
    'answerOption': const FhirField(true, 'QuestionnaireAnswerOption'),
    'initial': const FhirField(true, 'QuestionnaireInitial'),
    'item': const FhirField(true, 'QuestionnaireItem'),
  },
  'QuestionnaireEnableWhen': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'question': const FhirField(false, 'String'),
    '_question': const FhirField(false, 'Element'),
    'operator': const FhirField(false, 'Code'),
    '_operator': const FhirField(false, 'Element'),
    'answerBoolean': const FhirField(false, 'Boolean'),
    '_answerBoolean': const FhirField(false, 'Element'),
    'answerDecimal': const FhirField(false, 'Decimal'),
    '_answerDecimal': const FhirField(false, 'Element'),
    'answerInteger': const FhirField(false, 'Integer'),
    '_answerInteger': const FhirField(false, 'Element'),
    'answerDate': const FhirField(false, 'Date'),
    '_answerDate': const FhirField(false, 'Element'),
    'answerDateTime': const FhirField(false, 'FhirDateTime'),
    '_answerDateTime': const FhirField(false, 'Element'),
    'answerTime': const FhirField(false, 'Time'),
    '_answerTime': const FhirField(false, 'Element'),
    'answerString': const FhirField(false, 'String'),
    '_answerString': const FhirField(false, 'Element'),
    'answerCoding': const FhirField(false, 'Coding'),
    'answerQuantity': const FhirField(false, 'Quantity'),
    'answerReference': const FhirField(false, 'Reference'),
  },
  'QuestionnaireAnswerOption': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueReference': const FhirField(false, 'Reference'),
    'initialSelected': const FhirField(false, 'Boolean'),
    '_initialSelected': const FhirField(false, 'Element'),
  },
  'QuestionnaireInitial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'SpecimenDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'typeCollected': const FhirField(false, 'CodeableConcept'),
    'patientPreparation': const FhirField(true, 'CodeableConcept'),
    'timeAspect': const FhirField(false, 'String'),
    '_timeAspect': const FhirField(false, 'Element'),
    'collection': const FhirField(true, 'CodeableConcept'),
    'typeTested': const FhirField(true, 'SpecimenDefinitionTypeTested'),
  },
  'SpecimenDefinitionTypeTested': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'isDerived': const FhirField(false, 'Boolean'),
    '_isDerived': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'preference': const FhirField(false, 'Code'),
    '_preference': const FhirField(false, 'Element'),
    'container': const FhirField(false, 'SpecimenDefinitionContainer'),
    'requirement': const FhirField(false, 'String'),
    '_requirement': const FhirField(false, 'Element'),
    'retentionTime': const FhirField(false, 'FhirDuration'),
    'rejectionCriterion': const FhirField(true, 'CodeableConcept'),
    'handling': const FhirField(true, 'SpecimenDefinitionHandling'),
  },
  'SpecimenDefinitionContainer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'material': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'cap': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'capacity': const FhirField(false, 'Quantity'),
    'minimumVolumeQuantity': const FhirField(false, 'Quantity'),
    'minimumVolumeString': const FhirField(false, 'String'),
    '_minimumVolumeString': const FhirField(false, 'Element'),
    'additive': const FhirField(true, 'SpecimenDefinitionAdditive'),
    'preparation': const FhirField(false, 'String'),
    '_preparation': const FhirField(false, 'Element'),
  },
  'SpecimenDefinitionAdditive': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'additiveCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'additiveReference': const FhirField(false, 'Reference'),
  },
  'SpecimenDefinitionHandling': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'temperatureQualifier': const FhirField(false, 'CodeableConcept'),
    'temperatureRange': const FhirField(false, 'Range'),
    'maxDuration': const FhirField(false, 'FhirDuration'),
    'instruction': const FhirField(false, 'String'),
    '_instruction': const FhirField(false, 'Element'),
  },
  'Measure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'library': const FhirField(true, 'Canonical'),
    'disclaimer': const FhirField(false, 'Markdown'),
    '_disclaimer': const FhirField(false, 'Element'),
    'scoring': const FhirField(false, 'CodeableConcept'),
    'compositeScoring': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'riskAdjustment': const FhirField(false, 'String'),
    '_riskAdjustment': const FhirField(false, 'Element'),
    'rateAggregation': const FhirField(false, 'String'),
    '_rateAggregation': const FhirField(false, 'Element'),
    'rationale': const FhirField(false, 'Markdown'),
    '_rationale': const FhirField(false, 'Element'),
    'clinicalRecommendationStatement': const FhirField(false, 'Markdown'),
    '_clinicalRecommendationStatement': const FhirField(false, 'Element'),
    'improvementNotation': const FhirField(false, 'CodeableConcept'),
    'definition': const FhirField(true, 'Markdown'),
    '_definition': const FhirField(true, 'Element'),
    'guidance': const FhirField(false, 'Markdown'),
    '_guidance': const FhirField(false, 'Element'),
    'group': const FhirField(true, 'MeasureGroup'),
    'supplementalData': const FhirField(true, 'MeasureSupplementalData'),
  },
  'MeasureGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'population': const FhirField(true, 'MeasurePopulation'),
    'stratifier': const FhirField(true, 'MeasureStratifier'),
  },
  'MeasurePopulation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
  },
  'MeasureStratifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
    'component': const FhirField(true, 'MeasureComponent'),
  },
  'MeasureComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
  },
  'MeasureSupplementalData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'usage': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
  },
  'MeasureReport': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'measure': const FhirField(false, 'Canonical'),
    'subject': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'reporter': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'improvementNotation': const FhirField(false, 'CodeableConcept'),
    'group': const FhirField(true, 'MeasureReportGroup'),
    'evaluatedResource': const FhirField(true, 'Reference'),
  },
  'MeasureReportGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'population': const FhirField(true, 'MeasureReportPopulation'),
    'measureScore': const FhirField(false, 'Quantity'),
    'stratifier': const FhirField(true, 'MeasureReportStratifier'),
  },
  'MeasureReportPopulation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'count': const FhirField(false, 'Integer'),
    '_count': const FhirField(false, 'Element'),
    'subjectResults': const FhirField(false, 'Reference'),
  },
  'MeasureReportStratifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(true, 'CodeableConcept'),
    'stratum': const FhirField(true, 'MeasureReportStratum'),
  },
  'MeasureReportStratum': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'CodeableConcept'),
    'component': const FhirField(true, 'MeasureReportComponent'),
    'population': const FhirField(true, 'MeasureReportPopulation1'),
    'measureScore': const FhirField(false, 'Quantity'),
  },
  'MeasureReportComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'CodeableConcept'),
  },
  'MeasureReportPopulation1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'count': const FhirField(false, 'Integer'),
    '_count': const FhirField(false, 'Element'),
    'subjectResults': const FhirField(false, 'Reference'),
  },
  'TestReport': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'testScript': const FhirField(false, 'Reference'),
    'result': const FhirField(false, 'Code'),
    '_result': const FhirField(false, 'Element'),
    'score': const FhirField(false, 'Decimal'),
    '_score': const FhirField(false, 'Element'),
    'tester': const FhirField(false, 'String'),
    '_tester': const FhirField(false, 'Element'),
    'issued': const FhirField(false, 'FhirDateTime'),
    '_issued': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'TestReportParticipant'),
    'setup': const FhirField(false, 'TestReportSetup'),
    'test': const FhirField(true, 'TestReportTest'),
    'teardown': const FhirField(false, 'TestReportTeardown'),
  },
  'TestReportParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    '_uri': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
  },
  'TestReportSetup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestReportAction'),
  },
  'TestReportAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
    'assert': const FhirField(false, 'TestReportAssert'),
  },
  'TestReportOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'result': const FhirField(false, 'Code'),
    '_result': const FhirField(false, 'Element'),
    'message': const FhirField(false, 'Markdown'),
    '_message': const FhirField(false, 'Element'),
    'detail': const FhirField(false, 'FhirUri'),
    '_detail': const FhirField(false, 'Element'),
  },
  'TestReportAssert': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'result': const FhirField(false, 'Code'),
    '_result': const FhirField(false, 'Element'),
    'message': const FhirField(false, 'Markdown'),
    '_message': const FhirField(false, 'Element'),
    'detail': const FhirField(false, 'String'),
    '_detail': const FhirField(false, 'Element'),
  },
  'TestReportTest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'action': const FhirField(true, 'TestReportAction1'),
  },
  'TestReportAction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
    'assert': const FhirField(false, 'TestReportAssert'),
  },
  'TestReportTeardown': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestReportAction2'),
  },
  'TestReportAction2': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
  },
  'TestScript': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'origin': const FhirField(true, 'TestScriptOrigin'),
    'destination': const FhirField(true, 'TestScriptDestination'),
    'metadata': const FhirField(false, 'TestScriptMetadata'),
    'fixture': const FhirField(true, 'TestScriptFixture'),
    'profile': const FhirField(true, 'Reference'),
    'variable': const FhirField(true, 'TestScriptVariable'),
    'setup': const FhirField(false, 'TestScriptSetup'),
    'test': const FhirField(true, 'TestScriptTest'),
    'teardown': const FhirField(false, 'TestScriptTeardown'),
  },
  'TestScriptOrigin': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'index': const FhirField(false, 'Integer'),
    '_index': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Coding'),
  },
  'TestScriptDestination': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'index': const FhirField(false, 'Integer'),
    '_index': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Coding'),
  },
  'TestScriptMetadata': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'link': const FhirField(true, 'TestScriptLink'),
    'capability': const FhirField(true, 'TestScriptCapability'),
  },
  'TestScriptLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
  },
  'TestScriptCapability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'required': const FhirField(false, 'Boolean'),
    '_required': const FhirField(false, 'Element'),
    'validated': const FhirField(false, 'Boolean'),
    '_validated': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'origin': const FhirField(true, 'Integer'),
    '_origin': const FhirField(true, 'Element'),
    'destination': const FhirField(false, 'Integer'),
    '_destination': const FhirField(false, 'Element'),
    'link': const FhirField(true, 'FhirUri'),
    '_link': const FhirField(true, 'Element'),
    'capabilities': const FhirField(false, 'Canonical'),
  },
  'TestScriptFixture': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'autocreate': const FhirField(false, 'Boolean'),
    '_autocreate': const FhirField(false, 'Element'),
    'autodelete': const FhirField(false, 'Boolean'),
    '_autodelete': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
  },
  'TestScriptVariable': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'defaultValue': const FhirField(false, 'String'),
    '_defaultValue': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'headerField': const FhirField(false, 'String'),
    '_headerField': const FhirField(false, 'Element'),
    'hint': const FhirField(false, 'String'),
    '_hint': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    '_sourceId': const FhirField(false, 'Element'),
  },
  'TestScriptSetup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestScriptAction'),
  },
  'TestScriptAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
    'assert': const FhirField(false, 'TestScriptAssert'),
  },
  'TestScriptOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'resource': const FhirField(false, 'Code'),
    '_resource': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    '_label': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'accept': const FhirField(false, 'Code'),
    '_accept': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    '_contentType': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Integer'),
    '_destination': const FhirField(false, 'Element'),
    'encodeRequestUrl': const FhirField(false, 'Boolean'),
    '_encodeRequestUrl': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'Code'),
    '_method': const FhirField(false, 'Element'),
    'origin': const FhirField(false, 'Integer'),
    '_origin': const FhirField(false, 'Element'),
    'params': const FhirField(false, 'String'),
    '_params': const FhirField(false, 'Element'),
    'requestHeader': const FhirField(true, 'TestScriptRequestHeader'),
    'requestId': const FhirField(false, 'Id'),
    '_requestId': const FhirField(false, 'Element'),
    'responseId': const FhirField(false, 'Id'),
    '_responseId': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    '_sourceId': const FhirField(false, 'Element'),
    'targetId': const FhirField(false, 'Id'),
    '_targetId': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'String'),
    '_url': const FhirField(false, 'Element'),
  },
  'TestScriptRequestHeader': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'field': const FhirField(false, 'String'),
    '_field': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
  },
  'TestScriptAssert': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'label': const FhirField(false, 'String'),
    '_label': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'direction': const FhirField(false, 'Code'),
    '_direction': const FhirField(false, 'Element'),
    'compareToSourceId': const FhirField(false, 'String'),
    '_compareToSourceId': const FhirField(false, 'Element'),
    'compareToSourceExpression': const FhirField(false, 'String'),
    '_compareToSourceExpression': const FhirField(false, 'Element'),
    'compareToSourcePath': const FhirField(false, 'String'),
    '_compareToSourcePath': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    '_contentType': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'headerField': const FhirField(false, 'String'),
    '_headerField': const FhirField(false, 'Element'),
    'minimumId': const FhirField(false, 'String'),
    '_minimumId': const FhirField(false, 'Element'),
    'navigationLinks': const FhirField(false, 'Boolean'),
    '_navigationLinks': const FhirField(false, 'Element'),
    'operator': const FhirField(false, 'Code'),
    '_operator': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'requestMethod': const FhirField(false, 'Code'),
    '_requestMethod': const FhirField(false, 'Element'),
    'requestURL': const FhirField(false, 'String'),
    '_requestURL': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Code'),
    '_resource': const FhirField(false, 'Element'),
    'response': const FhirField(false, 'Code'),
    '_response': const FhirField(false, 'Element'),
    'responseCode': const FhirField(false, 'String'),
    '_responseCode': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    '_sourceId': const FhirField(false, 'Element'),
    'validateProfileId': const FhirField(false, 'Id'),
    '_validateProfileId': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'warningOnly': const FhirField(false, 'Boolean'),
    '_warningOnly': const FhirField(false, 'Element'),
  },
  'TestScriptTest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'action': const FhirField(true, 'TestScriptAction1'),
  },
  'TestScriptAction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
    'assert': const FhirField(false, 'TestScriptAssert'),
  },
  'TestScriptTeardown': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestScriptAction2'),
  },
  'TestScriptAction2': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
  },
  'Citation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'summary': const FhirField(true, 'CitationSummary'),
    'classification': const FhirField(true, 'CitationClassification'),
    'note': const FhirField(true, 'Annotation'),
    'currentState': const FhirField(true, 'CodeableConcept'),
    'statusDate': const FhirField(true, 'CitationStatusDate'),
    'relatesTo': const FhirField(true, 'CitationRelatesTo'),
    'citedArtifact': const FhirField(false, 'CitationCitedArtifact'),
  },
  'CitationSummary': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'style': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    '_text': const FhirField(false, 'Element'),
  },
  'CitationClassification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classifier': const FhirField(true, 'CodeableConcept'),
  },
  'CitationStatusDate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'actual': const FhirField(false, 'Boolean'),
    '_actual': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'CitationRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationshipType': const FhirField(false, 'CodeableConcept'),
    'targetClassifier': const FhirField(true, 'CodeableConcept'),
    'targetUri': const FhirField(false, 'FhirUri'),
    '_targetUri': const FhirField(false, 'Element'),
    'targetIdentifier': const FhirField(false, 'Identifier'),
    'targetReference': const FhirField(false, 'Reference'),
    'targetAttachment': const FhirField(false, 'Attachment'),
  },
  'CitationCitedArtifact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'relatedIdentifier': const FhirField(true, 'Identifier'),
    'dateAccessed': const FhirField(false, 'FhirDateTime'),
    '_dateAccessed': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'CitationVersion'),
    'currentState': const FhirField(true, 'CodeableConcept'),
    'statusDate': const FhirField(true, 'CitationStatusDate1'),
    'title': const FhirField(true, 'CitationTitle'),
    'abstract': const FhirField(true, 'CitationAbstract'),
    'part': const FhirField(false, 'CitationPart'),
    'relatesTo': const FhirField(true, 'CitationRelatesTo1'),
    'publicationForm': const FhirField(true, 'CitationPublicationForm'),
    'webLocation': const FhirField(true, 'CitationWebLocation'),
    'classification': const FhirField(true, 'CitationClassification1'),
    'contributorship': const FhirField(false, 'CitationContributorship'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CitationVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'baseCitation': const FhirField(false, 'Reference'),
  },
  'CitationStatusDate1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'actual': const FhirField(false, 'Boolean'),
    '_actual': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'CitationTitle': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    '_text': const FhirField(false, 'Element'),
  },
  'CitationAbstract': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    '_text': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
  },
  'CitationPart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'baseCitation': const FhirField(false, 'Reference'),
  },
  'CitationRelatesTo1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationshipType': const FhirField(false, 'CodeableConcept'),
    'targetClassifier': const FhirField(true, 'CodeableConcept'),
    'targetUri': const FhirField(false, 'FhirUri'),
    '_targetUri': const FhirField(false, 'Element'),
    'targetIdentifier': const FhirField(false, 'Identifier'),
    'targetReference': const FhirField(false, 'Reference'),
    'targetAttachment': const FhirField(false, 'Attachment'),
  },
  'CitationPublicationForm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'publishedIn': const FhirField(false, 'CitationPublishedIn'),
    'periodicRelease': const FhirField(false, 'CitationPeriodicRelease'),
    'articleDate': const FhirField(false, 'FhirDateTime'),
    '_articleDate': const FhirField(false, 'Element'),
    'lastRevisionDate': const FhirField(false, 'FhirDateTime'),
    '_lastRevisionDate': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'CodeableConcept'),
    'accessionNumber': const FhirField(false, 'String'),
    '_accessionNumber': const FhirField(false, 'Element'),
    'pageString': const FhirField(false, 'String'),
    '_pageString': const FhirField(false, 'Element'),
    'firstPage': const FhirField(false, 'String'),
    '_firstPage': const FhirField(false, 'Element'),
    'lastPage': const FhirField(false, 'String'),
    '_lastPage': const FhirField(false, 'Element'),
    'pageCount': const FhirField(false, 'String'),
    '_pageCount': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
  },
  'CitationPublishedIn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'identifier': const FhirField(true, 'Identifier'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'Reference'),
    'publisherLocation': const FhirField(false, 'String'),
    '_publisherLocation': const FhirField(false, 'Element'),
  },
  'CitationPeriodicRelease': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'citedMedium': const FhirField(false, 'CodeableConcept'),
    'volume': const FhirField(false, 'String'),
    '_volume': const FhirField(false, 'Element'),
    'issue': const FhirField(false, 'String'),
    '_issue': const FhirField(false, 'Element'),
    'dateOfPublication': const FhirField(false, 'CitationDateOfPublication'),
  },
  'CitationDateOfPublication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'Date'),
    '_date': const FhirField(false, 'Element'),
    'year': const FhirField(false, 'String'),
    '_year': const FhirField(false, 'Element'),
    'month': const FhirField(false, 'String'),
    '_month': const FhirField(false, 'Element'),
    'day': const FhirField(false, 'String'),
    '_day': const FhirField(false, 'Element'),
    'season': const FhirField(false, 'String'),
    '_season': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
  },
  'CitationWebLocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
  },
  'CitationClassification1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'whoClassified': const FhirField(false, 'CitationWhoClassified'),
    'artifactAssessment': const FhirField(true, 'Reference'),
  },
  'CitationWhoClassified': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'person': const FhirField(false, 'Reference'),
    'organization': const FhirField(false, 'Reference'),
    'publisher': const FhirField(false, 'Reference'),
    'classifierCopyright': const FhirField(false, 'String'),
    '_classifierCopyright': const FhirField(false, 'Element'),
    'freeToShare': const FhirField(false, 'Boolean'),
    '_freeToShare': const FhirField(false, 'Element'),
  },
  'CitationContributorship': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'complete': const FhirField(false, 'Boolean'),
    '_complete': const FhirField(false, 'Element'),
    'entry': const FhirField(true, 'CitationEntry'),
    'summary': const FhirField(true, 'CitationSummary1'),
  },
  'CitationEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'HumanName'),
    'initials': const FhirField(false, 'String'),
    '_initials': const FhirField(false, 'Element'),
    'collectiveName': const FhirField(false, 'String'),
    '_collectiveName': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'affiliationInfo': const FhirField(true, 'CitationAffiliationInfo'),
    'address': const FhirField(true, 'Address'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'contributionType': const FhirField(true, 'CodeableConcept'),
    'role': const FhirField(false, 'CodeableConcept'),
    'contributionInstance':
        const FhirField(true, 'CitationContributionInstance'),
    'correspondingContact': const FhirField(false, 'Boolean'),
    '_correspondingContact': const FhirField(false, 'Element'),
    'listOrder': const FhirField(false, 'PositiveInt'),
    '_listOrder': const FhirField(false, 'Element'),
  },
  'CitationAffiliationInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'affiliation': const FhirField(false, 'String'),
    '_affiliation': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'String'),
    '_role': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
  },
  'CitationContributionInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'time': const FhirField(false, 'FhirDateTime'),
    '_time': const FhirField(false, 'Element'),
  },
  'CitationSummary1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'style': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Markdown'),
    '_value': const FhirField(false, 'Element'),
  },
  'Evidence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'citeAsReference': const FhirField(false, 'Reference'),
    'citAsMarkdown': const FhirField(false, 'Markdown'),
    '_citeAsMarkdown': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'assertion': const FhirField(false, 'Markdown'),
    '_assertion': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'variableDefinition': const FhirField(true, 'EvidenceVariableDefinition'),
    'synthesisType': const FhirField(false, 'CodeableConcept'),
    'studyType': const FhirField(false, 'CodeableConcept'),
    'statistic': const FhirField(true, 'EvidenceStatistic'),
    'certainty': const FhirField(true, 'EvidenceCertainty'),
  },
  'EvidenceVariableDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'variableRole': const FhirField(false, 'CodeableConcept'),
    'observed': const FhirField(false, 'Reference'),
    'intended': const FhirField(false, 'Reference'),
    'directnessMatch': const FhirField(false, 'CodeableConcept'),
  },
  'EvidenceStatistic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'statisticType': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'numberOfEvents': const FhirField(false, 'UnsignedInt'),
    '_numberOfEvents': const FhirField(false, 'Element'),
    'numberAffected': const FhirField(false, 'UnsignedInt'),
    '_numberAffected': const FhirField(false, 'Element'),
    'sampleSize': const FhirField(false, 'EvidenceSampleSize'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
    'modelCharacteristic': const FhirField(true, 'EvidenceModelCharacteristic'),
  },
  'EvidenceSampleSize': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'numberOfStudies': const FhirField(false, 'UnsignedInt'),
    '_numberOfStudies': const FhirField(false, 'Element'),
    'numberOfParticipants': const FhirField(false, 'UnsignedInt'),
    '_numberOfParticipants': const FhirField(false, 'Element'),
    'knownDataCount': const FhirField(false, 'UnsignedInt'),
    '_knownDataCount': const FhirField(false, 'Element'),
  },
  'EvidenceAttributeEstimate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'level': const FhirField(false, 'Decimal'),
    '_level': const FhirField(false, 'Element'),
    'range': const FhirField(false, 'Range'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
  },
  'EvidenceModelCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
    'variable': const FhirField(true, 'EvidenceVar'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
  },
  'EvidenceVar': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'variableDefinition': const FhirField(false, 'Reference'),
    'handling': const FhirField(false, 'Code'),
    '_handling': const FhirField(false, 'Element'),
    'valueCategory': const FhirField(true, 'CodeableConcept'),
    'valueQuantity': const FhirField(true, 'Quantity'),
    'valueRange': const FhirField(true, 'Range'),
  },
  'EvidenceCertainty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'type': const FhirField(false, 'CodeableConcept'),
    'rating': const FhirField(false, 'CodeableConcept'),
    'rater': const FhirField(false, 'String'),
    '_rater': const FhirField(false, 'Element'),
    'subcomponent': const FhirField(true, 'EvidenceCertainty'),
  },
  'EvidenceReport': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'identifier': const FhirField(true, 'Identifier'),
    'relatedIdentifier': const FhirField(true, 'Identifier'),
    'citeAsReference': const FhirField(false, 'Reference'),
    'citeAsMarkdown': const FhirField(false, 'Markdown'),
    '_citeAsMarkdown': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subject': const FhirField(false, 'EvidenceReportSubject'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatesTo': const FhirField(true, 'EvidenceReportRelatesTo'),
    'section': const FhirField(true, 'EvidenceReportSection'),
  },
  'EvidenceReportSubject': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'characteristic': const FhirField(true, 'EvidenceReportCharacteristic'),
    'note': const FhirField(true, 'Annotation'),
  },
  'EvidenceReportCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'exclude': const FhirField(false, 'Boolean'),
    '_exclude': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'EvidenceReportRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'targetIdentifier': const FhirField(false, 'Identifier'),
    'targetReference': const FhirField(false, 'Reference'),
  },
  'EvidenceReportSection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'CodeableConcept'),
    'focusReference': const FhirField(false, 'Reference'),
    'author': const FhirField(true, 'Reference'),
    'text': const FhirField(false, 'Narrative'),
    'mode': const FhirField(false, 'Code'),
    '_mode': const FhirField(false, 'Element'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'entryClassifier': const FhirField(true, 'CodeableConcept'),
    'entryReference': const FhirField(true, 'Reference'),
    'entryQuantity': const FhirField(true, 'Quantity'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
    'section': const FhirField(true, 'EvidenceReportSection'),
  },
  'EvidenceVariable': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'shortTitle': const FhirField(false, 'String'),
    '_shortTitle': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'useContext': const FhirField(true, 'UsageContext'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'actual': const FhirField(false, 'Boolean'),
    '_actual': const FhirField(false, 'Element'),
    'characteristicCombination': const FhirField(false, 'Code'),
    '_characteristicCombination': const FhirField(false, 'Element'),
    'characteristic': const FhirField(true, 'EvidenceVariableCharacteristic'),
    'handling': const FhirField(false, 'Code'),
    '_handling': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'EvidenceVariableCategory'),
  },
  'EvidenceVariableCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'definitionReference': const FhirField(false, 'Reference'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    '_definitionCanonical': const FhirField(false, 'Element'),
    'definitionCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'definitionExpression': const FhirField(false, 'Expression'),
    'exclude': const FhirField(false, 'Boolean'),
    '_exclude': const FhirField(false, 'Element'),
    'timeFromStart': const FhirField(false, 'EvidenceVariableTimeFromStart'),
    'groupMeasure': const FhirField(false, 'Code'),
    '_groupMeasure': const FhirField(false, 'Element'),
  },
  'EvidenceVariableTimeFromStart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'range': const FhirField(false, 'Range'),
    'note': const FhirField(true, 'Annotation'),
  },
  'EvidenceVariableCategory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
  },
  'ResearchDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'shortTitle': const FhirField(false, 'String'),
    '_shortTitle': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'comment': const FhirField(true, 'String'),
    '_comment': const FhirField(true, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'library': const FhirField(true, 'Canonical'),
    'population': const FhirField(false, 'Reference'),
    'exposure': const FhirField(false, 'Reference'),
    'exposureAlternative': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Reference'),
  },
  'ResearchElementDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    '_implicitRules': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'Resource'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'shortTitle': const FhirField(false, 'String'),
    '_shortTitle': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    '_subtitle': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    '_status': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    '_experimental': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    '_date': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    '_publisher': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    '_description': const FhirField(false, 'Element'),
    'comment': const FhirField(true, 'String'),
    '_comment': const FhirField(true, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    '_purpose': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'String'),
    '_usage': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    '_copyright': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    '_approvalDate': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    '_lastReviewDate': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'library': const FhirField(true, 'Canonical'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'variableType': const FhirField(false, 'Code'),
    '_variableType': const FhirField(false, 'Element'),
    'characteristic':
        const FhirField(true, 'ResearchElementDefinitionCharacteristic'),
  },
  'ResearchElementDefinitionCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'definitionCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    '_definitionCanonical': const FhirField(false, 'Element'),
    'definitionExpression': const FhirField(false, 'Expression'),
    'definitionDataRequirement': const FhirField(false, 'DataRequirement'),
    'usageContext': const FhirField(true, 'UsageContext'),
    'exclude': const FhirField(false, 'Boolean'),
    '_exclude': const FhirField(false, 'Element'),
    'unitOfMeasure': const FhirField(false, 'CodeableConcept'),
    'studyEffectiveDescription': const FhirField(false, 'String'),
    '_studyEffectiveDescription': const FhirField(false, 'Element'),
    'studyEffectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_studyEffectiveDateTime': const FhirField(false, 'Element'),
    'studyEffectivePeriod': const FhirField(false, 'Period'),
    'studyEffectiveDuration': const FhirField(false, 'FhirDuration'),
    'studyEffectiveTiming': const FhirField(false, 'Timing'),
    'studyEffectiveTimeFromStart': const FhirField(false, 'FhirDuration'),
    'studyEffectiveGroupMeasure': const FhirField(false, 'Code'),
    '_studyEffectiveGroupMeasure': const FhirField(false, 'Element'),
    'participantEffectiveDescription': const FhirField(false, 'String'),
    '_participantEffectiveDescription': const FhirField(false, 'Element'),
    'participantEffectiveDateTime': const FhirField(false, 'FhirDateTime'),
    '_participantEffectiveDateTime': const FhirField(false, 'Element'),
    'participantEffectivePeriod': const FhirField(false, 'Period'),
    'participantEffectiveDuration': const FhirField(false, 'FhirDuration'),
    'participantEffectiveTiming': const FhirField(false, 'Timing'),
    'participantEffectiveTimeFromStart': const FhirField(false, 'FhirDuration'),
    'participantEffectiveGroupMeasure': const FhirField(false, 'Code'),
    '_participantEffectiveGroupMeasure': const FhirField(false, 'Element'),
  },
  'FhirExtension': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    '_url': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    '_valueBase64Binary': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    '_valueBoolean': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    '_valueCanonical': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    '_valueCode': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    '_valueDate': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    '_valueDecimal': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    '_valueId': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    '_valueInstant': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    '_valueInteger': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    '_valueMarkdown': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Oid'),
    '_valueOid': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    '_valuePositiveInt': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    '_valueString': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    '_valueTime': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    '_valueUnsignedInt': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    '_valueUri': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    '_valueUrl': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Uuid'),
    '_valueUuid': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueContributor': const FhirField(false, 'Contributor'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueDosage': const FhirField(false, 'Dosage'),
  },
  'Element': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
  },
  'Annotation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'authorReference': const FhirField(false, 'Reference'),
    'authorString': const FhirField(false, 'String'),
    '_authorString': const FhirField(false, 'Element'),
    'time': const FhirField(false, 'FhirDateTime'),
    '_time': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Markdown'),
    '_text': const FhirField(false, 'Element'),
  },
  'Attachment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'contentType': const FhirField(false, 'Code'),
    '_contentType': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    '_language': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'Base64Binary'),
    '_data': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    '_url': const FhirField(false, 'Element'),
    'size': const FhirField(false, 'UnsignedInt'),
    '_size': const FhirField(false, 'Element'),
    'hash': const FhirField(false, 'Base64Binary'),
    '_hash': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    '_title': const FhirField(false, 'Element'),
    'creation': const FhirField(false, 'FhirDateTime'),
    '_creation': const FhirField(false, 'Element'),
  },
  'Identifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'String'),
    '_use': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'assigner': const FhirField(false, 'Reference'),
  },
  'CodeableConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'coding': const FhirField(true, 'Coding'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
  },
  'Coding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    '_version': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'userSelected': const FhirField(false, 'Boolean'),
    '_userSelected': const FhirField(false, 'Element'),
  },
  'Quantity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'String'),
    '_comparator': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    '_unit': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
  },
  'FhirDuration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'String'),
    '_comparator': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    '_unit': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
  },
  'Distance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'String'),
    '_comparator': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    '_unit': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
  },
  'Count': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'String'),
    '_comparator': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    '_unit': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
  },
  'Money': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'currency': const FhirField(false, 'Code'),
    '_currency': const FhirField(false, 'Element'),
  },
  'Age': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    '_value': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'String'),
    '_comparator': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    '_unit': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    '_system': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    '_code': const FhirField(false, 'Element'),
  },
  'Range': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'low': const FhirField(false, 'Quantity'),
    'high': const FhirField(false, 'Quantity'),
  },
  'Period': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'FhirDateTime'),
    '_start': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'FhirDateTime'),
    '_end': const FhirField(false, 'Element'),
  },
  'Ratio': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'numerator': const FhirField(false, 'Quantity'),
    'denominator': const FhirField(false, 'Quantity'),
  },
  'RatioRange': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'lowNumerator': const FhirField(false, 'Quantity'),
    'highNumerator': const FhirField(false, 'Quantity'),
    'denominator': const FhirField(false, 'Quantity'),
  },
  'SampledData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'origin': const FhirField(false, 'Quantity'),
    'period': const FhirField(false, 'Decimal'),
    '_period': const FhirField(false, 'Element'),
    'factor': const FhirField(false, 'Decimal'),
    '_factor': const FhirField(false, 'Element'),
    'lowerLimit': const FhirField(false, 'Decimal'),
    '_lowerLimit': const FhirField(false, 'Element'),
    'upperLimit': const FhirField(false, 'Decimal'),
    '_upperLimit': const FhirField(false, 'Element'),
    'dimensions': const FhirField(false, 'PositiveInt'),
    '_dimensions': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'String'),
    '_data': const FhirField(false, 'Element'),
  },
  'Signature': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'Coding'),
    'when': const FhirField(false, 'Instant'),
    '_when': const FhirField(false, 'Element'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'targetFormat': const FhirField(false, 'Code'),
    '_targetFormat': const FhirField(false, 'Element'),
    'sigFormat': const FhirField(false, 'Code'),
    '_sigFormat': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'Base64Binary'),
    '_data': const FhirField(false, 'Element'),
  },
  'HumanName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'String'),
    '_use': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'family': const FhirField(false, 'String'),
    '_family': const FhirField(false, 'Element'),
    'given': const FhirField(true, 'String'),
    '_given': const FhirField(true, 'Element'),
    'prefix': const FhirField(true, 'String'),
    '_prefix': const FhirField(true, 'Element'),
    'suffix': const FhirField(true, 'String'),
    '_suffix': const FhirField(true, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'Address': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'String'),
    '_use': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    '_text': const FhirField(false, 'Element'),
    'line': const FhirField(true, 'String'),
    '_line': const FhirField(true, 'Element'),
    'city': const FhirField(false, 'String'),
    '_city': const FhirField(false, 'Element'),
    'district': const FhirField(false, 'String'),
    '_district': const FhirField(false, 'Element'),
    'state': const FhirField(false, 'String'),
    '_state': const FhirField(false, 'Element'),
    'postalCode': const FhirField(false, 'String'),
    '_postalCode': const FhirField(false, 'Element'),
    'country': const FhirField(false, 'String'),
    '_country': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'ContactPoint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'String'),
    '_system': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    '_value': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'String'),
    '_use': const FhirField(false, 'Element'),
    'rank': const FhirField(false, 'PositiveInt'),
    '_rank': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'Timing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'event': const FhirField(true, 'FhirDateTime'),
    '_event': const FhirField(true, 'Element'),
    'repeat': const FhirField(false, 'TimingRepeat'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'TimingRepeat': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'boundsDuration': const FhirField(false, 'FhirDuration'),
    'boundsRange': const FhirField(false, 'Range'),
    'boundsPeriod': const FhirField(false, 'Period'),
    'count': const FhirField(false, 'PositiveInt'),
    '_count': const FhirField(false, 'Element'),
    'countMax': const FhirField(false, 'PositiveInt'),
    '_countMax': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Decimal'),
    '_duration': const FhirField(false, 'Element'),
    'durationMax': const FhirField(false, 'Decimal'),
    '_durationMax': const FhirField(false, 'Element'),
    'durationUnit': const FhirField(false, 'String'),
    '_durationUnit': const FhirField(false, 'Element'),
    'frequency': const FhirField(false, 'PositiveInt'),
    '_frequency': const FhirField(false, 'Element'),
    'frequencyMax': const FhirField(false, 'PositiveInt'),
    '_frequencyMax': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Decimal'),
    '_period': const FhirField(false, 'Element'),
    'periodMax': const FhirField(false, 'Decimal'),
    '_periodMax': const FhirField(false, 'Element'),
    'periodUnit': const FhirField(false, 'String'),
    '_periodUnit': const FhirField(false, 'Element'),
    'dayOfWeek': const FhirField(true, 'Code'),
    '_dayOfWeek': const FhirField(true, 'Element'),
    'timeOfDay': const FhirField(true, 'Time'),
    '_timeOfDay': const FhirField(true, 'Element'),
    'when': const FhirField(true, 'String'),
    '_when': const FhirField(true, 'Element'),
    'offset': const FhirField(false, 'UnsignedInt'),
    '_offset': const FhirField(false, 'Element'),
  },
  'ContactDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'telecom': const FhirField(true, 'ContactPoint'),
  },
  'Contributor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
  },
  'DataRequirement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'mustSupport': const FhirField(true, 'String'),
    '_mustSupport': const FhirField(true, 'Element'),
    'codeFilter': const FhirField(true, 'DataRequirementCodeFilter'),
    'dateFilter': const FhirField(true, 'DataRequirementDateFilter'),
    'limit': const FhirField(false, 'PositiveInt'),
    '_limit': const FhirField(false, 'Element'),
    'sort': const FhirField(true, 'DataRequirementSort'),
  },
  'DataRequirementCodeFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'searchParam': const FhirField(false, 'String'),
    '_searchParam': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'code': const FhirField(true, 'Coding'),
  },
  'DataRequirementDateFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'searchParam': const FhirField(false, 'String'),
    '_searchParam': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    '_valueDateTime': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
  },
  'DataRequirementSort': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    '_path': const FhirField(false, 'Element'),
    'direction': const FhirField(false, 'String'),
    '_direction': const FhirField(false, 'Element'),
  },
  'ParameterDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    '_name': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Code'),
    '_use': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    '_min': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    '_max': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    '_documentation': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    '_type': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'RelatedArtifact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    '_label': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    '_display': const FhirField(false, 'Element'),
    'citation': const FhirField(false, 'Markdown'),
    '_citation': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    '_url': const FhirField(false, 'Element'),
    'document': const FhirField(false, 'Attachment'),
    'resource': const FhirField(false, 'Canonical'),
  },
  'TriggerDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'String'),
    '_type': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    '_name': const FhirField(false, 'Element'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingReference': const FhirField(false, 'Reference'),
    'timingDate': const FhirField(false, 'Date'),
    '_timingDate': const FhirField(false, 'Element'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    '_timingDateTime': const FhirField(false, 'Element'),
    'data': const FhirField(true, 'DataRequirement'),
    'condition': const FhirField(false, 'Expression'),
  },
  'UsageContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Coding'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'Expression': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    '_description': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'Id'),
    '_name': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'String'),
    '_language': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    '_expression': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'FhirUri'),
    '_reference': const FhirField(false, 'Element'),
  },
};
